#ifndef FORTRAN_C_INCLUDED
#define FORTRAN_C_INCLUDED

/* Fortran-C-Schnittstelle	
 *
 * Hinweise:  Der Typ FTN_INT muss dem default Integer Type in Fortran
 *            entsprechen. In der Regel ist das "int"
 *
 * Geaendert: G.Eymann, 06.02.03: Prototyp f. cinquire fehlte
 *                                C++ kompatibel
 *                                Prototyp f. spumb entfernt
 *            H.Weber, Mai 03:    _EXTNAME fuer ECMWF
 *                     Juni 04:   neue Funktion cclear
 *          C.Gramalla, Aug. 05:  neue Funktion dwdget_ffname
 *          C.Gramalla, Jan. 06:  neue Funktion cfindnext
 *          C.Gramalla, Feb. 06:  neue Funktion ccuegin,ccuegex - Aufruf nur aus C
 */
#define FTN_INT int

#ifdef __nec
#   define  FORTRAN_UNDERLINE 
#   undef   FORTRAN_UPPERCASE
#endif
#ifdef __sgi
#   undef   FORTRAN_UPPERCASE
#   define  FORTRAN_UNDERLINE 
#endif
#ifdef __uxp__
#   undef   FORTRAN_UPPERCASE
#   define  FORTRAN_UNDERLINE 
#endif
#ifdef __linux__
#   undef   FORTRAN_UPPERCASE
#   define  FORTRAN_UNDERLINE 
#endif
#ifdef _CRAY
#   undef   FORTRAN_UNDERLINE
#   define  FORTRAN_UPPERCASE
#endif
#ifdef _AIX
#   undef   FORTRAN_UNDERLINE
#   undef   FORTRAN_UPPERCASE
#endif
#ifdef _EXTNAME
#   define  FORTRAN_UNDERLINE
#endif
#ifdef _WIN32
#   undef   FORTRAN_UNDERLINE
#   define  FORTRAN_UPPERCASE
#endif

/* 
 * system dependencies 
 * Die Namen aller Aufrufe dieser Funktionen muessen system-abhaengig
 * umdefiniert werden, sowohl bei Aufruf von Fortran als auch von C!
 * Daher muss diese Datei unbedingt in der C Quelle inkludiert werden!
 */

#ifdef FORTRAN_UNDERLINE
#   define  fsleep    fsleep_
#   define  cback     cback_
#   define  cclose    cclose_
#   define  closeall  closeall_
#   define  cinquire  cinquire_
#   define  copen     copen_
#   define  crewind   crewind_
#   define  cclear    cclear_
#   define  cuegex    cuegex_
#   define  cuegin    cuegin_
#   define  cggid     cggid_
#   define  dwdget_ffname     dwdget_ffname_
#   define  cfindnext    cfindnext_
#endif
#ifdef FORTRAN_UPPERCASE
#   define  fsleep    FSLEEP
#   define  cback     CBACK
#   define  cclose    CCLOSE
#   define  closeall  CLOSEALL
#   define  cinquire  CINQUIRE
#   define  copen     COPEN
#   define  crewind   CREWIND
#   define  cclear    CCLEAR
#   define  cuegex    CUEGEX
#   define  cuegin    CUEGIN
#   define  cggid     CGGID
#   define  dwdget_ffname     DWDGET_FFNAME
#   define  cfindnext    CFINDNEXT
#endif

/*
 * ANSI-C forward declaration of C functions (prototypes):
 * Wird diese Datei in den C Quellen (Deklaration und ggfls. Aufruf)
 * inkludiert, kann der Compiler den korrekten Typ bzw. Rueckgabewert
 * sowie alle Argumente pruefen.
 */

#ifdef __cplusplus
extern "C" {
#endif	 

int fsleep (FTN_INT *secnds);
int cback (FTN_INT*, FTN_INT*);
int crewind (FTN_INT*, FTN_INT*);
int cclear (FTN_INT*, FTN_INT*);
int cuegex (FTN_INT*, char*, FTN_INT*, FTN_INT*);
int cuegin (FTN_INT*, FTN_INT*, char*, FTN_INT*, FTN_INT*);
int cinquire (FTN_INT*, char*, FTN_INT*, FTN_INT*, FTN_INT*, unsigned long);
char cget_datart (FTN_INT*);
FILE* cget_fdes( FTN_INT *iunit );
void cggid (FTN_INT*);
void dwdget_ffname (char*, char*, char*, FTN_INT*,
                  unsigned long, unsigned long, unsigned long);
int cfindnext (FTN_INT*, FTN_INT*, FTN_INT*, FTN_INT*, FTN_INT*);

extern int cclose (FTN_INT*, char*, FTN_INT*, unsigned long);
extern int cinquire (FTN_INT*, char*, FTN_INT*, FTN_INT*, FTN_INT*, 
                     unsigned long);
extern int copen (FTN_INT*, char*, char*, FTN_INT*, unsigned long, 
                  unsigned long);

/* internally used routines: */

int write_control_word(int, int, int);
void closeall(void);

/* weitere C Funktionen der DWDLIB, Aufruf nur von C, nicht Fortran! */

int rts_ed1 (FILE* fp_in, int iun, int *ihits, int *irm);
int cdaten_vonpb (char in[], char out[], int *iednr, int *ihits, int *irm);

int ccuegin (FTN_INT*, FTN_INT*, char*, FTN_INT*, int, FTN_INT*);
int ccuegex (FTN_INT*, char*, FTN_INT*, int, FTN_INT*);

#ifdef __cplusplus
}
#endif

#endif /* FORTRAN_C_INCLUDED */
