      SUBROUTINE ATMOS (YUPABS,IRM,YATMOD,EIN,AUS)
C**** ATMOS    -   STANDARD-ATMOSPHAEREN-AUSKUNFT JE NACH ENTRY-NAMEN
C**   AUFRUF       CALL ATMOS (YUPNAM,IRM,YATMOD,EIN,AUS)
C**   ZWECK        STANDARD-ATMOSPHAEREN-AUSKUNFT JE NACH ENTRY-NAMEN
C**                UND GEWUENSCHTER STANDARD-ATMOSPHAERE ERFOLGT DIE
C**                ZUORDNUNG EINES AUSGABE-ELEMENTES ZU EINEM VORGEB-
C**                BAREN EINGABE-ELEMENT.
C**   EXTERNAL     HERMES,UPSTYX,LISTB,PHYKON
C**   PARAMETER    ATMOS  ENTRY-NAME  EINGABE          AUSGABE
C**                       ----------  -------          -------
C**                       ATMOS       KEINE            DEFINITIONS-LISTE
C**                                                    AUF OUTPUT/PROTOK
C**                                                    (S. UP PROTOP)
C**                       ATMHP       HOEHE            DRUCK
C**                       ATMHT       .....            TEMPERATUR
C**                       ATMHG       .....            VERTIK.GRADIENT
C**                       ATMHN       .....            SCHICHT-NUMMER
C**                       ATMPH       DRUCK            HOEHE
C**                       ATMPT       .....            TEMPERATUR
C**                       ATMPG       .....            VERTIK.GRADIENT
C**                       ATMTH       TEMPERATUR       HOEHE
C**                       ATMTP       ..........       DRUCK
C**                       ATMNH       SCHICHT-NUMMER   HOEHE UNTERGRENZE
C**                YUPABS NAME DER RUFENDEN PROGRAMM-EINHEIT (CHAR.*6)
C**                IRM    RETURN-CODE 0=FEHLERFREI
C**                                  #0=FEHLER-NACHRICHT AUF LFN
C**                                     PROTOK/OUTPUT (S.UP PROTOP)
C**                                   0=WENN BEI /ATMNH/ -EIN- ZU GROSS
C**                                     WIRD, IST -IRM- DIE MAX.DEF.
C**                                     ATMOSPHAEREN-SCHICHTEN-ANZAHL
C**                YATMOD STANDARD-ATMOSPHAEREN-BEZEICHNUNG (CHAR.*6)
C**                       'US1976'=U.S.STANDARD-ATMOSPHAERE VON 1976
C**                        BIS 8.SCHICHT-UNTERGRENZE (MESOPAUSE)=86.KM
C**                EIN    EINGABE-ELEMENT JE NACH ENTRY-NAMEN
C**                       -DRUCK (HPA)
C**                       -HOEHE (M)
C**                       -TEMPERATUR (KELVIN)
C**                       -SCHICHT-NUMMER
C**                AUS    AUSGABE-ELEMENT JE NACH ENTRY-NAMEN
C**                       -DRUCK (GGF.DER SCHICHT-UNTERGRENZE) (HPA)
C**                       -HOEHE (GGF.DER SCHICHT-UNTERGRENZE) (M)
C**                       -TEMPERATUR (KELVIN)
C**                       -VERTIK.GRADIENT (KELVIN/M)
C**                       -SCHICHT-NUMMER
C**   ACHTUNG      /ATMTH/+/ATMTP/ SIND NICHT UNBEDINGT EINDEUTIG,
C**                DESHALB MUSS -AUS- BEI DER   E I N G A B E
C**                EINE UEBERSCHREIBARE DRUCK- BZW. HOEHEN-ANGABE
C**                ENTHALTEN, AB DER DIE NAECHSTE DARUEBERLIEGENDE
C**                TEMPERATUR-SCHICHT  -EIN- GESUCHT WERDEN SOLL.
C**                WIRD KEINE WEITERE GEFUNDEN RESULTIERT AUS=-9999.
C**                SOWIE IRM = -1

C     METHODE      ALS GLEICHUNGS-SYSTEME LIEGEN ZUGRUNDE:
C                  1) FUER POLYTROPE ATMOSPHAERE G0=-DT/DZ=CONST
C                  P(Z) = P0 * (1 - G0 * Z / T0) ** (GH/G0)
C                  2) FUER ISOTHERME ATMOSPHAERE DT/DZ=0.
C                  P(Z) = P0 * EXP ((-Z * G) / (RD * T0))
C                  MIT
C                  Z = HOEHE (M)
C                  G = GRAVITATIONS-BESCHLEUNIGUNG (M/S/S)
C                  RD= GASKONSTANTE TROCKENER LUFT (J/KG/K)
C                  T0= TEMPERATUR IN HOEHE P0 (K)
C                  P0= LUFTDRUCK BEI T0 (SCHICHTUNTERGRENZE,HPA)
C                  G0= VERTIK.TEMPERATUR-GRADIENT=DT/DZ (K/M)
C                  GH= ..........................EINER HOMOGENEN
C                      ATMOSPHAERE (K/M)
C                  EXP=EXPONENT ZUR BASIS E=2.7182828
C     LITERATUR    U.S.STANDARD-ATMOSPHAERE,1976
C                      NATIONAL OCEANIC AND ATMOSPHERIC ADMINISTRATION
C                      NATIONAL AERONAUTICS AND SPACE   ADMINISTRATION
C                      UNITED STATES AIR FORCE
C                      NOAA-S/T 76-1562 WASHINGTON D.C. OCTOBER 1976
C     VERFASSER    U.VOIGT
#include "undef.h"
#include "xyzprt.h"
      SAVE G,RDRY,VTGHOM,LFIRST
       LOGICAL LFIRST
      PARAMETER (NATM = 1 , MXSANZ = 8)
       CHARACTER *6,YUPNAM,YUPABS,YATML(NATM),YATMOD,YATMO2
      DIMENSION DEFINI (MXSANZ,4,NATM),SUNG(4)
      EQUIVALENCE (SUNG(1),H0)
     +           ,(SUNG(2),T0)
     +           ,(SUNG(3),G0)
     +           ,(SUNG(4),P0)
      DIMENSION ISANZ(NATM),G(NATM),RDRY(NATM)
C     SCHICHTANZAHL DER STANDARD-ATMOSPHAEREN
      DATA ISANZ /MXSANZ/
C     STANDARD-ATMOSPHAEREN-BEZEICHNUNGS-LISTE
      DATA YATML /'US1976'/
      DATA AUSV /0./
C     DEFINITION DER ATMOSPHAERE IM ARRAY DEFINI (I,J,K)
C     I = SCHICHT KONSTANTEN TEMP.-GRADIENTENS
C     J = 1 (M)  UNTERGRENZE EINER SCHICHT
C     J = 2 (K)  UNTERGRENZEN-TEMPERATUR
C     J = 3 (K/M)VERTIKALER-TEMP-GRADIENT
C     J = 4 (HPA)UTERGRENZEN-LUFTDRUCK
C     K = NUMMER DES STANDARDS
C
      DATA (((DEFINI (I,J,K),I=1,MXSANZ),J=1,4),K=1,NATM)/
C     U.S. STANDARD-ATMOSPHAERE VON 1976
     H      0., 11000., 20000., 32000., 47000., 51000., 71000., 86000.
     T ,288.15, 216.65, 216.65, 228.65, 270.65, 270.65, 214.65, 186.87
     G ,0.0065,     0., -0.001,-0.0028,     0., 0.0028,  0.002,     0.
     P,1013.25, 226.32, 54.748, 8.6801,  1.109,0.66938,.039564,.003733
     +/
      DATA LFIRST /.TRUE./
      YUPNAM = 'ATMOS'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMHP
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMHP'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMHT
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMHT'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMHG
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMHG'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMHN
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMHN'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMPH
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMPH'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMPT
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMPT'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMPG
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMPG'
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMTH
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMTH'
      AUSV = AUS
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMTP
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMTP'
      AUSV = AUS
      GO TO 1
C     ------------------------------------------------------------------
      ENTRY ATMNH
     +(YUPABS,IRM,YATMOD,EIN,AUS)
      YUPNAM = 'ATMNH'

1     AUS = -9999.
      IRM     = 0
      IF (LFIRST) THEN
         LFIRST = .FALSE.
C        U.S.STANDARD-ATMOSPHAERE 1976
         CALL PHYKON (YUPNAM,IRM,'AGRAUS',G(1))
         CALL PHYKON (YUPNAM,IRM,'RDRY'  ,RDRY(1))
         CALL PHYKON (YUPNAM,IRM,'VTGHOM',VTGHOM)
      ENDIF
C     ATMOSPHAEREN-NAMEN PRUEFEN
      YATMO2 = YATMOD
      CALL LISTB (YATMO2,YATML,IATDEX,NATM,'YATMOD',IRM,YUPNAM)
C     ------------------------------------------------------------------
C     ALLE ENTRIES MIT  H  - EINGABE
C     ------------------------------------------------------------------
      IF (YUPNAM(4:4) .EQ. 'H') THEN
         IF (EIN .LT. -500.) GO TO 998
         DO 100 ISDEX = ISANZ(IATDEX) , 1 , -1
         IF( EIN .GE. DEFINI(ISDEX,1,IATDEX)) GO TO 101
100      CONTINUE
         ISDEX = 1
101      DO 102 I=1,4
102      SUNG(I) = DEFINI(ISDEX,I,IATDEX)
         EIR = EIN - H0
         IF (YUPNAM .EQ. 'ATMHP') THEN
            IF (G0 .EQ. 0.) THEN
C              ISOTHERME ATMOSPHAERE
               AUS = P0 * EXP (-EIR / (RDRY(IATDEX) * T0 / G (IATDEX)))
            ELSE
C              POLYTROPE ATMOSPHAERE
               AUS = P0 * (1- G0 * EIR / T0) ** (VTGHOM / G0)
            ENDIF
         ELSE IF (YUPNAM .EQ. 'ATMHT') THEN
            IF (G0 .EQ. 0.) THEN
C              ISOTHERME ATMOSPHAERE
               AUS = T0
            ELSE
C              POLYTROPE ATMOSPHAERE
               AUS = T0 - G0 * EIR
            ENDIF
         ELSE IF (YUPNAM .EQ. 'ATMHG') THEN
               AUS = G0
         ELSE IF (YUPNAM .EQ. 'ATMHN') THEN
               AUS = ISDEX
         ENDIF
C     ------------------------------------------------------------------
C     ALLE ENTRIES MIT  P  - EINGABE
C     ------------------------------------------------------------------
      ELSE IF (YUPNAM(4:4) .EQ. 'P') THEN
         IF(EIN .LE. 0.) GO TO 998
         DO 200 ISDEX = ISANZ (IATDEX) , 1 , -1
         IF( EIN .LE. DEFINI(ISDEX,4,IATDEX)) GO TO 201
200      CONTINUE
         ISDEX = 1
201      DO 202 I=1,4
202      SUNG(I) = DEFINI(ISDEX,I,IATDEX)
         IF (YUPNAM .EQ. 'ATMPH') THEN
            IF (G0 .EQ. 0.) THEN
C              ISOTHERME ATMOSPHAERE
               AUS = - (RDRY(IATDEX) * T0 / G(IATDEX)) * LOG (EIN/P0)
            ELSE
C              POLYTROPE ATMOSPHAERE
               AUS = T0 / G0 * (1 - (EIN/P0) ** (G0 / VTGHOM))
            ENDIF
            AUS = AUS + H0
         ELSE IF (YUPNAM .EQ. 'ATMPT') THEN
            IF (G0 .EQ. 0.) THEN
C              ISOTHERME ATMOSPHAERE
               AUS = T0
            ELSE
               AUS = T0 * (EIN/P0) ** (G0/VTGHOM)
            ENDIF
         ELSE IF (YUPNAM .EQ. 'ATMPG') THEN
            AUS = G0
         ENDIF
C     ------------------------------------------------------------------
C     ALLE ENTRIES MIT  T  - EINGABE
C     ------------------------------------------------------------------
      ELSE IF (YUPNAM(4:4) .EQ. 'T') THEN
         IF (EIN .LE. 0.) GO TO 998
         IF (AUSV .LE. 0.) AUSV = -RUNDEF
         IF (YUPNAM .EQ. 'ATMTH') THEN
            DO 300 ISD = ISANZ (IATDEX) , 1 , -1
            IF (AUSV .GE. DEFINI (ISD,1,IATDEX)) GO TO 401
300         CONTINUE
            ISD = 0
         ELSE
            DO 400 ISD = ISANZ (IATDEX) , 1 , -1
            IF (AUSV .LE. DEFINI (ISD,4,IATDEX)) GO TO 401
400         CONTINUE
            ISD = 0
         ENDIF
401      DO 500 ISDEX = ISD+1 , ISANZ (IATDEX)
            NEXT = MIN0 (ISDEX + 1 , ISANZ(IATDEX))
            IF (((EIN .LE. DEFINI (ISDEX,2,IATDEX)) .AND.
     +           (EIN .GT. DEFINI (NEXT ,2,IATDEX)))      .OR.
     +          ((EIN .GE. DEFINI (ISDEX,2,IATDEX)) .AND.
     +           (EIN .LT. DEFINI (NEXT ,2,IATDEX)))
     +         ) GO TO 501
500      CONTINUE
            IRM = -1
            GO TO 999
501      DO 502 I=1,4
502      SUNG(I) = DEFINI (ISDEX,I,IATDEX)
         IF (YUPNAM .EQ. 'ATMTH') THEN
            IF (G0 .EQ. 0.) THEN
C              ISOTHERME ATMOSPHAERE
               AUS = H0
            ELSE
C              POLYTROPE ATMOSPHAERE
               AUS = (EIN-T0) / (-G0) + H0
            ENDIF
         ELSE IF (YUPNAM .EQ. 'ATMTP') THEN
            IF (G0 .EQ. 0.) THEN
C              ISOTHERME ATMOSPHAERE
               AUS = P0
            ELSE
C              POLYTROPE ATMOSPHAERE
               AUS = P0 * (EIN/T0) ** (VTGHOM/G0)
            ENDIF
         ENDIF
C     ------------------------------------------------------------------
C     ALLE ENTRIES MIT  N  - EINGABE
C     ------------------------------------------------------------------
      ELSE IF (YUPNAM (4:4) .EQ. 'N') THEN
         IZW = EIN
         IF (IZW .LT. 1) GO TO 998
         IF (IZW .GT. ISANZ(IATDEX)) THEN
            IRM = - ISANZ (IATDEX)
            WRITE (YBOZEL,20000) YATMO2,IRM
20000       FORMAT (A6,' MAX.DEF.SCHICHTANZAHL=',I2.1)
            CALL HERMES (YUPNAM,IRM1,'W',UPSTYX)
         ELSE
            AUS = DEFINI (IZW,1,IATDEX)
         ENDIF
      ELSE
C     ------------------------------------------------------------------
C     ENTRY ATMOS-DEFINITIONS-LISTING
C     ------------------------------------------------------------------
         DO 600 I = 1 , ISANZ(IATDEX)
         WRITE (YBOZEL,30000) YATMOD,I,(DEFINI(I,J,IATDEX),J=1,4)
30000    FORMAT (A6,' SCHICHT-NR.',I3.1,' H=',F10.2,' T=',F10.4
     +      ,' G=',F12.8,' P=',F13.8)
600      CALL HERMES (YUPNAM,IRM,'I',UPSTYX)
      ENDIF
      GO TO 999
998   WRITE (YBOZEL,40000)
40000 FORMAT ('KEINE SCHICHTZUORDNUNG MOEGL.')
      CALL HERMES (YUPNAM,IRM1,'W',UPSTYX)
      IRM = 1
999   IF ((NPR .GE. 9) .OR. (IRM .GT. 0)) THEN
         WRITE (YBOZEL,50000) YUPABS,IRM,YATMOD,EIN,AUS,AUSV
50000    FORMAT (A6,' IRM=',I2.1,' YATMOD=',A6,' EIN=',F12.4
     +      ,' AUS=',F12.4,' AUSV=',F12.4)
         CALL HERMES (YUPNAM,IRM1,'I',UPSTYX)
      ENDIF
      RETURN
      END
