/*
C**** cback    -   Zuruecksetzen des Dateizeigers um einen Datensatz    
C**   Aufruf   :   call cback ( iun, irm )
C**   Zweck    :   Der Schreib/Lesezeiger einer Datei wird um einen Datensatz
C**                zurueckgesetzt.
C**   Parameter:   iun   - Filedeskriptor, wird von dem Programm copen, mit dem
C**                        die Datei eroeffnet werden muss, geliefert.
C**                        Typ: INTEGER
C**                irm   - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte
C**                        nicht zurueckgesetzt werden.
C**   Methode  :   Mit Hilfe der C-Funktion fseek wird zuruekgesetzt. 
C**   Hinweise :   Die Datei muss mit dem DWDLIB-Programm copen eroeffnet 
C**                worden sein.
C**                Steht der Schreib/Lesezeiger bereits am Dateianfang, 
C**                passiert nichts.
C**                Die Datei muss auch mit Lesezugriff eroeffnet sein, da die 
C**                Recordlaengen gelesen werden.
C**                Fehlerbotschaften werden auf stderr ausgegeben, es wird dazu
C**                die C-Routine perror genutzt.
C**                Falls die Routine aus einem C-Programm gerufen werden soll:
C**                int cback ( &iun, &irm )
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Im Fehlerfall wird der Wert -1 geliefert, sonst 0.
C**   Verfasser:   A.Freyberg, T23, Januar 1994
C     Geaendert:   R.Johanni, B.Loeser, April 2001
C                  G.Eymann, 11.12.01: erweiterte Fehlerausgaben (keine fkt. Aend.)
C                  G.Eymann,M.Stahlhacke, 05.02.03: Verhalten im Fall Dateiende 
C                      modifiziert. Bei Fehler zunaechst backspace 8 Byte u. Auswerten 
C                      Byte 5-7, falls Null wird Routine verlassen; zusaetzl. Kennung BUFR  
C                  H.Weber, 27.11.03: Fehler beseitigt: Nach cback muss die Laenge 
C                      des vorherigen records ausgelesen und in prevrecl[] 
C                      gespeichert werden
C                  H.Weber, 2.4.04: Bug bei GRIB/BUFR Erkennung
C                  H.Weber, 20.4.04: LEER Erkennung
*/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

/* Filedeskriptoren werden extern definiert */
#define EXTERN extern
#include "libdwd_c.h"

/* Fortran-C-Schnittstelle wird includiert */
#include "fortran_c.h"

/* Fehlerbehandlung mit Hilfe der Fehlernummer errno */
extern int errno;

int cback ( FTN_INT *iun, FTN_INT *irm )
{

/* Variablendefinitionen */
  long ii, pos, cur_pos, last_pos, recl;
  unsigned char ctrl[8];

/* Fehlerzelle vorbesetzen */
  *irm = 0;
  
/* Filedeskriptor bestimmen */
  for ( ii=0; ii < MAX_FILES; ii++ ) 
    {
      if ( fdes[ii] > 0 && fdes[ii] == *iun ) break;
    }
    
  if ( ii >= MAX_FILES ) 
    {
      fprintf (stderr,"cback F Filedeskriptor %d existiert nicht!\n", *iun );
      *irm  = 1;
      return -1;
    }

/*  Momentane Position bestimmen. Falls am Anfang, braucht nichts getan werden  */
  cur_pos = our_tell ( dateizeiger[ii] );
  if ( cur_pos == 0 ) return 0;
 
/* negative Position? */
/* Innerhalb des "if" unten wird bei Fehlern mit goto ans Ende gesprungen */
 
  if ( cur_pos < 0 ) 
    {
      fprintf(stderr,"cback F Fehler beim Positionieren, Position < Null!");
      fprintf(stderr,"Datei: %s\n",dateien[ii]);
      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
      *irm = 1;
      return -1;
    }
  
/* Verfahren fuer Dateien im Standardformat */
  if ( dat_art[ii] == 's' ) 
    {
/* Falls noetig, abschliessendes Kontrollwort schreiben */
      if ( lastwrite[ii] ) 
	{
	  if ( write_control_word (ii,prevrecl[ii],0) ) 
	    {
	      fprintf (stderr,"cback F Fehler bei Ausgabe des Kontrollworts!\n");
	      fprintf (stderr,"Datei: %s\n",dateien[ii]);
	      fprintf (stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm  = 1;
	      return -1;
	    }
	  lastwrite[ii] = 0;
	  prevrecl[ii]  = 0;

/* Vor Kontrollwort zurueckpositionieren */
	  if ( our_seek( dateizeiger[ii], -8, SEEK_CUR ) < 0 ) goto seek_err;
	}

/* Die vier naechsten Controlbytes lesen */
      if ( our_read ( dateizeiger[ii], ctrl, 4 ) != 4 ) 
	{ 
/* falls Dateizeiger am Ende, gehe 8 byte zurueck und teste, ob bytes 5-8 Null */
	  if ( our_seek( dateizeiger[ii], -8, SEEK_CUR ) < 0 ) goto seek_err;
     	
/* lese bytes n-7 ... n der Datei */
	  if ( our_read ( dateizeiger[ii],ctrl,8 ) != 8 ) goto read_err;
	  recl = (ctrl[4]<<24)|(ctrl[5]<<16)|(ctrl[6]<<8)|(ctrl[7]);
     	
/* Ende-Kennung gefunden: verlasse Routine */
	  if ( recl == 0 ) 
	    {
	      fprintf(stderr,"cback W Dateiende erreicht - Dateizeiger vor Kontrollwort!\n");
	      fprintf(stderr,"Datei: %s\n",dateien[ii]);
	      if ( our_seek( dateizeiger[ii], -8, SEEK_CUR ) < 0 ) goto seek_err;
	      return 0;
	    } 
	  else 
	    {
	      goto read_err;
	    }
	}

/*  Umwandeln in eine Integer Zahl, unabhaengig von Wortlaenge und Bytereihenfolge  */

      recl = (ctrl[0]<<24)|(ctrl[1]<<16)|(ctrl[2]<<8)|(ctrl[3]);
      recl += 12; /*  Fuer Controlbytes  */

/*  Dateizeiger um einen record zuruecksetzen  */
      if ( our_seek( dateizeiger[ii], -recl, SEEK_CUR ) < 0 ) goto seek_err;

/*  Laenge das vorhergehenden records lesen und in prevrecl[] speichern  */
      if ( our_read ( dateizeiger[ii], ctrl, 4 ) != 4 ) goto seek_err;
      recl = (ctrl[0]<<24)|(ctrl[1]<<16)|(ctrl[2]<<8)|(ctrl[3]);
      prevrecl[ii] = recl;
      if ( our_seek( dateizeiger[ii], -4, SEEK_CUR ) < 0 ) goto seek_err;
    } 
  else 
    {
/* Verfahren fuer Dateien ohne Kontrollworte                  
   Vom Dateianfang ausgehend vorwaertsspringen, dabei immer               
   die Position des Vorgaengerrecords behalten, bis zu jetziger Position 
 */ 
      if ( our_seek ( dateizeiger[ii], 0L, SEEK_SET ) < 0 ) goto seek_err;
      
      while (1) 
	{
/* Momentane Position bestimmen */
	  last_pos = our_tell (dateizeiger[ii]);
	  if ( last_pos < 0 ) goto seek_err;
	  
/* Einlesen der ersten 8 Byte des Grib Records: */
      	  if ( our_read ( dateizeiger[ii],&ctrl[0],4 ) != 4 ) goto read_err;
	  
	  if ((ctrl[0]=='G' && ctrl[1]=='R' && ctrl[2]=='I' && ctrl[3]=='B') ||
	      (ctrl[0]=='B' && ctrl[1]=='U' && ctrl[2]=='F' && ctrl[3]=='R') ) 
	    {
/* Berechnen der Grib Laenge als Integer Zahl, unabhaengig von
   Wortlaenge und Bytereihenfolge  
 */
	      if ( our_read ( dateizeiger[ii],&ctrl[4],4 ) != 4 ) goto read_err;
	      recl = (ctrl[4]<<16)|(ctrl[5]<<8)|(ctrl[6]);
	      recl -= 8;  /*  wir haben schon 8 eingelesen  */
	      if ( our_seek (dateizeiger[ii], recl, SEEK_CUR) < 0 ) goto seek_err;
	    }
	  else if (!(ctrl[0]=='L' && ctrl[1]=='E' && ctrl[2]=='E' && ctrl[3]=='R') )
	    {
	      fprintf(stderr,"cback F Record hat keine GRIB, BUFR oder LEER Kennung!\n");
	      fprintf(stderr,"Datei: %s\n",dateien[ii]);
	      *irm = 1;
	      return -1;
	    }
	  
	  pos = our_tell (dateizeiger[ii]);
	  
	  if ( pos > cur_pos )
	    {
	      fprintf(stderr,"cback F Positionierung bei Aufruf falsch, Datei: %s\n",
		      dateien[ii]);
	      *irm = 1;
	      return -1;
	    }
	  
	  if( pos == cur_pos )
	    {
/* Position von vor der Suche erreicht */
	      if (our_seek (dateizeiger[ii], last_pos, SEEK_SET) < 0) goto seek_err;
	      break;
	    }
	}
    }
  return 0;

/* Fehlerbehandlung (nur Meldungen) */
 seek_err:
  fprintf(stderr,"cback F Fehler beim Positionieren, Datei: %s",dateien[ii]);
  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
  *irm = 1;
  return -1;

 read_err:
  fprintf(stderr,"cback F Fehler beim Lesen, Datei: %s\n",dateien[ii]);
  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
  *irm = 1;
  return -1;
}
