/*
C**** cclear  -   Zuruecksetzen einer Datei auf die laenge NULL
C**   Aufruf   :   call cclear ( iun, irm )
C**   Zweck    :   Der Dateizeiger einer Datei wird auf den Dateianfang 
C**                gesetzt und die Datei auf Laenge NULL gekuerzt
C**   Parameter:   iun   - Filedeskriptor, wird von dem Programm copen, mit dem
C**                        die Datei eroeffnet werden muss, geliefert.
C**                        Typ: INTEGER
C**                irm   - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte
C**                        nicht geloescht werden.
C**   Methode  :   Mit Hilfe der C-Funktion ftruncate wird abgeschnitten.
C**   Hinweise :   Die Datei muss mit dem DWDLIB-Programm copen eroeffnet 
C**                worden sein.
C**                Fehlerbotschaften werden auf stderr ausgegeben, wobei die
C**                C-Funktion perror genutzt wird.
C**                Falls das Programm aus einem C-Programm gerufen werden soll:
C**                int crewind_ ( &iun, &irm )
C**                Im Fehlerfall wird der Wert -1, sonst 0 geliefert.
C**   Verfasser:   H. Weber, TI17, Juni 2004
*/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

/* Filedeskriptoren werden extern definiert */
#define EXTERN extern

#include "libdwd_c.h"

/* Fortran-C-Schnittstelle */
#include "fortran_c.h"

/* Fehlerbehandlung mit Hilfe der Fehlernummer errno */
extern int errno;

int cclear ( FTN_INT *iun, FTN_INT *irm )
{
  unsigned int  ii;

/* Fehlerzelle vorbesetzen */
  *irm = 0;

/* Filedeskriptor bestimmen */
  for ( ii=0; ii<MAX_FILES; ii++ ) 
    if ( fdes[ii]>0 && fdes[ii] == *iun ) break;
  
  if ( ii >= MAX_FILES )
    {
      fprintf (stderr,"cclear  F Filedeskriptor %d existiert nicht!\n", *iun );
      *irm  = 1;
      return -1;
    }

/* Kontrollwortspeicher */
  lastwrite[ii] = 0;
  prevrecl[ii]  = 0;
  
/* Zuruecksetzen */
  if ( our_seek ( dateizeiger[ii], 0L, SEEK_SET ) < 0 )
    {
      fprintf(stderr,"cclear  F Fehler bei Fkt. our_seek, Datei: %s\n",
	      dateien[ii]);
      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
      *irm = errno;
      return -1;
    }

/* Abschneiden */
  if ( ftruncate ( fdes[ii], 0 ) < 0 )
    {
      fprintf(stderr,"cclear  F Fehler bei Fkt. ftruncate, Datei: %s\n",
	      dateien[ii]);
      fprintf(stderr, "error %d: %s\n", errno, strerror(errno));
      *irm = errno;
      return -1;
    }

  return 0;
}
