/*
C**** cclose   -   Schliessen einer Datei von FORTRAN aus
C**   Aufruf   :   call cclose ( iun, modus, irm )
C**   Zweck    :   Eine Datei, die Binaerdaten im Standardformat enthaelt,
C**                soll geschlossen werden.
C**   Parameter:   iun    - Filedeskriptor, wird zurueckgegeben, Typ: INTEGER
C**                modus  - Modus des Schliessens, Typ CHARACTER*3, erlaubt 
C**                sind:
C**                del: Die Datei wird geloescht.
C**                exi: Die Datei bleibt erhalten.
C**                irm    - Fehlerrueckmeldezelle, ist irm ungleich 0, ist bei   
C**                         der Ausfuehrung des Programms ein Fehler aufge-
C**                         treten. Typ: INTEGER  
C**   Methode  -   Die entsprechenden C-Routinen zum Schliessen oder Vernichten
C**                von Dateien werden aufgerufen.
C**   Hinweise -   Die Datei muss mit dem DWDLIB-Programm copen eroeffnet   
C**                worden sein. Es koennen nicht mehr als hundert Dateien 
C**                gleichzeitig offen sein. Sollte dies zu wenig sein, bitte 
C**                bei TI 12 melden. Fehlerbotschaften werden auf stderr aus-
C**                gegeben, wobei die C-Funktion perror genutzt wird.
C**                Falls das Programm aus einem C-Programm gerufen werden 
C**                soll (auf SGI, IBM etc., nicht Cray: 
C**                int cclose(&iun,modus,&irm,sizeof(modus))
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Im Fehlerfall wird -1, sonst 0 zurueck gegeben.
C**   Verfasser:   A.Freyberg, T23, Januar 1994
C     Geaendert:   R.Johanni, B.Loeser, April 2001
C                  M.Stahlhacke, G.Eymann, 7.12.01:lokale Kopie Modus,
C                  Gross- u. Kleinschreibung erlaubt
C                  G.Eymann, 11.12.01: 
C                    erweiterte Fehlerausgaben, keine fkt. Aenderungen
C                    Funktion closeall hier 
*/   
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/file.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/*   Filedeskriptoren werden extern definiert                                 */

#define EXTERN extern
#include "libdwd_c.h"

/*   Fortran-C-Schnittstelle wird includiert                                  */
#include "fortran_c.h"

/*   Fehlerbehandlung mit Hilfe der Fehlernummer errno                        */
extern int errno;

#if defined _CRAY
int cclose ( FTN_INT *iun, _fcd modus_fcd, FTN_INT *irm )
{
   char *modus = _fcdtocp(modus_fcd);
   /* unsigned long modus_len = _fcdlen(modus_fcd); */ /* Z.Z. nicht benutzt */
#else
int cclose
   ( FTN_INT *iun, char modus[], FTN_INT *irm, unsigned long modus_len)
{
#endif
/*   Variablendefinitionen                                                    */

 unsigned int ii;
 char cmodus[4];

/*   Fehlerzelle vorbesetzen                                                  */

 *irm = 0;

/*   Filedeskriptor bestimmen                                                 */

 for ( ii=0; ii < MAX_FILES; ii++ ) 
    if ( fdes[ii] > 0 && fdes[ii] == *iun ) break;
 
 if ( ii >= MAX_FILES )
 {
     fprintf(stderr,"cclose  F Filedeskriptor %d existiert nicht!\n", *iun );
     *irm  = 1;
     return -1;
 }

/*   Falls noetig, abschliessendes Kontrollwort schreiben */

 if(lastwrite[ii])
 {
     if( write_control_word(ii,prevrecl[ii],0) )
     {
         fprintf(stderr,"cclose  F Fehler bei Ausgabe des Kontrollworts, Datei %s:\n",
                 dateien[ii]);
         fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
         *irm  = 1;
         return -1;
     }
 }

/*   Schliessen des Streams                                                   */
/*   Beachte: return Wert von ffclose ist bei Erfolg auf Cray positiv,
              fclose auf SGI, IBM etc. == 0! */

#ifdef FFIO
 close (fdes[ii]);
 if ( ffclose ( dateizeiger[ii] ))
#else
 if ( fclose  ( dateizeiger[ii] ))
#endif
    {
    fprintf(stderr,"cclose  F Fehler bei Schliessen Datei %s\n",
            dateien[ii]);
    fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
    *irm  = errno;
    return -1;
    }

/*   Schliessen der Datei                                                     */

 strcpy (cmodus, "   ");
 cmodus[0] = tolower (modus[0]);
 
 if ( cmodus[0] == 'e')
  {
   fdes[ii] = 0;
  }

/*   Loeschen der Datei                                                       */

 else if ( cmodus[0] == 'd')
  {
   if (( unlink ( dateien[ii] )) != 0)
      {
      fprintf(stderr,"cclose  W Fehler bei Loeschen Datei %s\n",
              dateien[ii]);
      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
      }
   fdes[ii] = 0;
  }

/*   Falsche Parametereingabe                                                 */

 else 
  {
   fprintf(stderr,"cclose  F Falscher Wert des Parameters modus: %s\n", modus);
   *irm  = 1;
   return -1;
  }

/*   Loeschen des Dateinamens in dateien[ii] und der Dateiart in dat_art[ii]. */

 memset ( dateien[ii], ' ', MAX_FILENAME_LEN );
 dat_art[ii] = ' ';
 prevrecl[ii]  = 0;
 lastwrite[ii] = 0;
 return 0;
}

/****************************************************************************
 * closeall: 
 * Wird im Fall von exit oder abort aufgerufen und
 * schreibt die letzten Record-Control-Worte, wo diese
 * noch geschrieben werden muessen.
 * Anschliessend erfolgt ein close auf noch offene Files.
 * Fehler werden nicht abgefragt, da sie in diesem Fall
 * (exit oder abort) nicht sinnvoll behandelt werden koennen.
 * Nur Ausgabe Meldung (ge)
 ***************************************************************************/

void closeall(void)
{
 int ii;
 errno = 0;

 for ( ii=0; ii < MAX_FILES; ii++ )
 {
     if (fdes[ii])
     {
         if (lastwrite[ii]) write_control_word(ii,prevrecl[ii],0);
         if (errno != 0)
            {
            fprintf(stderr,"closeall  W Fehler bei Ausgabe Kontrollwort - ignoriert!\n");
            fprintf(stderr,"Datei: %s\n",dateien[ii]);
            fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
            }
         
#ifdef FFIO
         close (fdes[ii]);
         if (ffclose ( dateizeiger[ii] ))
#else
         if (fclose  ( dateizeiger[ii] ))
#endif
            {
            fprintf(stderr,"closeall  W Fehler von Fkt. f(f)close - ignoriert!\n");
            fprintf(stderr,"Datei: %s\n",dateien[ii]);
            fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
            }
         fdes[ii] = 0;
     }
  }
}

