/*
C**** cdaten_vonpb - Recordkontrollworte in GRIB oder BUFR Datei einsetzen
C***
C**   Aufruf   :   cdaten_vonpb (char infile[], char outfile [], int *iednr,
C**                               int *ihits, int *irm)
C**   Zweck    :   Aufbereitung von GRIB- oder BUFR-Daten, die sich auf einem
C**                nicht strukturierten File (ohne Record-Marken) befinden, in
C**                das Standardformat.Bei Ed. 1 wird rts_ed1 gerufen.
C**   Parameter:   infile  - Inputfile im puren Binaerformat
C**                outfile - Outputfile im Standardformat  
C**                iednr   - Steuerparameter fr die Daten, die auf infile stehen
C**                ihits   - Anzahl der verarbeiteten Felder
C**                irm     - Rueckgabewert
C**
C**   Externals :  copen, cclose, uegex, rts_ed1
C**   ACHTUNG   :  Bei Grib 0 wird nicht die Laenge geprueft!
C**                GRIB 0 nicht fuer LINUX portiert
C**   Verfasser :  R.Stanek, T23, August 1995
C     Geaendert:   A.Freyberg, November 1999
C                    TRUE/FALSE evtl. bereits definiert 
C                  G.Eymann, 11.12.01: Fehlerausgaben erweitert
C                  C.Vollbrecht TI12, Juni 2002, Ergnzung fr BUFR
C                  B.Loeser, juli 2002, Ausstieg Ed. 0 fuer LINUX
C                  H.Weber, april 2004, Anpassung an autom. Kontrollworte
C                  H. Weber, 02/2006  max_char increased for ECMWF 0.25deg
C                  C. Gramalla 02/2006  Aufruf von ccuegex ohne byte-swapping
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#define MAX_CHAR           3000000
#define MAX_FILE_CHAR      256
#ifndef FALSE
#define FALSE (char)0
#endif
#ifndef TRUE
#define TRUE (char)1
#endif

/*   Fortran-C-Schnittstelle wird includiert      */
#include "fortran_c.h"

typedef char LOGICAL;

int cdaten_vonpb ( char infile[], char outfile[], int *iednr, int *ihits, int *irm)

{
  int i;
  int ch, iun;
  char ifeld [MAX_CHAR];
  char cinfile[MAX_FILE_CHAR];
  char coutfile[MAX_FILE_CHAR];
  LOGICAL lgrib;
  FILE * fp_in;

  char leerzeichen = ' ';

/* keine Portierung von GRIB 0 - Ed. 0 - auf LINUX */

#ifdef __linux__
   if ( *iednr == 0)
      {
       printf("raw_to_stf fuer GRIB 0 nicht auf LINUX portiert!\n");
       exit(1);
      }
#endif

   for ( i=0; i < MAX_FILE_CHAR ; ++i )
        if ( infile[i] == leerzeichen ) break;

   strncpy ( cinfile, infile, i);
   cinfile[i] = '\0';

   for ( i=0; i < MAX_FILE_CHAR ; ++i )
        if ( outfile[i] == leerzeichen ) break;

   strncpy ( coutfile, outfile, i);
   coutfile[i] = '\0';

/* Oeffnen der Dateien, Fehlerabfrage: */

  if (*iednr < 0 || *iednr > 1)
     {
      printf("Nur Edition-Nr 0 oder 1 ist erlaubt!\n");
      fprintf(stderr,"eingegebene Edition-Nr: %d\n",*iednr);
      *irm = 1;
      return *irm;
     } 
  
  if ( (fp_in = fopen (cinfile,"r")) == NULL)
     {
      printf("File %s kann nicht geoeffnet werden!",cinfile);
      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
      *irm = 2;
      return *irm;
     }
  
  copen (&iun,coutfile,"w+s",irm,strlen(coutfile),3);
  if (*irm != 0)
     {
      printf("Fehler bei COPEN, IRM= %d\n",*irm);
      *irm = 2;
      return *irm;
     }

/* Verzweigung, bei Edition 1 Aufruf von Up rts_ed1 */

  if ( *iednr == 1 )
     {
      *ihits = 0;
      rts_ed1 ( fp_in, iun, ihits, irm);
     }
   else
     {
             
/* Initialisierung: */

    i = 0;
    *ihits = 0;

    lgrib = FALSE;

/* Einlesen, Byte fuer Byte: */
/* (implizite Typkonversion, ch ist int - OK?) */

    while ( (ch = fgetc (fp_in)) != EOF )
        {
        ifeld[i++] = ch;

/* GRIB-Kennung suchen: */

         if (i >3 && ifeld[i-4] == 'G' && ifeld[i-3] == 'R' &&
                     ifeld[i-2] == 'I' && ifeld[i-1] == 'B')
            {
            lgrib = TRUE;
            ifeld[0] = 'G';
            ifeld[1] = 'R';
            ifeld[2] = 'I';
            ifeld[3] = 'B';
            i = 4 ;
           }

/* Falls keine GRIB-Kennung, nach BUFR-Kennung suchen: */

         if ( ! lgrib && i >3 && ifeld[i-4] == 'B' && ifeld[i-3] == 'U' &&
                                 ifeld[i-2] == 'F' && ifeld[i-1] == 'R')
            {
            lgrib = TRUE;
            ifeld[0] = 'B';
            ifeld[1] = 'U';
            ifeld[2] = 'F';
            ifeld[3] = 'R';
            i = 4 ;
           }

/* Ende-Kennung 7777 suchen. Bei Erfolg Rausschreiben mit "uegex". */

         if (lgrib && i > 3 && ifeld[i-1] == '7' && ifeld[i-2] == '7' &&
                               ifeld[i-3] == '7' && ifeld[i-4] == '7' ) 
            {
             ccuegex(&iun,ifeld,&i,0,irm);

             (*ihits)++;

             if (*irm != 0)
                 printf("Fehler bei ccuegex, IRM= %d",*irm);
             i = 0;
             lgrib = FALSE;
            }
         }
     }
     
  printf("Anzahl der Datensaetze:  %d\n",*ihits);
  
  fclose(fp_in);
  
  cclose(&iun,"exi",irm,3);
  
  return 0;
}
