/*
C**** cfindnext -  Laenge des zu lesenden Datensatzes finden
C**   Aufruf   :   call cfindnext ( iun, max_skip, offset, lflb, irm )
C**   Zweck    :   Die zu allokierende Datensatzlaenge wird von einer Datei,
C**                die Binaerdaten im Standardformat enthaelt, gefunden.
C**                Der Dateizeiger wird am Anfang des Datensatzes positioniert.
C**   Parameter:   iun      - Filedeskriptor, wird von dem Programm copen, mit 
C**                           dem die Datei eroeffnet werden muss, geliefert.
C**                           Typ: INTEGER
C**                max_skip - Maximale Anzahl von Zeichen durchsuchen, um Datensatz
C**                           zufinden
C**                           !!! nur fuer Dateien ohne Kontrollworte von Bedeutung
C**                offset   - Anzahl der vorgespulten Bytes
C**                           !!! nur fur Dateien ohne Kontrollworte von Bedeutung
C**                lflb     - Laenge des zu lesenden Records = Kontrollwort bzw.
C**                           im GRIB, BUFR gespeicherte Laenge in Bytes.
C**                           Ist das Dateiende erreicht oder ein Fehler
C**                           aufgetreten, ist lflb=0. Typ: INTEGER
C**                irm      - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte 
C**                           kein Datensatz gelesen werden.
C**                           irm = -3 nach max_skip kein Datensatz gefunden
C**   Methode  :   Mit Hilfe der C-Funktion fread wird die Laenge zurueckgeliefert
C**                Die Datei ist am Anfang des zu lesenden Records positioniert
C**                also hinter dem offset.
C**                Der Record kann mit cuegin gelesen werden.
C**   Hinweise :   Die Datei muss mit dem DWDLIB-Programm copen mit Lesezugriff
C**                eroeffnet worden sein.
C**                Fehlermeldungen werden auf stderr ausgegeben
C**                Falls die Routine aus einem C-Programm gerufen werden 
C**                soll: int cfindnext ( &iun, &lflb, &max_skip, &offset, &irm )
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Die Funktion liefert bei erfolgreicher Beendigung den 
C**                Wert 0, im Fehlerfall -1, -3
C**                Die Datei ist am Anfang des zu lesenden Records positioniert
C**                Ist max_skip=0 wird Kontrollwort bzw. in den naechsten 8 Bytes
C**                GRIB bzw. BUFR & die Laenge erwartet
C**   Verfasser:   C. Gramalla , Jan. 2006
C**   Version      : 1.0
C**   Geaendert: (contrachronologisch)
C**   dd.mm.yyyy  <name>       - <beschreibung>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

/* Filedeskriptoren werden extern definiert */

#define EXTERN extern

#include "libdwd_c.h"

/* Fortran-C-Schnittstelle wird includiert */
#include "fortran_c.h"

/* Fehlerbehandlung mit Hilfe der Fehlernummer errno */

extern int errno;

int cfindnext ( FTN_INT *iun, FTN_INT *max_skip, FTN_INT *offset, FTN_INT *lflb,
             FTN_INT *irm )
{

/* Variablendefinitionen */

  long ii, recl;
  int numchar,  max_chars, a;
  unsigned char ctrl[8];

/* Fehlerzelle vorbesetzen */

  *irm = 0;

/* Wir setzen die Anzahl der eingelesenen Bytes auf 0 */

  *lflb = 0;

/* Anzahl der ueberlesenen Bytes auf 0 */
  *offset = 0;

  numchar=0;
  max_chars=*max_skip;

/* Filedeskriptor bestimmen */

  for ( ii=0; ii<MAX_FILES; ii++ ) 
    if ( fdes[ii]>0 && fdes[ii] == *iun ) 
      break;

  if ( ii >= MAX_FILES )
    {
      fprintf (stderr,"cfindnext: F Filedeskriptor %d existiert nicht!\n", *iun );
      *irm  = -1;
      return -1;
    }

/* Einlesen eines Records */

/* Bestimmen der Dateiart */

  if ( dat_art[ii] == 'b' )
    {

/* Einlesen der ersten 8 Bytes des Records, die die Kennung GRIB und die 
   Laenge des GRIBs enthalten muessen. 
   Falls an dieser Stelle das read fehlschlaegt, nehmen wir an, 
   dass das Dateiende erreicht ist. 
 */
      if( our_read( dateizeiger[ii], &ctrl[0], 4 ) != 4 ) return 0;

      while(( ctrl[0]!='G' || ctrl[1]!='R' || ctrl[2]!='I' || ctrl[3]!='B') &&
	 ( ctrl[0]!='B' || ctrl[1]!='U' || ctrl[2]!='F' || ctrl[3]!='R') )
      {

         if(numchar>=max_chars) {
         fprintf(stderr,"Recordlength not found, skips max. of %d characters\n",max_chars);
         *irm  = -3;
         return -3;
         }

         a = fgetc(dateizeiger[ii]);
         if(a==EOF) return 0;

         numchar++; /* number of characters read after the first 4 */

         /* Shift search string */
         ctrl[0]=ctrl[1]; ctrl[1]=ctrl[2]; ctrl[2]=ctrl[3]; ctrl[3]=a;
      }
      *offset=numchar;
      if(( ctrl[0]=='G' && ctrl[1]=='R' && ctrl[2]=='I' && ctrl[3]=='B') || 
	 ( ctrl[0]=='B' && ctrl[1]=='U' && ctrl[2]=='F' && ctrl[3]=='R') )
        {
/* OK, GRIB oder BUFR */
	  if( our_read( dateizeiger[ii], &ctrl[4], 4 ) != 4 ) return 0;
/* Berechnen der Grib Laenge als Integer Zahl, unabhaengig von
   Wortlaenge und Bytereihenfolge 
 */
	  recl = (ctrl[4]<<16)|(ctrl[5]<<8)|(ctrl[6]);

/* Zurueckpositionieren um die 8 gelesenen Bytes */

	  if( our_seek( dateizeiger[ii], -8, SEEK_CUR) < 0 ) goto seek_err;
	}
      else if ( ctrl[0]=='L' && ctrl[1]=='E' && ctrl[2]=='E' && ctrl[3]=='R')
	{
/* OK, LEER */
	  recl = 4;

/* Zurueckpositionieren um die 4 gelesenen Bytes */

	  if( our_seek( dateizeiger[ii], -4, SEEK_CUR) < 0 ) goto seek_err;
	}
      else
	{
/* Wir nehmen an, da das Dateiende erreicht ist */
	  return 0;
        }

    }
  else
    {

/* Einlesen der ersten 8 Bytes des Records, die die Laengeninformationen 
   fuer vorhergehenden und aktuellen Record enthalten. 
*/
      
      if( our_read( dateizeiger[ii], ctrl, 8 ) != 8 ) goto read_err;

/* Berechnen der Record-Laenge als Integer Zahl, unabhaengig von
   Wortlaenge und Bytereihenfolge 
 */

      recl = (ctrl[4]<<24)|(ctrl[5]<<16)|(ctrl[6]<<8)|(ctrl[7]);
/*      *lflb = recl;*/

      if( our_seek( dateizeiger[ii], -8, SEEK_CUR) < 0 ) goto seek_err;
/* Dateiende wenn recl == 0 */

      if (recl == 0) return 0;

    }

  *lflb = recl;

  return 0;

/* Fehlerbehandlung */

 seek_err:
  fprintf(stderr,"cfindnext:  F Fehler beim Positionieren, Datei: %s",
	  dateien[ii]);
  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
  *irm = -1; 
  return -1;
  
 read_err:
  fprintf(stderr,"findnext:  F Fehler beim Lesen, Datei: %s\n",
	  dateien[ii]);
  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
  *irm = -1;
  return -1;
  
}
