/*
C**** cinquire -   Feststellen, ob eine Datei existiert und offen ist
C**   Aufruf   :   call cinquire ( iun, infile, iexi, iopn, irm )
C**   Zweck    :   Es wird festgestellt, ob eine Datei existiert und mit dem
C**                Programm copen_.c eroeffnet wurde.         
C**   Parameter:   iun    - Filedeskriptor, wird zurueckgegeben, wenn die    
C**                         Datei schon geoeffnet wurde, sonst -2. Typ: INTEGER
C**                infile - Dateiname, Typ: CHARACTER*n , n <= 254
C**                iexi   - Schalter, = 0, wenn die Datei nicht existiert,     
C**                                   = 1, wenn es die Datei schon gibt.         
C**                         Typ: INTEGER                                  
C**                iopn   - Schalter, = 0, wenn die Datei noch nicht mit copen
C**                                        geoeffnet wurde,
C**                                   = 1, wenn die Datei schon mit copen ge-
C**                                        oeffnet wurde und das Standardformat 
C**                                        enthalten soll, dann wird der File- 
C**                                        deskriptor zurueckgegeben.    
C**                                   = 2, wenn die Datei schon mit copen ge-
C**                                        oeffnet wurde und keine Kontrollworte
C**                                        enthalten soll, dann wird der File- 
C**                                        deskriptor zurueckgegeben.    
C**                         Typ: INTEGER
C**                irm    - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte 
C**                         ein Fehler nicht abgefangen werden. Typ: INTEGER  
C**   Methode  -   Die C-Routine stat wird gerufen, um festzustellen, ob die 
C**                Datei existiert, anschliessend wird geprueft, ob es schon 
C**                einen Filedeskriptor zu dem Dateinamen gibt.
C**   Hinweise -   Es koennen nicht mehr als hundert Dateien gleichzeitig 
C**                offen sein. Sollte dies zu wenig sein, bitte bei TI 12 
C**                melden. Fehlerbotschaften werden auf stderr ausgegeben, es 
C**                wird dabei die C-Funktion perror genutzt.
C**                Falls das Programm aus einem C-Programm heraus aufgerufen 
C**                werden soll: 
C**                cinquire ( &fdes, infile, &iexi, &iopn, &irm, strlen(infile))
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Die Funktion besitzt den Typ int und liefert bei erfolg-
C**                reicher Beendigung den Wert 0, sonst -1.
C     Verfasser-   A.Freyberg, TI 12, Februar 1994
C     Geaendert:   G.Eymann, 11.12.01: Fehlerausgaben erweitert, lint-clean
*/   
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/file.h>
#include <stdlib.h>
#include <string.h>

/*   Filedeskriptoren werden extern definiert                                 */

#define EXTERN extern

#include "libdwd_c.h"

/*   Fortran-C-Schnittstelle wird includiert                                  */

#include "fortran_c.h"

/*   Fehlerbehandlung mit Hilfe der Fehlernummer errno                        */

extern int errno;

#if defined _CRAY
int cinquire ( FTN_INT *iun, _fcd infile_fcd, FTN_INT *iexi, FTN_INT *iopn, FTN_INT *irm)
{
   char *infile = _fcdtocp(infile_fcd);
   unsigned long infile_len = _fcdlen(infile_fcd);
#else
int cinquire ( FTN_INT *iun, char infile[], FTN_INT *iexi, FTN_INT *iopn, 
     FTN_INT *irm, unsigned long infile_len)
{
#endif

/*   Variablendefinitionen                                                    */

 int          i, ii, slen;
 char         cinfile[MAX_FILENAME_LEN];
 struct stat  buf;

/*   Fehlerzelle und Schalter vorbesetzen                                     */

 *irm  = 0;
 *iexi = 0;
 *iopn = 0;

 /* Laenge des Dateinamens ohne Blanks am Ende bestimmen */

 for ( i=infile_len; i > 0 ; i-- )
     if ( infile[i-1] != ' ' ) break;

/*   ist ein Dateiname uebergeben worden, feststellen, ob die Datei  */
/*   existiert und offen ist  */

 if ( i )
 {
     if ( infile[0] != '/' )
     {
         if ( getcwd ( cinfile, MAX_FILENAME_LEN-1 ) == NULL)
         {
             fprintf(stderr,"cinquire  F Verzeichnis nicht gefunden: %s\n",cinfile);
             fprintf(stderr,"getcwd: error %d: %s\n",errno,strerror(errno));
             *irm = 1;
             return -1;
         }
         strcat (cinfile,"/");
     }
     else
        cinfile[0] = 0;

     /* Check ob der gesamte Filename zu lang wird */

     slen = strlen (cinfile);
     if((slen + i) > MAX_FILENAME_LEN-1)
     {
        fprintf(stderr,"cinquire  F Pfad+Dateiname zu lang, Maximum ist %d\n",
                        MAX_FILENAME_LEN-1);
        *irm = 1;
        return -1;
     }

     strncat (cinfile,infile,i);

     *iun = -2;

/*   feststellen, ob die Datei existiert                                      */

     if ( stat ( cinfile, &buf ) == 0 ) 
        {
         *iexi = 1;
        }
     else
        {
         /* NB: errno 2 ist "No such file or directory" */
         if ( errno == 2 )
            {
            *iexi = 0;
            }
         else
            {
             fprintf(stderr,"cinquire  F Fehler von Fkt. stat bei Verzeichnis: %s\n",
                     cinfile);
             fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
             *irm = 1;
             return -1;
            }
        }

/*   feststellen, ob die Datei offen ist */
/*   weil es einen Fall gab, bei dem ein File geschlossen und geloescht wurde, */
/*   obwohl er an anderer Stelle noch offen war, werden auch nicht mehr existiernde */
/*   Dateien ueberprueft und evtl. geschlossen */

/*   RJ: Ob das so gut ist ???? */

     for ( ii=0; ii < MAX_FILES; ++ii )
        {
         if ( fdes[ii] )
            {   
             if ( !strcmp ( cinfile, dateien[ii] ) )  
                {
                 if ( *iexi == 1 )
                    {
                     if ( dat_art[ii] == 's' ) 
                        *iopn = 1;
                     else                      
                        *iopn = 2;
                     
                     *iun  = fdes[ii];
                    }
                 else if ( *iexi == 0)
                    {
#ifdef FFIO
                     close (fdes[ii]);
                     ffclose ( dateizeiger[ii] );
#else
                     fclose ( dateizeiger[ii] );
#endif
                     fdes[ii] = 0;
                     memset ( dateien[ii], ' ', MAX_FILENAME_LEN );
                     dat_art[ii] = ' ';
                    }
                }
            }
        }
    }

/*   ist ein Filedeskriptor uebergeben worden, feststellen, ob dazu eine      */
/*   geoeffnete Datei existiert und ihren Namen zurueckgeben                  */

 else 
    {
     for ( ii=0; ii < MAX_FILES; ++ii )
        {
         if ( fdes[ii] == *iun )
            {
             if (strlen(dateien[ii]) > infile_len)
                {
                 fprintf(stderr,"cinquire  F Dateiname zu lang, Max: %d\n",
                         MAX_FILENAME_LEN);
                 *irm = 1;
                 return -1;
                }
             
             strcpy ( infile, dateien[ii] );
             for(i=strlen(dateien[ii]); i < infile_len; i++) 
                infile[i] = ' ';
             
             if ( dat_art[ii] == 's' ) 
                *iopn = 1;
             else                      
                *iopn = 2;
             
             *iexi    = 1;
            }
        }
    }
 return 0;
}
