/*
C**** copen    -   Oeffnen einer Datei von FORTRAN aus
C**   Aufruf   :   call copen ( iun, infile, modus, irm )
C**   Zweck    :   Eine Datei, die Binaerdaten im Standardformat oder ohne
C**                Kontrollworte enthaelt oder enthalten wird, soll eroeffnet
C**                werden. Die Verarbeitung von Daten ohne Kontrollworte 
C**                ist nur fuer das Format GRIB Edition 1 moeglich.
C**   Parameter:   iun    - Filedeskriptor, wird zurueckgegeben, Typ: INTEGER
C**                infile - Dateiname, Typ: CHARACTER*n , n <= 254
C**                modus  - Modus des Oeffnens, Typ CHARACTER*3,  erlaubt sind:
C**                r  : Eine Datei, die schon existieren muss, wird zum Lesen 
C**                     geoeffnet. Es wird eine Sperre gesetzt, die verhindert,
C**                     dass andere Prozesse die Datei beschreiben.
C**                     Die Datei kann Kontrollworte haben.
C**                r b: wie oben
C**                r s: wie oben
C**                r+ : Eine Datei, die schon existieren muss, wird zum Lesen
C**                     und Schreiben geoeffnet. Der Inhalt der Datei geht 
C**                     nicht verloren, es wird aber am Anfang geoeffnet.  
C**                     Andere Prozesse koennen nicht auf die Datei zugreifen.
C**                     Die Datei kann Kontrollworte haben.
C**                r+b: wie oben
C**                r+s: wie oben
C**                w  : Datei wird zum Schreiben geoeffnet. Falls die Datei 
C**                     schon existiert, geht der alte Inhalt verloren, gibt es  
C**                     sie noch nicht, wird sie mit den Rechten rw-r--r-- 
C**                     angelegt. Andere Prozesse haben keinen Zugriff.
C**                     Ist die Umgebungsvariable LIBDWD_FORCE_CONTROLWORDS
C**                     gesetzt, so hat die neue Datei Kontrollworte.
C**                w b: wie oben, jedoch in jedem Fall ohne Kontrollworte
C**                w s: wie oben, jedoch in jedem Fall mit Kontrollworten
C**                w+ : Wie Modus w, es kann aber auch von der Datei gelesen    
C**                     werden, nachdem sie beschrieben wurde.                
C**                w+b: wie oben, jedoch in jedem Fall ohne Kontrollworte
C**                w+s: wie oben, jedoch in jedem Fall mit Kontrollworten
C**                w+t: Wie Modus w+, es wird aber kein Dateiname uebergeben,
C**                     sondern vom System ein Name in /tmp zugewiesen. Die
C**                     Datei wird geoeffnet und "Unit" und Name zurueckge-
C**                     geben. Der Name ist unter EP/IX 18 Zeichen lang, unter
C**                     UNICOS wird eine Datei mit einem 9 Zeichen langen Namen
C**                     in Ihrem $TMPDIR angelegt.  Die Datei hat keine 
C**                     Kontrollworte!
C**                     Gedacht fuer temporaere Dateien, die am Programmende
C**                     geloescht werden. Bitte loeschen Sie die Datei, wenn 
C**                     sie nicht mehr gebraucht wird, da es dafuer keinen
C**                     Automatismus gibt!
C**                     Das Verzeichnis /tmp wird zwar regelmaessig aufge-
C**                     raeumt, aber die Dateien werden erst weggeworfen, wenn
C**                     sie aelter als vierzehn Tage sind. Benutzen Sie also
C**                     bitte den Modus 'del' beim Aufruf von cclose fuer diese
C**                     Datei. 
C**                a  : Datei wird am Ende zum Schreiben geoeffnet. Ob 
C**                     Kontrollworte verwendet werden, richtet sich nach der
C**                     bestehenden Datei. 
C**                     Gibt es die Datei noch nicht, wird sie mit den Rechten
C**                     rw-r--r-- angelegt.  Die Kontrollworte richten sich dann 
C**                     nach der Umgebungsvariablen LIBDWD_FORCE_CONTROLWORDS.
C**                     Andere Prozesse haben keinen Zugriff auf die Datei.  
C**                a b: wie oben, jedoch bei neuer Datei ohne Kontrollworte
C**                a s: wie oben, jedoch bei neuer Datei mit Kontrollworten
C**                a+ : Wie Modus a, es kann aber auch von der Datei gelesen
C**                     werden.
C**                a+b: wie oben, jedoch bei neuer Datei ohne Kontrollworte
C**                a+s: wie oben, jedoch bei neuer Datei mit Kontrollworten
C**                irm    - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte
C**                         die Datei nicht geoeffnet werden. Typ: INTEGER  
C**   Methode  -   Die entsprechenden C-Routinen zum Oeffnen von Dateien werden
C**                aufgerufen.
C**   Hinweise -   Es koennen nicht mehr als MAX_FILES Dateien gleichzeitig  
C**                offen sein (z.Zt. 100).
C**                Fehlerbotschaften werden auf stderr ausgegeben, es wird 
C**                dabei die C-Funktion strerror genutzt.
C**                Falls das Programm aus einem C-Programm heraus aufgerufen 
C**                werden soll (gilt f. SGI, IBM etc., nicht f. Cray): 
C**                copen(&fdes,infile,modus,&irm,strlen(infile),strlen(modus))
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Die Funktion besitzt den Typ int und liefert bei erfolg-
C**                reicher Beendigung den Wert 0, sonst -1.
C**                Das Modul funktioniert nur auf UNIX Plattformen!
C**   Verfasser-   A.Freyberg, TI12, Januar 1994, geaendert September 1997
C     Aenderung -  bei Ende des Prozesses closeall rufen, um abschliessende
C                  Kontrollwoerter zu schreiben    A. Freyberg, 16.1.2000
C     Geaendert:   R.Johanni, B.Loeser, April 2001
C                  G.Eymann, Juli 2001 (test-weise Portierung IBM SP: _AIX)
C                    Array fdes mit file descriptors wird bei Erstaufruf mit
C                    Nullen belegt.
C                    Nicht vorhandene Funktion sysfs durch statvfs ersetzt.
C                  G.Eymann, 19.09.2001: f. IBM AIX Verwendung der Fkt.
C                    stat & statvfs (anstelle statfs & sysfs)
C                    (Fkt. getvfsbytype ersetzt)
C                  O.Streicher, 27.11.2001: Bugfix (cos4); Auslesen der 
C                    Recordlaenge bei COPEN(), modus "append".
C                  M.Stahlhacke,G.Eymann, 07.12.01: fuer die Modi "a" und "a+"
C                    wird ein identischen Verhalten erzwungen, da f. die Datei
C                    Leserechte vorhanden sein muessen!
C                    Abfrage der file pos. mit our_tell, falls 0 kein seek/read
C                    Als Modus ist Gross- u. Kleinschreibung erlaubt.
C                    Fehlerbehandlung f. getcwd, tmpnam, our_seek, our_read 
C                    Beachte: return-Werte von f(f)seek u. f(f)read verschieden!
C                  G.Eymann, 11.12.01: erweiterte Fehlerausgaben (keine fkt. 
C                    Aenderungen); Fkt. closeall in Modul cclose.c
C                  R.Johanni, Mai 2002; Anpassungen fuer LINUX, Erkennen, ob ein 
C                    File im NFS liegt (LINUX)
C                  H.Weber, Maerz 2004; Automatische Erkennung von Kontrollworten
C                    Umgebungsvariable LIBDWD_FORCE_CONTROLWORDS eingefuehrt
C                  H.Weber, April 2004; Temporaere Dateien sind jetzt im 
C                    Verzeichnis TMPDIR
C                  H.Weber, Juni 2004: Oeffnen leerer Dateien erlaubt.
*/   

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/statfs.h>

#if defined (_AIX)
#include <sys/statvfs.h>
#define  FSTYPSZ  16
#elif defined(__linux__)
#include <sys/vfs.h>
#else
#include <sys/fstyp.h>
#include <sys/fsid.h>
#endif

/* Alle externen Variablen werden hier in copen deklariert, 
   deshalb wird EXTERN hier nicht, in allen anderen C Modulen
   als "extern" definiert
 */

#define EXTERN        

/* allg. Header f. C Funktionen der DWDLIB */ 

#include "libdwd_c.h"

/* Fortran-C-Schnittstelle wird includiert */

#include "fortran_c.h"

/* globales Flag indiziert Erstaufruf */

static int firstcall_copen = 0;

/* Fehlerbehandlung mit Hilfe der Fehlernummer errno */

extern int errno;

int check_cwords( char buffer[], char *daten_art);

int copen( FTN_INT *iun, char infile[], char modus[], FTN_INT *irm, 
	   unsigned long infile_len, unsigned long modus_len)
{

/* Variablendefinitionen */

  int  i, ii, ioerr, oflag;
  long offset;
  mode_t mode;
  char work_dir[MAX_FILENAME_LEN];
  char cinfile[MAX_FILENAME_LEN];
  char *temp_name;
  char cmodus[4];
  unsigned char ctrl[4];
  char buffer[20];
#ifndef __linux__
  char buf[FSTYPSZ];
#endif
  struct flock lock;
  struct statfs  stats;
#ifdef _AIX
  struct stat aixbuf;
  struct statvfs aixvfs;
  int vfslen;
#endif

/* Fehlerzelle vorbesetzen */

  *irm = ioerr = 0;

/* file descriptor bei Erstaufruf mit Null besetzen */

  if (firstcall_copen == 0)
    {
#ifndef __linux__
      memset (buf, 0, sizeof(buf));
#endif
      for (ii = 0; ii < MAX_FILES; ii++) 
	fdes[ii] = 0;
      firstcall_copen = 1;
    }

/* Umspeichern des Modus */
/* Hierbei beachten, dass Modus sowohl kuerzer als auch laenger als
   3 Buchstaben sein darf, entsprechend abschneiden oder mit
   Blanks auffuellen 
 */

  strcpy(cmodus,"   "); /* Vorbesetzen mit 3 Blanks und \0 */
  for (i = 0; i < modus_len && i < 3; i++) 
    cmodus[i] = tolower (modus[i]);

/* nicht belegten Filedeskriptor bestimmen */

  for ( ii=0; ii < MAX_FILES; ii++ ) 
    if (fdes[ii] == 0) break;

  if ( ii >= MAX_FILES )
    {
      fprintf(stderr,"copen  F Mehr als %d Dateien koennen nicht geoeffnet werden!\n",
	      MAX_FILES);
      *irm  = 1;
      return -1;
    }

/* Soll eine scratch-Datei angelegt werden? */

  if ( !(strcmp ( cmodus, "w+t" )) || !(strcmp ( cmodus, "w+v")) )
    {
      temp_name = tempnam ( NULL, "ldwd-" );
      if (temp_name == NULL) 
	{
	  fprintf(stderr,"copen  F Temporaerer Dateiname %s kann nicht generiert werden!\n",
		  dateien[ii]);
	  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	  *irm = 1;
	  return -1;
	}
      if ( strlen (temp_name) >= MAX_FILENAME_LEN )
	{
	  fprintf(stderr,"copen  F Temporaerer Dateiname %s ist zu lang!\n",
		  temp_name);
	  *irm = 1;
	  return -1;
	}
      strncpy ( dateien[ii], temp_name, MAX_FILENAME_LEN );
/* der alte Modus "w+v" wird nun behandelt, wie "w+t" */
      if ( !(strcmp ( cmodus, "w+t" )) ) 
	strncpy ( cmodus, "w+ ", 3);
      if ( !(strcmp ( cmodus, "w+v" )) ) 
	strncpy ( cmodus, "w+ ", 3);
      strncpy ( infile, dateien[ii], infile_len );
      for ( i=strlen(dateien[ii]); i < infile_len; i++ ) 
	infile[i] = ' ';
    }

/* Umspeichern des Dateinamens                                              
   an infile selbst darf nichts geaendert werden, weil das verheerende      
   Auswirkungen auf das FORTRAN-Programm hat!!                              
 */

  else
    {
      for ( i=infile_len; i > 0 ; i-- ) 
	if ( infile[i-1] != ' ' ) break;
     
      if(i > MAX_FILENAME_LEN-1)
	{
	  fprintf(stderr,"copen  F Dateiname zu lang, Maximum ist %d\n",
		  MAX_FILENAME_LEN-1);
	  *irm = 1;
	  return -1;
	}
      strncpy ( cinfile, infile, i );
      cinfile[i] = '\0';
     
/* das folgende statement funktioniert nur bei UNIX ! */
     
      if ( cinfile[0] != '/' )
        {
          if ( getcwd ( work_dir, MAX_FILENAME_LEN ) == NULL )
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. getcwd, Verzeichnis: %s\n",
		      work_dir);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm = 1;
	      return -1;
	    }
          sprintf ( dateien[ii], "%s%c%s", work_dir, '/', cinfile );
        }
      else
        {
	  sprintf ( dateien[ii], "%s", cinfile );
        }

/* Nachschauen, ob es eine Datei im silocache ist, dann nicht oeffnen */
/*
 * Fallunterscheidung IBM AIX - sonst. System, da sysfs nicht vorhanden 
 * beachten: andere if Abfragen evtl. mit nur einer Klammer
 */

#ifdef __linux__
      if (( statfs ( cinfile, &stats )) != -1 )
        {
	  if(stats.f_type == 0x6969) /* RJ: 0x6969 = NFS_SUPER_MAGIC */
#else
#ifndef _AIX
      if (( statfs ( cinfile, &stats, sizeof(struct statfs), 0 )) != -1 )
	{
	  if (( sysfs ( GETFSTYP, stats.f_fstyp, buf )) == -1 )
            {
	      fprintf(stderr,"copen  F Fehler bei Fkt. sysfs, Datei: %s\n",
		      cinfile);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm  = 1;
	      return -1;
            }
#else
      if (( statfs ( cinfile, &stats )) != -1 )
        {
	  if (( statvfs ( cinfile, &aixvfs)) == -1 )
            {
	      fprintf(stderr,"copen  F Fehler bei Fkt. statvfs, Datei: %s\n",
		      cinfile);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm  = 1;
	      return -1;
            }
/* file system type der Struktur umkopieren */
	  vfslen = strlen (aixvfs.f_basetype);
	  if (vfslen > sizeof(buf)) 
	    vfslen = sizeof(buf);
	  strncpy (buf, aixvfs.f_basetype, vfslen);
#endif
	  if ( strncasecmp ( buf, "NFS", 3 ) == 0 )
#endif
            {
	      if ( strstr ( cinfile, "/silocache/" ) != NULL )
                {
		  fprintf(stderr, "copen  F Dateien im Silocache duerfen nicht %s\n",
			  "direkt mit der Funktion copen geoeffnet werden!");
		  fprintf(stderr, "Bitte kopieren Sie die Datei %s erst zurueck!\n",
			  cinfile);
		  *irm  = 1;
		  return -1;
                }
            } 
        }
      else
        {
/* NB: errno 2 ist "No such file or directory" */
	  if ( errno != 2 )
            {
	      fprintf(stderr,"copen  F Fehler bei Fkt. statfs, Datei: %s\n",
		      cinfile);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm  = 1;
	      return -1;
            }
        }              
    }

/* Oeffnen der Datei */
 
  memset (&lock,0,sizeof(struct flock));

/* f. Modi "a" und "a+" wird ident. Verhalten erzwungen */

  if ( cmodus[0] == 'a')
    cmodus[1] = '+';

  if ( !(strncmp( cmodus, "r ", 2 )) ) 
    {
      oflag = O_RDONLY;
      lock.l_type   = F_RDLCK;    
    }
  else if ( !(strncmp( cmodus, "r+", 2 )) )
    {
      oflag = O_RDWR;
      lock.l_type   = F_WRLCK;    
    }
  else if ( !(strncmp( cmodus, "w ", 2 )) )
    {
      oflag = O_WRONLY | O_TRUNC | O_CREAT;
      lock.l_type   = F_WRLCK;    
    }
  else if ( !(strncmp( cmodus, "w+", 2 )) )
    {
      oflag = O_RDWR | O_TRUNC | O_CREAT;
      lock.l_type   = F_WRLCK;    
    }
  else if ( !(strncmp( cmodus, "a ", 2 )) )
    {
      oflag = O_WRONLY | O_CREAT;
      lock.l_type   = F_WRLCK;    
    }
  else if ( !(strncmp( cmodus, "a+", 2 )) )
    {
      oflag = O_RDWR | O_CREAT;
      lock.l_type   = F_WRLCK;    
    }
 
/* Falsche Parametereingabe */
 
  else 
    {
      fprintf (stderr,"copen  F Falscher Wert des Parameters modus: %s\n", 
	       cmodus);
      *irm = 1;
      return -1;
    }
 
/* Oeffnen der Datei */
/* Die Datei wird auch dann mit open geoeffnet, wenn FFIO verwendet wird,
   um ein Lock setzen zu koennen 
 */
 
  mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
  fdes[ii] = open( dateien[ii], oflag, mode );
  if(fdes[ii] == -1)
    {
      fprintf(stderr,"copen  F Datei %s konnte nicht geoeffnet werden!\n", 
	      dateien[ii]);
      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
      *irm  = errno;
      fdes[ii] = 0;
      return -1;
    }

/* Lock setzen */

/* begin_of_comment
   Just comment this part, because it leads to problems on some 
   systems and is necessary only in the DWD environment
  lock.l_whence = 0;
  lock.l_whence = 0; 
  lock.l_start  = 0;       
  lock.l_len    = 0;    

  if ( (fcntl ( fdes[ii], F_SETLK, &lock ) ) == -1 )
    {
      fprintf(stderr,"copen  F Datei %s konnte nicht gesperrt werden!\n", 
	      dateien[ii]);
      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
      *irm  = errno;
      close(fdes[ii]);
      fdes[ii] = 0;
      return -1;
    }
end_of_comment  */

  if ( cmodus[2] == 'b' )
    dat_art[ii] = 'b';
  else if ( cmodus[2] == 's' )
    dat_art[ii] = 's';
  else 
    dat_art[ii] = 'a';

/* Oeffnen fuer stdio/FFIO */

#ifdef FFIO
/* File wird nochmal geoeffnet */
  oflag = oflag & ~O_TRUNC;
  if ( (dateizeiger[ii] = ffopen ( dateien[ii], oflag, mode )) == -1 ) 
#else
/* RJ: If we compile with -ansi on SGI, there is NO prototype defined
       for fdopen in "stdio.h" - thats why there is a cast to FILE* 
 */
    if ( (dateizeiger[ii] = (FILE*) fdopen ( fdes[ii], cmodus ) ) == NULL )
#endif
      {
	fprintf(stderr,"copen  F Datei %s konnte nicht geoeffnet werden!\n", 
		dateien[ii]);
	fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	*irm  = errno;
	close(fdes[ii]);
	fdes[ii] = 0;
	return -1;
      }

/* Beim Lesen einer Datei wird die Art durch check_cwords() festgestellt */

  if ( cmodus[0] == 'r' )
    {
      ioerr = our_read ( dateizeiger[ii], buffer, 20);
      if (ioerr < 0)
	{
	  fprintf(stderr,"copen  F Fehler bei Fkt. our_read, Datei: %s\n",
		  dateien[ii]);
	  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	  *irm = 1;
	  return -1;
	}
/* Ist die Datei leer, so wird die Datenart aus der Umgebung genommen */
      if (ioerr == 0)
	{     
	  if ( getenv( "LIBDWD_FORCE_CONTROLWORDS" ) )
	    dat_art[ii] = 's';
	  else
	    dat_art[ii] = 'b';
	}
      else
	{
	  if ( check_cwords( buffer, &dat_art[ii] ) < 0 ) 
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. check_cwords\n");
	      fprintf(stderr,"Datei %s hat kein bekanntes Format!\n", dateien[ii]);
	      *irm = 1;
	      return -1;
	    }
	  ioerr = our_seek( dateizeiger[ii], SEEK_SET, SEEK_SET );
	  if (ioerr < 0)
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. our_seek, Datei: %s\n",
		      dateien[ii]);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm = 1;
	      return -1;
	    }
	}
    }

/* Beim Schreiben einer neuen Datei entscheidet die Umgebungsvariable
   ueber die Kontrollworte
 */

  if ( cmodus[0] == 'w' && dat_art[ii] == 'a' )
    {
      if ( getenv( "LIBDWD_FORCE_CONTROLWORDS" ) )
	dat_art[ii] = 's';
      else
	dat_art[ii] = 'b';
    }

/* Wird an eine bestehende Datei angehaengt, so wird ihr Typ mit 
   check_cwords() festgestellt.  Ist die Datei leer, wird die neue
   Datei nach Massgabe der Umgebungsvariablen angelegt.
 */

  if ( cmodus[0] == 'a' )
    {
      if ( our_tell( dateizeiger[ii] ) == 0 ) 
	{
	  if ( dat_art[ii] == 'a' )
	    {
	      if ( getenv( "LIBDWD_FORCE_CONTROLWORDS" ) )
		dat_art[ii] = 's';
	      else
		dat_art[ii] = 'b';
	    }
	}
      else
	{
	  ioerr = our_seek( dateizeiger[ii], SEEK_SET, SEEK_SET );
	  if (ioerr < 0)
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. our_seek, Datei: %s\n",
		      dateien[ii]);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm = 1;
	      return -1;
	    }
	  ioerr = our_read( dateizeiger[ii], buffer, 20 );
	  if (ioerr < 0)
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. our_read, Datei: %s\n",
		      dateien[ii]);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm = 1;
	      return -1;
	    }
	  if ( check_cwords( buffer, &dat_art[ii] ) < 0 ) 
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. check_cwords\n");
	      fprintf(stderr,"Datei %s hat kein bekanntes Format!\n", dateien[ii]);
	      *irm = 1;
	      return -1;
	    }
	  our_seek ( dateizeiger[ii], SEEK_SET, SEEK_END );
	  if (ioerr < 0)
	    {
	      fprintf(stderr,"copen  F Fehler bei Fkt. our_seek, Datei: %s\n",
		      dateien[ii]);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	      *irm = 1;
	      return -1;
	    }
	}

/* Der Dateizeiger muss zum Anhaengen an der Stelle stehen,
   die vom Programm uegex erwartet wird.
   Zuerst Test, ob Datei bereits beschrieben, kein seek bei Erstaufruf 
 */

      if (our_tell (dateizeiger[ii]) > 8)
	{
	  offset = dat_art[ii] == 's' ? -8 : 0;
	  errno = 0;
	  our_seek ( dateizeiger[ii], offset, SEEK_END );
	  if (errno != 0)
	    {
	      fprintf(stderr,"copen  W Fehler bei Fkt. our_seek - ignoriert!\n");
	      fprintf(stderr,"Datei: %s\n",dateien[ii]);
	      fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
	    }
       
/* Die Recordlaenge (Groesse: 4 Byte) des letzten Blocks muss
   ausgelesen und an der dafuer vorgesehenen Stelle gespeichert
   werden. Danach setzen wir den Dateizeiger wieder auf die
   urspruengliche Stelle.  
 */

	  if (offset == -8) 
	    { 
	      ioerr = our_read ( dateizeiger[ii], ctrl, 4);
	      if (ioerr <= 0)
		{
		  fprintf(stderr,"copen  F Fehler bei Fkt. our_read, Datei: %s\n",
			  dateien[ii]);
		  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
		  *irm = 1;
		  return -1;
		}
	      prevrecl[ii] = (ctrl[0]<<24)|(ctrl[1]<<16)|(ctrl[2]<<8)|(ctrl[3]);
	      errno = 0;
	      our_seek ( dateizeiger[ii], -4, SEEK_CUR );
	      if (errno != 0)
		{
		  fprintf(stderr,"copen  W Fehler bei Fkt. our_seek - ignoriert!\n");
		  fprintf(stderr,"Datei: %s\n",dateien[ii]);
		  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
		}
	    }
	}
    }

  *iun  = fdes[ii];
  
  return 0;
}
      
      
      
/* check_cwords sucht in buffer[] nach den Kennungen von BUFR,
   GRIB und STRING.  Je nachdem ob Kontrollworte gefunden werden,
   wird daten_art auf 's' oder 'b' gesetzt.  Bei Fehlerfreier 
   Ausfuehrung ist der Rueckgabewert=0.
   H. Weber, Maerz 2004
 */ 

int check_cwords( char buffer[], char *daten_art )
{
  char *ctrl;
  char temp[10];
  int year, month, day, hour;

/* vier nullen sind Indiz fuer ein Kontrollwort */
  if ( buffer[0]==0 && buffer[1]==0 && buffer[2]==0 && buffer[3]==0 )
    {
      *daten_art = 's';
      return 0;
    }
  else 
    {
      ctrl = buffer;
      *daten_art = 'b';
    }
/* Suche nach BUFR, GRIB oder LEER Kennung */
  if (( ctrl[0]=='G' && ctrl[1]=='R' && ctrl[2]=='I' && ctrl[3]=='B') || 
      ( ctrl[0]=='B' && ctrl[1]=='U' && ctrl[2]=='F' && ctrl[3]=='R') ||
      ( ctrl[0]=='L' && ctrl[1]=='E' && ctrl[2]=='E' && ctrl[3]=='R'))
    {
      return 0;
    }
/* Pruefe, ob es ein STRING ist */
  else 
    {
      strxfrm( temp, ctrl, 5 );
      year = atoi(temp);
      strxfrm( temp, &ctrl[4], 3 );
      month = atoi(temp);
      strxfrm( temp, &ctrl[6], 3 );
      day = atoi(temp);
      strxfrm( temp, &ctrl[8], 3 );
      hour = atoi(temp);
      if ( (year > 1850) && (year < 2050) &&
	   (month > 0) && (month < 13) &&
	   (day > 0) && (day < 32) &&
	   (hour > 0) && (hour < 25) ) 
	return 0;
    }
/* Record hat nicht die Kennung GRIB oder BUFR und ist kein STRING! */
  return -1;
}

/* Hilfsfunktion um auf die von copen festgestellte Datenart zugreifen 
   zu koennen.
   H. Weber, Maerz 2004
 */
 
char cget_datart( FTN_INT *iunit )
{
  int ii;
  for ( ii=0; ii<MAX_FILES; ii++ ) 
    {
      if ( fdes[ii]>0 && fdes[ii] == *iunit ) 
	{
	  break;
	}
    }
  return dat_art[ii];
}

/* Hilfsfunktion fuer den direkten Zugriff auf den internen Filehandle
   von copen.  
   H. Weber, Maerz 2004
 */

FILE* cget_fdes( FTN_INT *iunit )
{
  int ii;
  for ( ii=0; ii<MAX_FILES; ii++ ) 
    {
      if ( fdes[ii]>0 && fdes[ii] == *iunit ) 
	{
	  break;
	}
    }
  return dateizeiger[ii];
}
 
