      SUBROUTINE copyBit (isrc,ibs,numbits,idst,ibd)

!**** copyBit  -   UP: Bitweises Umspeichern von ISRC (QUELLE) N. IDST (ZIEL)
!**   AUFRUF   :   CALL copyBit (isrc,ibs,numbits,idst,ibd)
!**   ZWECK    :   Bitweises Umspeichern von ISRC nach IDST
!**   PARAMETER:   ISRC      Umzuspeichernder String (1. Wort)
!**                IBS       1. Umzuspeicherndes Bit in ISRC
!**                          (Zaehlweise von 1 bis IBMAX, Big Endian:
!**                           Most significant bit ist 1, LSB ist IBMAX)
!**                          IBS kann beliebig gross sein (also groesser als IBMAX)
!**                NUMBITS   Anzahl der umzuspeichernden Bits (beliebig gross)
!**                IDST      Ziel, in das zu speichern ist
!**                IBD       1. Bit in IDST, Zaehlweise etc siehe IBS
!**                          IBD kann beliebig gross sein (also groesser als IBMAX)
!**
!**   HINWEIS  :   - DIESES PROGRAMM DARF NICHT ZUM UMSPEICHERN VON INTEGER-
!**                  WERTEN IN CHARAKTERSTRINGS UND UMGEKEHRT VERWENDET WERDEN.
!**                - RUECKWAERTIGE UEBERLAPPUNGEN DES ZIELS BEZUEGLICH DER
!**                  QUELLE FUEHRT ZU FEHLERHAFTER UMSPEICHERUNG.
!**                - FUER AUSREICHENDE DIMENSIONIERUNG IST ZU SORGEN.
!**                - DA IN DEM ZIELBEREICH NUR DIE ANGEGEBENEN BITS ERSETZT
!**                  WERDEN, IST DIESER GGF. VORHER ZU LOESCHEN.
!**   VERFASSER:   R. Johanni, SGI, 03/2001
!**                B. Loeser, TI 14, Fortran-INCLUDE durch #include ersetzt

      IMPLICIT     NONE
#include "machconsts.h"

      INTEGER      isrc(*), ibs, idst(*), ibd, numbits
      INTEGER      iws, ibsl, iwd, ibdl, nbf, nw, nbl, i, ihlp, isave

      IF (numbits <= 0) 
     &RETURN

!.....iws,  Start word in isrc            (Counting from 1)
!.....ibsl, Start bit  in isrc(iws)       (Counting from 1)
!.....iwd,  Start word in idst            (Counting from 1)
!.....ibdl, Start bit  in idst(iwd)       (Counting from 1)
!.....nbf,  Number of bits that go into the first (partial) word of idst
!.....nw,   Number of full words of idst
!.....nbl,  Number of bits that go into the last  (partial) word of idst
      iws           = (ibs-1)/ibmax+1       
      ibsl          = MOD(ibs-1,ibmax)+1 
      iwd           = (ibd-1)/ibmax+1   
      ibdl          = MOD(ibd-1,ibmax)+1 
      nbf           = MIN(ibmax-ibdl+1,numbits) 
      nw            = (numbits-nbf)/ibmax 
      nbl           = numbits-nbf-nw*ibmax      

!.....If we have a partial word at the end, we will overwrite it
!.....and restore the unchanged part later
      IF (nbl > 0)                                         THEN
         nw         = nw+1
         isave      = idst(iwd+nw)
      ENDIF

!.....Care about the first partial word
!.....Shift to the position needed in idst
      ihlp          = ISHFT(isrc(iws),ibsl-ibdl) 

!.....Append second source word if needed 
      IF (nbf > ibmax-ibsl+1)           
     &ihlp          = IOR(ihlp,ISHFT(isrc(iws+1),ibsl-ibdl-ibmax))

!.....Transfer to  idst
      CALL MVBITS (ihlp     ,ibmax-ibdl-nbf+1,nbf, 
     &             idst(iwd),ibmax-ibdl-nbf+1)

!.....Fill remainder of idst
      IF (ibdl < ibsl)                                     THEN
!........Shift isrc(iws+i) to the left, isrc(iws+i+1) to the right
         DO       i = 1, nw
         idst(iwd+i)= IOR(ISHFT(isrc(iws+i),  ibsl-ibdl),
     &                    ISHFT(isrc(iws+i+1),ibsl-ibdl-ibmax))
         ENDDO
      ELSEIF (ibdl == ibsl)                                THEN
!........Copy full words
         DO       i = 1, nw
         idst(iwd+i)= isrc(iws+i)
         ENDDO
      ELSE
!........Shift isrc(iws+i) to the right, isrc(iws+i-1) to the left
         DO       i = 1, nw
         idst(iwd+i)= IOR(ISHFT(isrc(iws+i),  ibsl-ibdl),
     &                    ISHFT(isrc(iws+i-1),ibsl-ibdl+ibmax))
         ENDDO
      ENDIF

      IF (nbl == 0) 
     &RETURN

!.....Restore unchanged part of last partial word in idst
      CALL MVBITS (isave,0,ibmax-nbl,idst(iwd+nw),0)

      END
