      SUBROUTINE   CP2SP ( DS,KJ,KK,KM,KJL,KKL,KML,KP,KN,IRM )

C**** CP2SP    -   Spektrale Koeffizienten nach Entpacken f.Modellstart aufbereiten 
C***               Es ordnet um und de-normiert.
C**   AUFRUF:      CALL CP2SP(DS,KJ,KK,KM,KJL,KKL,KML,KP,KN,IRM)
C**   ZWECK:       DATENSATZ UMORDNEN UND DE-NORMIEREN
C**   PARAMETER:
C**   INPUT:       DS          DATENSATZ MIT SP.HARM.KOEFFIZIENTEN
C**                KJ,KK,KM    PENT.RES.PAR. AUS GRID-DESCR.SECTION
C**                KJL,KKL,KML PENT.RES.PAR. AUS BIN.DATA-SECTION
C**                KP          NORMIERUNGS-POTENZ VON N*(N+1)
C**                KN          ANZAHL UNGEPACKTER DATEN AM ANFANG VON DS
C**   OUTPUT:      DS          ENTHAELT UMGEORDNETE DENORMIERTE WERTE
C**                IRM         0= BEI FEHLERFREIEM ABLAUF
C**   VERFASSER:   W. Wergen - F - AUGUST 90
C*    ------------------------------------------------------------------
C     Methode
C     -------
C     CP2SP uebernimmt die KN genauer gepackten und unnormierten Werte am
C     Anfang von DS  und ordnet sie wie das Modell es erawrtet. Die rest-
C     lichen Koeffizienten werden de-normiert und geordnet. Nach Aufruf
C     enthaelt DS  die Werte wie fuer den Modellstart benoetigt. CP2SP kehrt
C     die Aktionen von SP2CP um.
C     Literatur
C     ---------
C       FM92 GRIB, complex packing
C       ECMWF Rsearch Manual 2
C*    ------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER KJ, KK, KM, KJL, KKL, KML, KP, KN, IRM
      INTEGER J, IS, ISPL, ISP, JM, JEL, JE, JS, JN

      REAL    DS, SPB

      DIMENSION    DS(*)
      DIMENSION    SPB(72)
      IRM        = 0
C*    ------------------------------------------------------------------
C*       1. Test auf Konsistenz der Aufrufparameter
      IF         ( KJL.GT.KJ.OR.KKL.GT.KK.OR.KML.GT.KM)    THEN
      PRINT        1000,KJL,KJ,KKL,KK,KML,KM
 1000 FORMAT     (' CP2SP  - Anrufparameter falsch!',6I10 )
      IRM        = 1000
                                       RETURN
                                       ENDIF
C*    ------------------------------------------------------------------
      IF         ( KJL.LT.1.OR.KKL.LT.1.OR.KML.LT.1)       THEN
      PRINT        2000,KJL,KKL,KML
 2000 FORMAT     (' CP2SP  - KJL, KKL und KML muessen',
     +            ' groesser Null sein!',3I6)
      IRM        = 2000
                                       RETURN
                                       ENDIF
C*    ------------------------------------------------------------------
C*       LAENGE DES ZWISCHENSPEICHERS SPB ABSICHERN
      IF         ( KN .GT. 72 )        THEN
      PRINT        3000, KN
 3000 FORMAT     (' CP2SP  - KN=2*(KJL+1)*(KML+1) =',I4,
     +            ' Z.ZT. NUR LAENGE 72 ERLAUBT.')
      IRM        = 3000
                                       RETURN
                                       END IF
C*    ------------------------------------------------------------------
C*       1.8 Umkopieren der genauer gepackten Werte
      DO 4000  J = 1,KN
      SPB(J)     = DS (J)
 4000 CONTINUE
C*    ------------------------------------------------------------------
C*       2.  Schleife ueber die zonalen Wellenzahlen
      IS         = 0
      ISPL       = 0
      ISP        = 0
      DO 7000 JM = 0,KM
      JEL        = MIN(KJL+JM,KKL)
      JE         = MIN(KJ+JM,KK)
      JS         = JM
C*    ------------------------------------------------------------------
C*       2.2 Einordnung der genauer gepackten Werte
      IF         ( JM.LE.KML)          THEN
      DO 5000 JN = JM,JEL
      DS (ISP+1) = SPB(ISPL+1)
      DS (ISP+2) = SPB(ISPL+2)
      ISPL       = ISPL+2
      ISP        = ISP+2
      JS         = JS+1
 5000 CONTINUE
                                       ENDIF
C*    ------------------------------------------------------------------
C*       2.4 De-normierung der uebrigen Koeffizienten
CDIR$ IVDEP
      DO 6000 JN = JS,JE
      DS (ISP+1) = DS (KN+IS+1)/(JN*(JN+1))**KP
      DS (ISP+2) = DS (KN+IS+2)/(JN*(JN+1))**KP
      IS         = IS+2
      ISP        = ISP+2
 6000 CONTINUE
 7000 CONTINUE
C*    ------------------------------------------------------------------
      END
