/*
C**** crewind  -   Zuruecksetzen des Dateizeigers auf den Anfang einer Datei
C**   Aufruf   :   call crewind ( iun, irm )
C**   Zweck    :   Der Dateizeiger einer Datei wird auf den Dateianfang 
C**                gesetzt.
C**   Parameter:   iun   - Filedeskriptor, wird von dem Programm copen, mit dem
C**                        die Datei eroeffnet werden muss, geliefert.
C**                        Typ: INTEGER
C**                irm   - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte
C**                        nicht zurueckgesetzt werden.
C**   Methode  :   Mit Hilfe der C-Funktion fseek wird an den Anfang gesetzt.
C**   Hinweise :   Die Datei muss mit dem DWDLIB-Programm copen eroeffnet 
C**                worden sein.
C**                Fehlerbotschaften werden auf stderr ausgegeben, wobei die
C**                C-Funktion perror genutzt wird.
C**                Falls das Programm aus einem C-Programm gerufen werden soll:
C**                int crewind_ ( &iun, &irm )
C**                Im Fehlerfall wird der Wert -1, sonst 0 geliefert.
C**   Verfasser:   A.Freyberg, T23, Januar 1994
C     Geaendert:   R.Johanni, B.Loeser, April 2001
C                  G.Eymann, 11.12.01: Fehlerausgaben erweitert
*/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

/*   Filedeskriptoren werden extern definiert                                 */

#define EXTERN extern

#include "libdwd_c.h"

/*   Fortran-C-Schnittstelle wird includiert*/
#include "fortran_c.h"

/*   Fehlerbehandlung mit Hilfe der Fehlernummer errno                        */

extern int errno;

int crewind ( FTN_INT *iun, FTN_INT *irm )
{
 unsigned int  ii;

/*   Fehlerzelle vorbesetzen */

 *irm = 0;

/*   Filedeskriptor bestimmen */

 for ( ii=0; ii<MAX_FILES; ii++ ) 
    if ( fdes[ii]>0 && fdes[ii] == *iun ) break;
 
 if ( ii >= MAX_FILES )
 {
     fprintf (stderr,"crewind  F Filedeskriptor %d existiert nicht!\n", *iun );
     *irm  = 1;
     return -1;
 }

/*   Falls noetig, abschliessendes Kontrollwort schreiben */

 if(lastwrite[ii])
 {
     if( write_control_word(ii,prevrecl[ii],0) )
     {
         
         *irm  = 1;
         return -1;
     }
     lastwrite[ii] = 0;
     prevrecl[ii]  = 0;
 }

 
/*   Zuruecksetzen */

 if  ( our_seek ( dateizeiger[ii], 0L, SEEK_SET ) < 0 )
 {
    fprintf(stderr,"crewind  F Fehler bei Fkt. our_seek, Datei: %s\n",
            dateien[ii]);
    fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
    *irm = errno;
    return -1;
 }

 return 0;
}
