/*
C**** cuegex   -   Rausschreiben eines Datensatzes auf eine uegabe-Datei
C**   Aufruf   :   call cuegex ( iun, ifeld, lflb, irm )
C**   C-Aufruf :   ccuegex ( &iun, ifeld, &lflb, iswap, &irm )
C**   Zweck    :   Ein Datensatz wird auf eine Datei, die Binaerdaten im
C**                Standardformat oder ohne Kontrollworte enthaelt, 
C**                geschrieben.
C**   Parameter:   iun   - Filedeskriptor, wird von dem Programm copen, mit 
C**                        dem die Datei eroeffnet werden muss, geliefert.
C**                        Typ: INTEGER
C**                ifeld - Feld, das die Daten enthaelt. Typ: INTEGER
C**                        RJ: Auch wenn ifeld in C mit char[] deklariert ist
C**                        muss es in Fortran INTEGER sein.
C**                lflb  - Laenge des Datensatzes in Byte. Typ: INTEGER 
C**                irm   - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte 
C**                        kein Datensatz geschrieben werden.
C**                iswap - int (nur in ccuegex )
C**   Methode  :   Mit Hilfe der C-Funktion fwrite wird ein Datensatz  
C**                geschrieben.
C**   Hinweise :   Die Datei muss mit dem DWDLIB-Programm copen eroeffnet 
C**                worden sein.
C**                Fehlerbotschaften werden auf stderr ausgegeben, es wird 
C**                dazu die C-Funktion perror benutzt.
C**                Falls die Routine aus einem C-Programm gerufen werden 
C**                soll: int ccuegex ( &iun, ifeld, &lflb, iswap, &irm )
C**                - empfohlen in C ifeld als char deklarieren
C**                - ist in C ifeld als char deklariert, muss iswap = 0 gesetzt sein
C**                 um auf einer Little-Endian Maschine byte-swapping zu vermeiden,
C**                - ist in C ifeld als int deklariert, muss iswap = 1 gesetzt sein
C**                um auf einer Little-Endian Maschine byte-swapping zu erzwingen
C**                - ist in C ifeld als int deklariert, kann auch
C**                  ccuegex ( &iun, ifeld, &lflb, iswap, &irm ) benutzt werden
C**
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Das Programm liefert im Fehlerfall den Wert -1, sonst 0.
C**   Verfasser:   A.Freyberg, T23, Januar 1994
C     Geaendert:   R.Johanni, B.Loeser, April 2001
C                  G.Eymann, 11.12.01: Fehlerausgaben erweitert (keine fkt. Aend.)
C                  R.Johanni, Mai 2002: Byte-swapping fuer Little-Endian Maschinen
C                  C.Gramalla, 10.02.2006 Byte-swapping fuer Little-Endian
C                                         erweitert (nicht nur fuer __linux__)
C                                         aus C soll ccuegex aufgerufen werden,
C                                         um byte-swapping zu steuern
*/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>

/*   Filedeskriptoren werden extern definiert                                 */

#define EXTERN extern

#include "libdwd_c.h"

/*   Fortran-C-Schnittstelle wird includiert                                  */
#include "fortran_c.h"

/*  Fehlerbehandlung mit Hilfe der Fehlernummer errno                        */

extern int errno;

int cuegex ( FTN_INT *iun, char ifeld[], FTN_INT *lflb, FTN_INT *irm )
{
/* evtl. byte-swapping */
int iswap = 1;
/* Parameter werden durchgegeben */
           ccuegex (iun, ifeld, lflb, iswap, irm );
return 0;
}

int ccuegex ( FTN_INT *iun, char ifeld[], FTN_INT *lflb, int iswap, FTN_INT *irm )
{

/*   Variablendefinitionen */
 int ii;
 long i;
 int itest, le;
 int bswap = 0;
 unsigned char *ctest;

/*   Fehlerzelle vorbesetzen */

 *irm = 0;

/*   ist lflb = 0, sofort zurueckspringen */

 if ( *lflb == 0 ) return 0;

/*   Dateideskriptor bestimmen */

 for ( ii=0; ii<MAX_FILES; ii++ ) 
   if ( fdes[ii]>0 && fdes[ii] == *iun ) 
     break;

 if ( ii >= MAX_FILES )
 {
     fprintf(stderr,"(c)cuegex F Filedeskriptor %d existiert nicht!\n",*iun );
     *irm  = 1;
     return -1;
 }

/* Fuer alle Little-Endian Maschinen muessen die Bytes geswapt werden! */

  bswap=sizeof(FTN_INT);

  if ( bswap >1 ) {
  /* Check if we are running on a little endian machine */
      le = 0;
      itest = 0xaa; /* Magic number */
      ctest = (unsigned char *) &itest;
      if(ctest[0] == 0xaa) le = 1;
      if ( ( iswap == 1 ) && le ) {

          for(i=0;i<*lflb;i+=bswap) {
             int j, k;
             char t;
             j = bswap - 1;
             for(k=0;k<bswap/2;k++) {
                 t = ifeld[k+i]; ifeld[k+i] = ifeld[j+i]; ifeld[j+i] = t;
                 j--;
             }
          }
      } /* else no Byte-swappin */
  } else {
         fprintf(stderr,"(c)cuegex  F Fehler, FORTRAN Integer Laenge %d\n",
                 sizeof(FTN_INT));
         *irm = -1;
         return -1;

  }

/*   Dateiart bestimmen                                                       */

 if ( dat_art[ii] == 's' )
    {

/*   Abschliessendes Kontrollwort fuer letzten Record und neues
     Kontrollwort schreiben */

     if (write_control_word(ii,prevrecl[ii],*lflb)) goto write_err;
     prevrecl[ii] = *lflb;
     lastwrite[ii] = 1;

/*   Datensatz schreiben */

     if( our_write(dateizeiger[ii],ifeld,*lflb)!=*lflb ) goto write_err;

/*   Das abschliessende Kontrollwort fuer diesen Record wird beim
     naechsten Record oder beim Close/Rewind/Backspace geschrieben */

    }
 else
    {

/*   Nur der Datensatz wird geschrieben */

     if( our_write(dateizeiger[ii],ifeld,*lflb)!=*lflb ) goto write_err;

    }
    
 return 0;

/*   Fehlerbehandlung */

 write_err:
     fprintf(stderr,"(c)cuegex  F Fehler beim Schreiben, Datei: %s\n",
             dateien[ii]);
     fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
     *irm = 1;
     return -1;
}

/*********************************************************************
 * write_control_word: 
 * Interne Routine zum Schreiben des Kontrollworts,
 * liefert 0 im Erfolgsfall und -1 bei Fehler zurueck
 *********************************************************************/

int write_control_word(int ii, int recl1, int recl2)
{
 unsigned char ctrl[8];

/*   Umwandlung der Recordlaenge in Bytes unabhaengig von
        Wortlaenge und Byte-Reihenfolge */

 ctrl[0] = ((recl1)>>24) & 0xff;
 ctrl[1] = ((recl1)>>16) & 0xff;
 ctrl[2] = ((recl1)>> 8) & 0xff;
 ctrl[3] =  (recl1)      & 0xff;
 ctrl[4] = ((recl2)>>24) & 0xff;
 ctrl[5] = ((recl2)>>16) & 0xff;
 ctrl[6] = ((recl2)>> 8) & 0xff;
 ctrl[7] =  (recl2)      & 0xff;

 if ( our_write(dateizeiger[ii],ctrl,8) != 8 ) 
 {
    fprintf(stderr,"(c)cuegex  F Fehler beim Schreiben Kontrollwort %s\n"
                  ,"(Anzahl byte != 8)");
    fprintf(stderr,"Datei: %s\n",dateien[ii]);
    fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
    return -1;
 }

 return 0;
}
