/*
C**** cuegin   -   Einlesen eines Datensatzes von einer uegabe-Datei
C**   Aufruf   :   call cuegin ( iun, lfdb, ifeld, lflb, irm )
C**   C-Aufruf :   ccuegin ( &iun, &lfdb, ifeld, &lflb, iswap, &irm )
C**   Zweck    :   Ein Datensatz wird von einer Datei, die Binaerdaten im
C**                Standardformat enthaelt, gelesen.
C**   Parameter:   iun   - Filedeskriptor, wird von dem Programm copen, mit 
C**                        dem die Datei eroeffnet werden muss, geliefert.
C**                        Typ: INTEGER
C**                lfdb  - Laenge des Feldes ifeld, das die Daten aufnehmen 
C**                        soll, in Bytes. Typ: INTEGER
C**                ifeld - Feld, das die Daten aufnehmen soll, es ist aus-
C**                        reichend zu dimensionieren. Typ: INTEGER
C**                lflb  - Laenge des gelesenen Datensatzes in Byte, ist das
C**                        Dateiende erreicht oder ein Fehler aufgetreten,
C**                        ist lflb=0. Typ: INTEGER
C**                irm   - Fehlerrueckmeldezelle, ist irm ungleich 0, konnte 
C**                        kein Datensatz gelesen werden.
C**                iswap - int (nur in ccuegin )
C**   Methode  :   Mit Hilfe der C-Funktion fread wird ein Datensatz gelesen. 
C**   Hinweise :   Die Datei muss mit dem DWDLIB-Programm copen eroeffnet 
C**                worden sein.
C**                Fehlerbotschaften werden auf stderr ausgegeben, es wird 
C**                dabei die C-Funktion perror benutzt.
C**                Falls die Routine aus einem C-Programm gerufen werden 
C**                soll: int ccuegin ( &iun, &lfdb, ifeld, &lflb, &irm, iswap )
C**                - empfohlen in C ifeld als char deklarieren
C**                - ist in C ifeld als char deklariert, muss iswap = 0 gesetzt sein
C**                 um auf einer Little-Endian Maschine byte-swapping zu vermeiden, 
C**                - ist in C ifeld als int deklariert, muss iswap = 1 gesetzt sein
C**                um auf einer Little-Endian Maschine byte-swapping zu erzwingen
C**                - ist in C ifeld als int deklariert, kann auch
C**                  cuegin (&iun, &lfdb, ifeld, &lflb, &irm) benutzt werden
C**                 
C**                Ausserdem muss die Datei fortran_c.h includiert werden:
C**                #include "fortran_c.h"
C**                Die Funktion liefert bei erfolgreicher Beendigung den 
C**                Wert 0, im Fehlerfall -1, wenn lfdb zu klein ist, wird
C**                -2 zurueckgegeben.
C**   Hinweis zur Fehlerbehandlung (RJ):
C**                Wenn lfdb zu klein ist wird dennoch die erforderliche 
C**                Laenge in lflb zurueckgegeben und die Datei ist NACH
C**                dem zu lesenden Record positioniert. irm ist in diesem
C**                Fall -2, es wird keine Fehlermeldung ausgegeben, da 
C**                dieser Fall beabsichtigt sein kann, um die Laenge des 
C**                naechsten records zu erfahren.
C**   Verfasser:   A.Freyberg, T23, Januar 1994
C     Geaendert:
C                  G.Eymann, 11.12.01: Fehlerausgaben erweitert
C                  R.Johanni, Mai 02: Byte-swappin fuer Little-Endian
C                                     Maschinen eingebaut (LINUX)
C                  G.Eymann,M.Stahlhacke,05.02.2003: zusaetzl. Kennung BUFR 
C                  H.Weber, 06.04.2004: Kennung LEER
C                  H.Weber, 20.04.2004: Fehler in LEER Erkennung
C                  C.Gramalla, 10.02.2006 Byte-swapping fuer Little-Endian
C                                        erweitert (nicht nur fuer __linux__)
C                                        aus C soll ccuegin aufgerufen werden,
C                                        um byte-swapping zu steuern
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

/* Filedeskriptoren werden extern definiert */
#define EXTERN extern

#include "libdwd_c.h"

/* Fortran-C-Schnittstelle wird includiert */
#include "fortran_c.h"

/* Fehlerbehandlung mit Hilfe der Fehlernummer errno */

extern int errno;

int cuegin ( FTN_INT *iun, FTN_INT *lfdb, char ifeld[], FTN_INT *lflb,
             FTN_INT *irm )
{
/* evtl. byte-swapping */
int iswap = 1;
/* Parameter werden durchgegeben */
           ccuegin (iun, lfdb, ifeld, lflb, iswap, irm );
return 0;
}



int ccuegin ( FTN_INT *iun, FTN_INT *lfdb, char ifeld[], FTN_INT *lflb,
             int iswap, FTN_INT *irm )
{

/* Variablendefinitionen */

  long ii, recl;
  long i;
  int itest, le;
  int bswap = 0;
  unsigned char ctrl[8];
  unsigned char *ctest;
/* Fehlerzelle vorbesetzen */

  *irm = 0;

/* Wir setzen die Anzahl der eingelesenen Bytes auf 0 */

  *lflb = 0;

/* Filedeskriptor bestimmen */

  for ( ii=0; ii<MAX_FILES; ii++ ) 
    if ( fdes[ii]>0 && fdes[ii] == *iun ) 
      break;

  if ( ii >= MAX_FILES )
    {
      fprintf (stderr,"(c)cuegin F Filedeskriptor %d existiert nicht!\n", *iun );
      *irm  = -1;
      return -1;
    }

/* Einlesen eines Records */

/* Bestimmen der Dateiart */

  if ( dat_art[ii] == 'b' )
    {

/* Einlesen der ersten 8 Bytes des Records, die die Kennung GRIB und die 
   Laenge des GRIBs enthalten muessen. 
   Falls an dieser Stelle das read fehlschlaegt, nehmen wir an, 
   dass das Dateiende erreicht ist. 
 */
      if( our_read( dateizeiger[ii], &ctrl[0], 4 ) != 4 ) return 0;

      if(( ctrl[0]=='G' && ctrl[1]=='R' && ctrl[2]=='I' && ctrl[3]=='B') || 
	 ( ctrl[0]=='B' && ctrl[1]=='U' && ctrl[2]=='F' && ctrl[3]=='R') )
        {
/* OK, GRIB oder BUFR */
	  if( our_read( dateizeiger[ii], &ctrl[4], 4 ) != 4 ) return 0;
/* Berechnen der Grib Laenge als Integer Zahl, unabhaengig von
   Wortlaenge und Bytereihenfolge 
 */
	  recl = (ctrl[4]<<16)|(ctrl[5]<<8)|(ctrl[6]);

/* Zurueckpositionieren um die 8 gelesenen Bytes */

	  if( our_seek( dateizeiger[ii], -8, SEEK_CUR) < 0 ) goto seek_err;
	}
      else if ( ctrl[0]=='L' && ctrl[1]=='E' && ctrl[2]=='E' && ctrl[3]=='R')
	{
/* OK, LEER */
	  recl = 4;

/* Zurueckpositionieren um die 4 gelesenen Bytes */

	  if( our_seek( dateizeiger[ii], -4, SEEK_CUR) < 0 ) goto seek_err;
	}
      else
	{
/* Wir nehmen an, da das Dateiende erreicht ist */
	  return 0;
        }

    }
  else
    {

/* Einlesen der ersten 8 Bytes des Records, die die Laengeninformationen 
   fuer vorhergehenden und aktuellen Record enthalten. 
*/
      
      if( our_read( dateizeiger[ii], ctrl, 8 ) != 8 ) goto read_err;

/* Berechnen der Record-Laenge als Integer Zahl, unabhaengig von
   Wortlaenge und Bytereihenfolge 
 */

      recl = (ctrl[4]<<24)|(ctrl[5]<<16)|(ctrl[6]<<8)|(ctrl[7]);
      *lflb = recl;

/* Dateiende wenn recl == 0 */

      if (recl == 0) return 0;

    }
  
/* Test, ob ausreichend dimensioniert */

  *lflb = recl;
  if ( recl > *lfdb )
    {
/* Record ueberspringen, Return mit irm = -2 */
      if( our_seek( dateizeiger[ii], recl, SEEK_CUR) < 0 ) goto seek_err;
      *irm  = -2;
      return -2;
    }

/* Lesen */

  if ( our_read(dateizeiger[ii], ifeld, recl) != recl) goto read_err;
 
/* Fuer alle Little-Endian Maschinen muessen die Bytes geswapt werden! */

  bswap=sizeof(FTN_INT);

  if ( bswap >1 ) {
  /* Check if we are running on a little endian machine */
      le = 0;
      itest = 0xaa; /* Magic number */
      ctest = (unsigned char *) &itest;
      if(ctest[0] == 0xaa) le = 1;
      if ( ( iswap == 1 ) && le ) {

          for(i=0;i<recl;i+=bswap) {
             int j, k;
             char t;
             j = bswap - 1;
             for(k=0;k<bswap/2;k++) {
                 t = ifeld[k+i]; ifeld[k+i] = ifeld[j+i]; ifeld[j+i] = t;
                 j--;
             }
          }
      } /* else no Byte-swappin */
  } else {
         fprintf(stderr,"(c)cuegin  F Fehler, FORTRAN Integer Laenge %d\n",
                 sizeof(FTN_INT));
         *irm = -1;
         return -1;

  }
  return 0;

/* Fehlerbehandlung */

 seek_err:
  fprintf(stderr,"(c)cuegin  F Fehler beim Positionieren, Datei: %s",
	  dateien[ii]);
  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
  *irm = -1; 
  return -1;
  
 read_err:
  fprintf(stderr,"(c)cuegin  F Fehler beim Lesen, Datei: %s\n",
	  dateien[ii]);
  fprintf(stderr,"error %d: %s\n",errno,strerror(errno));
  *irm = -1;
  return -1;
}
