      SUBROUTINE   FCRACK1 (IAR1, IABDB, UNDEF, ISCAL, REFMIN, IBMJ,
     +                      IBMAP, NRGRP, NRBIT, AR2, IRM)

C**** FCRACK1  -   ENTZERREN EINES KOMPRIMIERTEN FELDES (GRIB-CODE)
C**   AUFRUF   :   CALL FCRACK1 (IAR1, IABDB, UNDEF, ISCAL, REFMIN, IBMJ, IBMAP,
C**               +              NRGRP, NRBIT, AR2, IRM)
C**   ZWECK    :   ZERLEGEN EINES IM GRIB-CODE GEPACKTEN FELDES IN EIN FELD
C**                MIT FLOATING-WERTEN; BEI VORLIEGEN EINER BITMAP WERDEN
C**                UNDEFINIERTE PUNKTE MIT UNDEF BESETZT
C**   PARAMETER:
C**     INPUT  :   IAR1      EINGABEFELD, GEPACKT
C**                IABDB     ANFANGSADRESSE=1.BYTE MIT DEM DER DATENSATZ BEGINNT
C**                UNDEF     WERT, MIT DEM UNDEFINIERTE GITTERPUNKTE BELEGT
C**                          WERDEN (I.A. -1E10)
C**                ISCAL     EXPONENT (ZUR BASIS 2) D. SKAL.FAKTORS
C**                REFMIN    REFERENZWERT(MINIMUM) FUER DIE UMRECHNUNG
C**                IBMJ      =0, WENN KEINE BITMAP VORLIEGT (ALLE PUNKTE DEFIN.)
C**                          =1, WENN BITMAP VORLIEGT (IN IBMAP)
C**                IBMAP     BITMAP (ZU LIEFERN, WENN IBMJ=1, SONST DUMMY)
C**                NRGRP     ANZAHL DER GITTERPUNKTE VON IAR1 (GESAMT)
C**                NRBIT     ANZAHL BITS JE GITTERPUNKT
C**                          - DAS PROGRAMM LAEUFT AUF DER CRAY Z.ZT. NUR MIT
C**                          8/16/32 BITS JE WERT(=NRBIT) VEKTORISIERT -
C**     OUTPUT :   AR2       AUSGABEFELD (ENTZERRT, FLOATING);
C**                          DARF IAR1 NICHT UEBERLAPPEN (DIMENS.: >= NRGRP)
C**                IRM       FEHLER, WENN UNGL. 0
C**   VERFASSER:   G.BAGINSKI / A.ZALEWSKY MRZ3 / 10/88
C     MODIFICATION: CRAY WEGEN NEUEM BANKTYP KWB02, 1-GRAD-AUFLOESUNG DER
C                        WASSERTEMPERATUREN, --> LDSMX = 80000
C     MODIFICATION: CRAY WEGEN NEUEM BANKTYP GME  , LDSMX = 205.000;    03.11.97
C.....MODIFICATION: CRAY, ZWISCHENSPEICHER IARZW, IBLZW MITTELS AUFRUF-PARAMETER 
C.........................NRGRP DIMENSIONIEREN..........................21.04.98
C.........................LETZTE AENDERUNG FUER OLDC ZURUECKGENOMMEN....04.06.98
C.........................Anpassung FUJITSU, CD4 und NOS/VE-Zweig raus..17.05.00
C     FORMEL FUER DIE UMRECHNUNG JE WERT:
C         U      = (IS*ZSCAL) + REF
C     MIT:U      = FLOATING-WERT
C         IS     = SKALIERTER WERT
C       ZSCAL    = SKALIERUNGSFAKTOR
C        REF     = REFERENZWERT (MINIMUM)
C*    ------------------------------------------------------------------
C*    GEAENDERT:   R.Johanni, SGI, 09/00, Fortran 90, Einfuehrung "machconsts.h"
C*                            Ueberfluessige Parameter raus, #ifdef(CRAY) raus
C*                 R.Johanni, SGI, 03/00, Sonderfaelle fuer NRBIT==8 und NRBIT==16

      IMPLICIT NONE

!     Uebergabeparameter:

      INTEGER, INTENT(IN)  :: IAR1(*)
      INTEGER, INTENT(IN)  :: IABDB
      REAL,    INTENT(IN)  :: UNDEF
      INTEGER, INTENT(IN)  :: ISCAL
      REAL,    INTENT(IN)  :: REFMIN
      INTEGER, INTENT(IN)  :: IBMJ
      INTEGER, INTENT(IN)  :: IBMAP(*)
      INTEGER, INTENT(IN)  :: NRGRP
      INTEGER, INTENT(IN)  :: NRBIT
      REAL,    INTENT(OUT) :: AR2(*)
      INTEGER, INTENT(OUT) :: IRM


      INTEGER IANFB, IAP, IND, IZW, IZW1, IZW2, INB, IBMSK, J 

      REAL    ZSCAL

#include "machconsts.h"

!     Variablen fuer Behandlung der Sonderfaelle

      INTEGER, PARAMETER :: MAXSTRIP = 4096

      INTEGER IZWV(MAXSTRIP), IPCK(MAXSTRIP), JSTART, JLEN

      INTEGER MSK8, MSK16

      DATA MSK8, MSK16 / Z'FF', Z'FFFF' /


!     Einige Fortran Compiler moegen keinen Ausdruck der Form ISHFT(I,-X)
!     mit X > 32, wenn Integers nur 32 Bit weit sind.
!     Die folgenden Parameter-Deklarationen verhindern diese
!     Fehlermeldungen auf Maschinen mit 32-Bit Integers.
!     Man beachte, dass diese Parameter nur im Zweig IBMAX==64 benutzt werden!

      INTEGER, PARAMETER :: I56 = IBMAX- 8
      INTEGER, PARAMETER :: I48 = IBMAX-16
      INTEGER, PARAMETER :: I40 = IBMAX-24
      INTEGER, PARAMETER :: I32 = IBMAX-32



!     INITIALISIERUNG

      IRM        = 0
      ZSCAL      = 2.**ISCAL

!     Abfrage auf Sonderfall

      IF(IBMJ.NE.1 .AND. (NRBIT==8 .OR. NRBIT==16) .AND.
     &   (IBMAX==32 .OR. IBMAX==64) ) GOTO 1000

      INB        = 0                ! Zaehler fuer Worte in IBMAP, zaehlt ab 1
      IBMSK      = 0                ! Bitmaske fuer aktuelles Bit in IBMAP


      ! IANFB: Anfangsbit im Datensatz, zaehlt ab 0 und von links
      ! IND:   Zaehler fuer Worte in IAR, zaehlt ab 1
      ! IAP:   Nummer des Bits unmittelbar vor dem zu entpackenden Teil,
      !        zaehlt ab 0 und von rechts

      IANFB = (IABDB-1)*IBYTE
      IND   = IANFB/IBMAX+1
      IAP   = IBMAX - MOD(IANFB,IBMAX)

      DO J=1,NRGRP

!        Falls mit Bitmap, unbesetzte Werte auf UNDEF setzen

         IF ( IBMJ.EQ.1) THEN
            IBMSK = ISHFT(IBMSK,-1)
            IF ( IBMSK .EQ. 0 ) THEN
!              NEUES BITMAP-WORT BEREITSTELLEN
               INB   = INB + 1
               IBMSK = ISHFT(1,IBMAX-1)
            ENDIF
!           ABFRAGE, OB GI.PUNKT UNDEFINIERT
            IF(IAND(IBMAP(INB),IBMSK).EQ.0 ) THEN
               AR2(J) = UNDEF
               CYCLE
            ENDIF
         ENDIF

         IF ( IAP >= NRBIT ) THEN

!           Bits aus 1 Wort holen

            IZW  = IBITS(IAR1(IND),IAP-NRBIT,NRBIT)

         ELSE

!           Bits aus 2 Worten holen

            IZW1 = IBITS(IAR1(IND),0,IAP)
            IZW2 = IBITS(IAR1(IND+1),IBMAX+IAP-NRBIT,NRBIT-IAP)
            IZW  = IOR(ISHFT(IZW1,NRBIT-IAP),IZW2)

         ENDIF

!        Update von IAP und IND

         IAP = IAP - NRBIT
         IF ( IAP <= 0 ) THEN
            IAP = IAP + IBMAX
            IND = IND + 1
         ENDIF

         AR2(J)   = REFMIN + IZW*ZSCAL

      ENDDO

      RETURN

 1000 CONTINUE

!     Behandlung der Sonderfaelle fuer 8 und 16 Bit

      DO JSTART = 1, NRGRP, MAXSTRIP  ! Loop fuer Stripmining

         JLEN = MIN(NRGRP-JSTART+1,MAXSTRIP)

         IF (NRBIT==16 .AND. IBMAX==32) THEN

            CALL SPUMB(IAR1,IABDB+(JSTART-1)*2,JLEN*2,1,IPCK,IRM)

            DO J=1,(JLEN+1)/2
               IZWV(2*J-1) = IAND(ISHFT(IPCK(J), -16),MSK16)
               IZWV(2*J)   = IAND(      IPCK(J)      ,MSK16)
            ENDDO

         ELSEIF (NRBIT==16 .AND. IBMAX==64) THEN

            CALL SPUMB(IAR1,IABDB+(JSTART-1)*2,JLEN*2,1,IPCK,IRM)

            DO J=1,(JLEN+3)/4
               IZWV(4*J-3) = IAND(ISHFT(IPCK(J),-I48),MSK16)
               IZWV(4*J-2) = IAND(ISHFT(IPCK(J),-I32),MSK16)
               IZWV(4*J-1) = IAND(ISHFT(IPCK(J), -16),MSK16)
               IZWV(4*J)   = IAND(      IPCK(J)      ,MSK16)
            ENDDO

         ELSEIF (NRBIT==8 .AND. IBMAX==32) THEN

            CALL SPUMB(IAR1,IABDB+(JSTART-1),JLEN,1,IPCK,IRM)

            DO J=1,(JLEN+3)/4
               IZWV(4*J-3) = IAND(ISHFT(IPCK(J), -24),MSK8)
               IZWV(4*J-2) = IAND(ISHFT(IPCK(J), -16),MSK8)
               IZWV(4*J-1) = IAND(ISHFT(IPCK(J),  -8),MSK8)
               IZWV(4*J)   = IAND(      IPCK(J)      ,MSK8)
            ENDDO

         ELSEIF (NRBIT==8 .AND. IBMAX==64) THEN

            CALL SPUMB(IAR1,IABDB+(JSTART-1),JLEN,1,IPCK,IRM)

            DO J=1,(JLEN+7)/8
               IZWV(8*J-7) = IAND(ISHFT(IPCK(J),-I56),MSK8)
               IZWV(8*J-6) = IAND(ISHFT(IPCK(J),-I48),MSK8)
               IZWV(8*J-5) = IAND(ISHFT(IPCK(J),-I40),MSK8)
               IZWV(8*J-4) = IAND(ISHFT(IPCK(J),-I32),MSK8)
               IZWV(8*J-3) = IAND(ISHFT(IPCK(J), -24),MSK8)
               IZWV(8*J-2) = IAND(ISHFT(IPCK(J), -16),MSK8)
               IZWV(8*J-1) = IAND(ISHFT(IPCK(J),  -8),MSK8)
               IZWV(8*J)   = IAND(      IPCK(J)      ,MSK8)
            ENDDO

         ENDIF

         DO J = 1, JLEN
            AR2(J+JSTART-1)   = REFMIN + IZWV(J)*ZSCAL
         ENDDO

      ENDDO

      END
