      INTEGER FUNCTION IREFTS (REFW)

C**** IREFTS   -   UP: REFERENZWERT (GRIB-CODE) HEXADEZIMAL UMSETZEN
C**   AUFRUF   :   ISEDEM = IREFTS (REFW)
C**   ZWECK    :   GRIB-REFERENZWERT HEXADEZIMAL (32 BIT) UMSETZEN :
C**                (REFerenzwert_To_Sedezimal); UMGEKEHRT: FUNCTION REFSTF
C**   INPUT    :   REFW      UMZUSETZENDER FL.POINT-WERT
C**                          (SIEHE HINWEIS !)
C**   OUTPUT   :   ISEDEM    32 BIT RECHTSBUENDIG, DAVON
C**                            EXPONENT HEXADEZ. (8 BIT)
C**                            MANTISSE IN HEXADEZ.DARST. (24 BIT)
C**
C**              Falls REFW wegen Overflow nicht darstellbar ist
C**              (nur moeglich bei 64-bit REAL Zahlen), wird -1
C**              zurueckgegeben.
C**
C**   HINWEIS:   FUER NOS/VE - VERSION GILT:
C**              REFERENZWERTE BIS 16777216./-16777216. (2**24=16777216)
C**              MIT IREFTS UND UMKEHRFUNKTION REFSTF 1:1 UMSETZBAR
C**              FUER MFC-VERSION GILT:
C**              REFERENZWERTE NUR BIS 16777040./-16777040. MIT IREFTS UND
C**              UMKEHRFUNKTION REFSTF 1:1 UMSETZBAR
C**
C**   VERFASSER: EZMW 1985 /ANGEP.: G.BAGINSKI /MRZ3/ 11/88
C                LAST MODIF.: 02/1994 (A.ZALEWSKY)
C     MODIF.:    A.ZALEWSKY, TI12,                                  06.04.2000
C                auf Wunsch von FE (Schaettler,Majewski) geaendert:
C                32-BIT-RECHNER: 39<=IEXP=<101,
C                64-BIT-RECHNER:  0<=IEXP=<127;
C                nur noch #ifdef(CRAY),#else;
C                EXTERNAL SHIFT;
C
C                R. Johanni, SGI, 10/2000:
C                Numerik komplett geaendert, liefert jetzt auch in Grenzsituationen
C                (um Potenzen von 16) den exakten Wert, Kontrolle auf Overflow
 
C.....INITIALISIERUNG

      IMPLICIT NONE

      REAL        REFW, REF
      INTEGER     ISIGN, IEXP, IMANT

      REF                    = REFW
      
C.....VORZEICHEN
      ISIGN                  = 0
      IF (REF .LT. 0.)                                     THEN     
         ISIGN               = 128
         REF                 = -REF
      ENDIF
C.....EXPONENT
      IF (REF .GT. 0.0)                                    THEN

         ! Exakte Suche von Exponent und Mantisse ohne die
         ! zugrunde liegende Real-Zahl-Darstellung (z.B. IEEE)
         ! auszunutzen. Wir gehen davon aus, dass Zweierpotenzen
         ! immer exakt als Realzahlen dargestellt werden
         ! und Multiplikationen mit Zweierpotenzen ebenfalls
         ! exakt durchgefuehrt werden (naemlich nur als
         ! Manipulation des Exponenten)
         ! NB: 16777216 == 2**24

         ! Wir multiplizieren (oder dividieren) REF nun solange
         ! mit (durch) 16, bis gilt:
         ! 16777216/16 <= REF < 16777216
         ! Hierbei sollten KEINERLEI Rundungsfehler auftreten.
         ! REF ist dann bereits die gewuenschte Mantisse,
         ! das Umwandeln in eine Integer Zahl muss durch Abschneiden
         ! (nicht durch Runden!!) passieren

         IEXP = 70

         IF ( REF .LT. 16777216.0 ) THEN

            DO WHILE ( REF*16 .LT. 16777216.0 .AND. IEXP .GT. 0 )
               IEXP = IEXP-1
               REF  = REF*16.0
            ENDDO

            ! Falls IEXP an dieser Stelle 0 ist, liegt ein Underflow
            ! vor, das ist aber kein Problem, das Ergebniss wird
            ! (je nach Wert von REF) als unnormalisierte Zahl oder
            ! als 0 codiert

         ELSE

            DO WHILE ( REF .GE. 16777216.0 .AND. IEXP .LE. 127)
               IEXP = IEXP+1
               REF  = REF*0.0625
            ENDDO

            ! Falls IEXP an dieser Stelle 128 ist, liegt ein Overflow
            ! vor, die Zahl kann nicht in der gewuenschten Weise
            ! dargestellet werden

            IF ( IEXP .EQ. 128 ) THEN
               IREFTS = -1
               RETURN
            ENDIF

         ENDIF

         IMANT = REF ! Muss abgeschnitten werden, kein NINT

      ELSE
         IEXP                = 0
         IMANT               = 0
      ENDIF

C.....VORZEICHEN AUF EXPONENT ADDIEREN UND ERGEBNIS ERZEUGEN
      IEXP                  = IEXP+ISIGN
      IREFTS                = IOR(ISHFT(IEXP,24),IMANT)

      RETURN
      END
