      SUBROUTINE   packGRIB1Daten (AR1, LAR1, FAKT, UNDEF, IBMJ, 
     &                     IBMAP, NRBIT, IABDB, IAR2, ISCAL, REFMIN,
     &                      NRGB, IRM)

C**** packGRIB1Daten   -   PACKEN EINES FLOATING-FELDES FUER GRIB-CODE
C**   AUFRUF   :   CALL packGRIB1Daten (AR1, LAR1, FAKT, UNDEF, IBMJ, IBMAP, 
C**               +             NRBIT, IABDB,IAR2, ISCAL, REFMIN, NRGB, IRM)
C**   ZWECK    :   FELD BELIEBIGER LAENGE MIT FLOATING-WERTEN FUER
C**                GRIB-CODE PACKEN
C**   PARAMETER:
C**     INPUT  :   AR1       EINGABEFELD (UNGEPACKT, FLOATING)
C**                LAR1      LAENGE DES EINGABEFELDES AR1 (IN WORTEN)
C**                FAKT      FAKTOR, MIT DEM ALLE WERTE IN AR1 ZU
C**                          MULTIPLIZIEREN SIND
C**                UNDEF     WERT EINES UNDEFINIERTEN PUNKTES
C**                IBMJ      =-1, WENN ALLE PUNKTE DEFINIERT
C**                          = 1, WENN BITMAP VORLIEGT (IN IBMAP)
C**                IBMAP     BITMAP (WENN IBMJ=1, SONST DUMMY)
C**                NRBIT     ANZAHL BITS JE GITTERPUNKT (<= 48) (S.U.HINWEIS!)
C**                          - DAS PROGRAMM LAEUFT AUF DER CRAY Z.ZT. NUR MIT
C**                          8/16/32 BITS JE WERT(=NRBIT) VEKTORISIERT -
C**                IABDB     ADRESSE DES 1.BYTE IN IAR2, AB DEM DER KOMPRIM.
C**                          DATENSATZ ZU SPEICHERN IST
C**     OUTPUT :   IAR2      AUSGABEFELD, GEPACKT
C**                ISCAL     EXPONENT (BASIS 2) D. SKAL.FAKTORS
C**                REFMIN    REFERENZWERT (=MINIMUM)
C**                NRGB      ANZAHL BESETZTER GITTERPUNKTE
C**                IRM       RUECKMELDUNG (0= KEIN FEHLER)
C**   METHODE  :   FELD AUF EXTREMA UNTERSUCHEN.
C**                AUS DER ABWEICHUNG (MAX-MIN) UND DEN JE WERT ZU VERFUEGUNG
C**                STEHENDEN BITS WIRD DER SKALIERUNGSFAKTOR FUER JEDES AKTUELLE
C**                FELD BERECHNET; SKALIERUNGSFAKTOR UND REFERENZWERT (=MINIM.)
C**                WERDEN MIT JEDEM FELD GESPEICHERT.
C**   VERFASSER:   G. BAGINSKI/A. ZALEWSKY,MRZ3B (10/88), CRAY-OPTIM.7/90
C*                 OTT(17.4.91)IS=NINT(...) RUNDUNG LETZTES BIT
C*                 SEIT 26.7.91 MIT VECTOR_FORTRAN UEBERSETZT. OTT
C*    ------------------------------------------------------------------
C     FORMEL  FUER DIE UMRECHNUNG JE WERT (ENTSPRECHEND GRIB-CODE):
C           IS   = NINT ((U - AMIN) / ZSCAL)
C     MIT:  IS   = SKALIERTER WERT, INTEGER, LETZTES BIT GERUNDET.
C           U    = UNSKALIERTER WERT, REAL
C           AMIN = REFERENZWERT (=MINIMUM)
C           ZSCAL= SKALIERUNG
C        -----------------------------------------------
C        AENDERUNG: 15.06.1994 VON ARZ3ZAL/TE23FRE
C        IBP ENTFERNT, BRINGT KEINE BESSERE GENAUIGKEIT;
C        WEGEN FEHLER BEI DIFF = 128
C        STATT DESSEN:
C        IN      = INT(ZW + 1.E-13) + 1
C        ------------------------------------------------
C*    ------------------------------------------------------------------
C     INITIALISIERUNG
C.....MODIFICATION: NUR FUER CRAY, LDSMX = 205000; ARZ3ZAL..............25.11.97
C.....MODIFICATION: CRAY, ZWISCHENSPEICHER IAR, IARZW WIRD UEBER LAR1
C.........................DIMENSIONIERT.................................21.04.98
C.....FUER OLDC LETZTE AENDERUNG WIEDER ZURUECKGENOMMEN.................04.06.98
C.....MODIFICATION: nur noch #ifdef(CRAY,OLDC;TEST);#else
C...................NEU: IN = INT(ZW+2.**(-NRBIT))+1..(BOE/OTT/ZAL)...21.02.2000
C*    GEAENDERT:   R.Johanni, SGI, 09/00, Fortran 90, Einfuehrung "machconsts.h"
C*                            Ueberfluessige Parameter raus, #ifdef(CRAY) raus
C*                            Zusaetzlicher Parameter FAKT spart Zwischenfeld
C*                            in GRBEX1
C*                 R.Johanni, SGI, 03/00, Sonderfaelle fuer NRBIT==8 und NRBIT==16


      IMPLICIT NONE

!     Uebergabeparameter:

      REAL,    INTENT(IN)  :: AR1(*)
      INTEGER, INTENT(IN)  :: LAR1
      REAL,    INTENT(IN)  :: FAKT
      REAL,    INTENT(IN)  :: UNDEF
      INTEGER, INTENT(IN)  :: IBMJ
      INTEGER, INTENT(IN)  :: IBMAP(*)
      INTEGER, INTENT(IN)  :: NRBIT
      INTEGER, INTENT(IN)  :: IABDB
      INTEGER, INTENT(OUT) :: IAR2(*)
      INTEGER, INTENT(OUT) :: ISCAL
      REAL,    INTENT(OUT) :: REFMIN
      INTEGER, INTENT(OUT) :: NRGB
      INTEGER, INTENT(OUT) :: IRM

      INTEGER IANFB, IAP, IND, IZW, IZWMAX, INB, IBMSK, J

      REAL    AMIN, AMAX, DIFF, ZSCAL, ZWMAX

#include "machconsts.h"

      ! ISCAL muss in 2 Bytes passen
      INTEGER, PARAMETER:: ISCALMAX = 32767, ISCALMIN = -32767

!     Variablen fuer Behandlung der Sonderfaelle

      INTEGER, PARAMETER :: MAXSTRIP = 4096

      INTEGER IZWV(MAXSTRIP), IPCK(MAXSTRIP), JSTART, JLEN

      REAL    ZW(MAXSTRIP)

!     Einige Fortran Compiler moegen keinen Ausdruck der Form ISHFT(I,X)
!     mit X > 32, wenn Integers nur 32 Bit weit sind.
!     Die folgenden Parameter-Deklarationen verhindern diese
!     Fehlermeldungen auf Maschinen mit 32-Bit Integers.
!     Man beachte, dass diese Parameter nur im Zweig IBMAX==64 benutzt werden!

      INTEGER, PARAMETER :: I56 = IBMAX- 8
      INTEGER, PARAMETER :: I48 = IBMAX-16
      INTEGER, PARAMETER :: I40 = IBMAX-24
      INTEGER, PARAMETER :: I32 = IBMAX-32


C*    ------------------------------------------------------------------
C     GUELTIGE WERTE FUER NRBIT ABFRAGEN
      IF         ( NRBIT. GT. 3*IBMAX/4) THEN
         IRM     = 1
         PRINT     2000
 2000    FORMAT  (' ///FEHLER VON : FUER NRBIT ERLAUBT: <= 24/48')
         RETURN
      ENDIF
C*    ------------------------------------------------------------------
C     EXTREMWERTE BESTIMMEN
      CALL MINMAX (AR1, LAR1, UNDEF, IBMJ, IBMAP, AMIN, AMAX, IRM)
      IF         ( IRM .NE. 0) RETURN

      AMIN       = AMIN*FAKT
      AMAX       = AMAX*FAKT
      REFMIN     = AMIN
C*    ------------------------------------------------------------------
C     SKALIERUNGSFAKTOR BERECHNEN
C*    RJ: Dies geschieht hier ohne Zuhilfenahme der LOG Funktion
C*        um auch in numerischen Grenzfaellen exakte Ergebnisse
C*        zu erhalten

      DIFF       = AMAX - AMIN
      ISCAL      = 0
      ZSCAL      = 1.0
      ZWMAX      = 2.0**NRBIT-0.5   ! Maximalwert fuer skaliertes DIFF
      IZWMAX     = ISHFT(1,NRBIT)-1 ! Maximalwert fuer verpackte Werte

      ! Fuer den Fall DIFF .EQ. 0 lassen wir die obigen Werte fuer
      ! ISCAL und ZSCAL

      IF (DIFF.GT.0) THEN

         IF ( DIFF .GE. ZWMAX ) THEN

            DO WHILE (DIFF*ZSCAL.GE.ZWMAX .AND. ISCAL.LT.ISCALMAX)
               ISCAL = ISCAL+1
               ZSCAL = ZSCAL*0.5
            ENDDO

         ELSE

            DO WHILE (DIFF*ZSCAL*2.LT.ZWMAX .AND. ISCAL.GT.ISCALMIN)
               ISCAL = ISCAL-1
               ZSCAL = ZSCAL*2.0
            ENDDO

         ENDIF

      ENDIF

C*    ------------------------------------------------------------------

!     Abfrage auf Sonderfall

      IF(IBMJ.NE.1 .AND. (NRBIT==8 .OR. NRBIT==16) .AND.
     &   (IBMAX==32 .OR. IBMAX==64) ) GOTO 1000

C*    ------------------------------------------------------------------


      INB        = 0                ! Zaehler fuer Worte in IBMAP, zaehlt ab 1
      IBMSK      = 0                ! Bitmaske fuer aktuelles Bit in IBMAP

      ! IANFB: Anfangsbit im Datensatz, zaehlt ab 0 und von links
      ! IND:   Zaehler fuer Worte in IAR, zaehlt ab 1
      ! IAP:   Nummer des Bits unmittelbar vor dem zu entpackenden Teil,
      !        zaehlt ab 0 und von rechts

      IANFB = (IABDB-1)*IBYTE
      IND   = IANFB/IBMAX+1
      IAP   = IBMAX - MOD(IANFB,IBMAX)

      NRGB  = 0 ! Anzahl der besetzten Punkte


      DO J = 1, LAR1

C*       Falls mit Bitmap, unbesetzte Werte ueberspringen

         IF ( IBMJ.EQ.1) THEN
            IBMSK = ISHFT(IBMSK,-1)
            IF ( IBMSK .EQ. 0 ) THEN
C*             NEUES BITMAP-WORT BEREITSTELLEN
               INB   = INB + 1
               IBMSK = ISHFT(1,IBMAX-1)
            ENDIF
C*          ABFRAGE, OB GI.PUNKT UNDEFINIERT
            IF(IAND(IBMAP(INB),IBMSK).EQ.0 ) THEN
               CYCLE
            ENDIF
         ENDIF

         NRGB = NRGB+1

C        WERT UMRECHNEN UND PACKEN

         IZW = NINT((AR1(J)*FAKT-AMIN)*ZSCAL)
         IF(IZW.GT.IZWMAX) IZW = IZWMAX ! Zur Sicherheit, sollte nie auftreten
         IF(IZW.LT.0     ) IZW = 0      ! Zur Sicherheit, sollte nie auftreten


         IF ( IAP >= NRBIT ) THEN

!           1 Zielwort

            CALL MVBITS (IZW,0,NRBIT,IAR2(IND),IAP-NRBIT)

         ELSE

!           2 Zielworte

            CALL MVBITS (IZW,NRBIT-IAP,IAP,IAR2(IND),0)
            CALL MVBITS (IZW,0,NRBIT-IAP,IAR2(IND+1),IBMAX+IAP-NRBIT)

         ENDIF

!        Update von IAP und IND

         IAP = IAP - NRBIT
         IF ( IAP <= 0 ) THEN
            IAP = IAP + IBMAX
            IND = IND + 1
         ENDIF

      ENDDO

      RETURN

C*    ------------------------------------------------------------------

 1000 CONTINUE

!     Behandlung der Sonderfaelle fuer 8 und 16 Bit


      NRGB    = LAR1 ! Anzahl der besetzten Punkte
      IZWV(:) = 0

      DO JSTART = 1, LAR1, MAXSTRIP  ! Loop fuer Stripmining

         JLEN = MIN(LAR1-JSTART+1,MAXSTRIP)

!        Werte in Integer-Zahlen umrechnen

         DO J = 1, JLEN
            IZWV(J) = NINT((AR1(J+JSTART-1)*FAKT-AMIN)*ZSCAL)
            ! Das folgende ist nur zur Sicherheit, sollte nie relevant sein:
            IZWV(J) = MIN(IZWMAX,MAX(0,IZWV(J)))
         ENDDO

!        Werte packen

         IF (NRBIT==16 .AND. IBMAX==32) THEN

            DO J=1,(JLEN+1)/2
               IPCK(J) = IOR(ISHFT(IZWV(2*J-1),16),IZWV(2*J))
            ENDDO

            CALL SPUMB(IPCK,1,JLEN*2,IABDB+(JSTART-1)*2,IAR2,IRM)

         ELSEIF (NRBIT==16 .AND. IBMAX==64) THEN

            DO J=1,(JLEN+3)/4
               IPCK(J) = ISHFT(IZWV(4*J-3),I48)
     &                 + ISHFT(IZWV(4*J-2),I32)
     &                 + ISHFT(IZWV(4*J-1), 16)
     &                 +       IZWV(4*J)
            ENDDO

            CALL SPUMB(IPCK,1,JLEN*2,IABDB+(JSTART-1)*2,IAR2,IRM)

         ELSEIF (NRBIT==8 .AND. IBMAX==32) THEN

            DO J=1,(JLEN+3)/4
               IPCK(J) = ISHFT(IZWV(4*J-3), 24)
     &                 + ISHFT(IZWV(4*J-2), 16)
     &                 + ISHFT(IZWV(4*J-1),  8)
     &                 +       IZWV(4*J)
            ENDDO

            CALL SPUMB(IPCK,1,JLEN,IABDB+(JSTART-1),IAR2,IRM)

         ELSEIF (NRBIT==8 .AND. IBMAX==64) THEN

            DO J=1,(JLEN+7)/8
               IPCK(J) = ISHFT(IZWV(8*J-7),I56)
     &                 + ISHFT(IZWV(8*J-6),I48)
     &                 + ISHFT(IZWV(8*J-5),I40)
     &                 + ISHFT(IZWV(8*J-4),I32)
     &                 + ISHFT(IZWV(8*J-3), 24)
     &                 + ISHFT(IZWV(8*J-2), 16)
     &                 + ISHFT(IZWV(8*J-1),  8)
     &                 +       IZWV(8*J)
            ENDDO

            CALL SPUMB(IPCK,1,JLEN,IABDB+(JSTART-1),IAR2,IRM)

         ENDIF

      ENDDO

      END
