/*
C**** rts_ed1  - Einsetzen von Recordkontrollworten in eine Datei mit GRIB- oder BUFR-Daten
C**   Aufruf   :   rts_ed1 (File fp_in, int iun, int *ihits, int *irm )
C**   Zweck    :   Aufbereitung von GRIB-Daten ab Edition 1 oder BUFR-Daten ab
C**                Edition 2, die sich auf einem nicht strukturiertem File
C**                (ohne Record-Marken) befinden, in das Standardformat.
C**   Parameter:   fp_in   - Dateizeiger fuer die Inputdatei
C**                iun     - Filedeskriptor fuer die Outputdatei
C**                ihits   - Anzahl der verarbeiteten Felder
C**                irm     - Rueckgabewert
C**
C**   Externals :  uegex
C**   Verfasser :  A.Freyberg, TI12, November 1999
C     Geaendert :  C. Vollbrecht, TI12, Juni 2002, Erweiterung fr BUFR
C                  B.Loeser, Juli 2002, Byte-swapping fuer Little-Endian
C                                       Maschinen eingebaut (LINUX)
C                  H. Weber, 02/2006  max_char increased for ECMWF 0.25deg
C                  C. Gramalla 02/2006  Aufruf von ccuegex ohne byte-swapping
*/
#include <stdio.h>
#include <stdlib.h>

#define MAX_CHAR           3000000
#ifndef FALSE
#define FALSE (char)0
#endif
#ifndef TRUE
#define TRUE (char)1
#endif

typedef char LOGICAL;

extern int errno;

/*   Fortran-C-Schnittstelle wird includiert             */
#include "fortran_c.h"

int rts_ed1 ( FILE * fp_in, int iun, int *ihits, int *irm)

{
  int i;
  int ch;
  char ifeld [MAX_CHAR];
  LOGICAL lgrib;

#ifdef CRAY
 union {
        int grib_laenge;
        char lfeld[8];
       } laenge;

#else 
 union {
        int grib_laenge;
        char lfeld[4];
       } laenge;
#endif
 
#if defined __linux__
 long ii;
#endif

/* Initialisierung: */

 i = 0;

 *ihits = 0;

 lgrib = FALSE;
 laenge.grib_laenge = 0;

/* Einlesen, Byte fuer Byte: */

 while ( ( ch = fgetc(fp_in) ) != EOF )
       {
        ifeld[i++] = ch;

/* GRIB-Kennung suchen: */

        if ( i >3 && ifeld[i-4] == 'G' && ifeld[i-3] == 'R' &&
                     ifeld[i-2] == 'I' && ifeld[i-1] == 'B')
           {
            lgrib = TRUE;
            ifeld[0] = 'G';
            ifeld[1] = 'R';
            ifeld[2] = 'I';
            ifeld[3] = 'B';
            i = 4 ;
           }

/* BUFR-Kennung suchen, falls keine GRIB-Kennung gefunden: */

        if ( ! lgrib && i >3 && ifeld[i-4] == 'B' && ifeld[i-3] == 'U' &&
                                 ifeld[i-2] == 'F' && ifeld[i-1] == 'R')
           {
            lgrib = TRUE;
            ifeld[0] = 'B';
            ifeld[1] = 'U';
            ifeld[2] = 'F';
            ifeld[3] = 'R';
            i = 4 ;
           }

/* GRIB-Laenge feststellen */

        if ( i == 8 && lgrib )
           {
#ifdef CRAY
            laenge.grib_laenge=0;
            laenge.lfeld[4] = 0;
            laenge.lfeld[5] = ifeld[4];
            laenge.lfeld[6] = ifeld[5];
            laenge.lfeld[7] = ifeld[6];
#elif defined __linux__
 /* Fuer alle Little-Endian Maschinen muessen die Bytes geswapt werden! */
            laenge.lfeld[0] = ifeld[6];
            laenge.lfeld[1] = ifeld[5];
            laenge.lfeld[2] = ifeld[4];
            laenge.lfeld[3] = 0;
#else
            laenge.lfeld[0] = 0;
            laenge.lfeld[1] = ifeld[4];
            laenge.lfeld[2] = ifeld[5];
            laenge.lfeld[3] = ifeld[6];
#endif

/*            if ( ch != 1 )          */
/*               {                    */
                printf("Edition-Nr.des Datensatzes: %d\n",ch) ;
/*                lgrib = FALSE;      */
/*               }                    */

/* restliche Bytes des Gribs einlesen */

            if ( fread ( &ifeld[8], (laenge.grib_laenge - 8), 1, fp_in ) != 1 )
               {
                printf("Fehler beim Einlesen der Daten.\n" );
                perror("rts_ed1");
                lgrib = FALSE;
               }

            i = laenge.grib_laenge;

/* Ende 7777 des Feldes suchen und Rausschreiben mit ccuegex. */

            if ( ifeld[i-1] == '7' && ifeld[i-2] == '7' && ifeld[i-3] == '7' && ifeld[i-4] == '7' )
               {

                ccuegex(&iun,ifeld,&i,0,irm);
                (*ihits)++;
                if ( *irm != 0 ) 
                   {
                    printf("Fehler bei ccuegex, IRM= %d",*irm);
                    return *irm;
                   }
               i = 0;
               laenge.grib_laenge = 0;
               lgrib = FALSE;
               }
            else
               {
                printf("Datensatz Nr. %d SKIPPED\n",*ihits);
                printf("WARNUNG: Tatsaechliche Laenge: %d\n",i);
                printf("         Gemeldete     Laenge: %d\n",laenge.grib_laenge);
                i = 0;
                laenge.grib_laenge = 0;
                lgrib = FALSE;
               }

           }
       }
  return 0;
}
