        SUBROUTINE SP2CP(DS,KJ,KK,KM,KJL,KKL,KML,KP,KN,IRM )
C
C**** SP2CP    -   spektralen Koeffizienten fuer complexes Packen vorbereiten
C**   AUFRUF:      CALL SP2C(DS,KJ,KK,KM,KJL.CKML,KP-,IRM )
C**   ZWECK:       DATENSATZ FUER GRIBEX VORBEREITEN (COMPLEX PACKING)
C**   PARAMETER:
C**      INPUT :
C**                DS     DATENSATZ MIT KOEFFIZIENTEN. ENTHAELT NACH AUFRUF
C**                       DIE GEORDNETEN UND NORMIERTEN WERTE
C**                KJ, KK, KM   PENTAG.RES.PAR.AUS GRID-DESCR-SECTION
C**                KJL,KKL,KML  PENTAG.RES.PAR.AUS BIN.DATA SECTION
C**                KP           POTENZ VON N*(N+1), MIT DER NORMIERT WIRD
C**                           ( AUS BDS(7) )
C**      OUTPUT:   KN           ANZAHL WERTE AM ANFANG VON DS (NICHT PACKEN)
C**                IRM          =0 FALLS FEHLERFREIER ABLAUF
C*    WARNUNG:     INTERN WIRD EIN ZWISCHENSPEICHER VERWENDET DER GROESSE
C*                 2*(KJL+1)*(KML+1). DIESER HAT Z.ZT. DIMENSION 72. WENN
C*                 EINMAL HOEHERE WELLENZAHLEN ANFALLEN: VERGROESSERN.
C**   VERFASSER:   W. Wergen - F - AUGUST 90
C*    ------------------------------------------------------------------
C       Methode
C       -------
C       SP2CP extrahiert die durch KJL, KKL und KML definierten niedrigen
C       Koeffizienten und stellt sie an den Anfang von DS  fuer die Packung
C       mit erhoehter Genauigkeit (32 bit). Die uebrigen Koeffizienten
C       werden mit (n*(n+1))**KP normiert, um eine geringere Varianz der
C       Werte zu erreichen (n ist die Gesamtwellenzahl). UP CP2SP kehrt
C       diese Operationen um.
C       LITERATUR:
C       ----------
C       FM92 GRIB, complex packing
C       ECMWF Rsearch Manual 2
C*    ------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER KJ, KK, KM, KJL, KKL, KML, KP, KN, IRM
      INTEGER IS, ISPL, ISP, JM, JEL, JE, JS, JN, J

      REAL    DS, SPB

      DIMENSION    DS (*)
      DIMENSION    SPB(72)
      IRM        = 0
C*    ------------------------------------------------------------------
C       1. Test auf Konsistenz der Aufrufparameter
C*    --------------------------------------------
      IF         ( KJL.GT.KJ.OR.KKL.GT.KK.OR.KML.GT.KM)    THEN
      PRINT        1000,KJL,KJ,KKL,KK,KML,KM
 1000 FORMAT     (' Subroutine SP2CP: Anrufparameter falsch!',6I6)
      IRM        = 1000
                                                           RETURN
                                                           ENDIF
C*    ------------------------------------------------------------------
      IF         ( KJL.LT.1.OR.KKL.LT.1.OR.KML.LT.1 )      THEN
      PRINT        2000,KJL,KKL,KML
 2000 FORMAT     (' Subroutine SP2CP: KJL, KKL und KML muessen ',
     1            ' groesser Null sein!',3I6 )
      IRM        = 2000
                                                           RETURN
                                                           ENDIF
C*    ------------------------------------------------------------------
C     2. Schleife ueber die zonalen Wellenzahlen
C*    ------------------------------------------
      IS         = 0
      ISPL       = 0
      ISP        = 0
C
      DO 5000 JM = 0,KM
      JEL        = MIN(KJL+JM,KKL)
      JE         = MIN(KJ+JM,KK)
      JS         = JM
C*    ------------------------------------------------------------------
C       2.2 Speicherung der Werte fuer genaueres Packen
C*    -------------------------------------------------
      IF         ( JM.LE.KML )                             THEN
      DO 3000 JN = JM,JEL
      SPB(ISPL+1)= DS (ISP+1)
      SPB(ISPL+2)= DS (ISP+2)
      ISPL       = ISPL+2
      ISP        = ISP+2
      JS         = JS+1
 3000 CONTINUE
                                                           ENDIF
C*    ------------------------------------------------------------------
C       2.4 Normierung der uebrigen Koeffizienten
C*    -------------------------------------------
CDIR$   IVDEP
      DO 4000 JN = JS,JE
      DS (IS+1)  = DS (ISP+1)*(JN*(JN+1))**KP
      DS (IS+2)  = DS (ISP+2)*(JN*(JN+1))**KP
      IS         = IS+2
      ISP        = ISP+2
 4000 CONTINUE
C
 5000 CONTINUE
C*    ------------------------------------------------------------------
C       4. Kopieren der genauer zu packenden Werte an den Anfang von DS
C*    ------------------------------------------------------------------
CDIR$   IVDEP
      DO 6000 J  = ISP,ISPL+1,-1
      DS (J)     = DS (J-ISPL)
 6000 CONTINUE
C
      DO 7000 J  = 1,ISPL
      DS (J)     = SPB(J)
 7000 CONTINUE
C
      KN         = ISPL
C*    ------------------------------------------------------------------
C*    LAENGE DES ZWISCHENSPEICHERS SPB ABSICHERN
      IF         ( KN.GT.72 )                              THEN
      PRINT        8000,KN
 8000 FORMAT     (' SP2CP  - KN=',I6,'. Z.ZT. NUR 72 ERLAUBT - ERROR.')
      IRM        = 8000
                                                           RETURN
                                                           END IF
C*    ------------------------------------------------------------------
      END

