This is the source code for PARASO, a fully coupled Antarctic model. 

**Description paper**: *PARASO, a circum-Antarctic fully-coupled atmosphere - land - ocean - sea-ice - ice-sheet model involving COSMO5.0, CLM4.5, NEMO3.6, LIM3.6 and f.ETISh1.7*.
C. Pelletier, T. Fichefet, H. Goosse, K. Haubner, S. Helsen, P.-V. Huot, C. Kittel, F. Klein, S. Le clec'h, N. P. M. van Lipzig, S. Marchi, F. Massonnet, P. Mathiot, E. Moravveji, E. Moreno-Chamarro, P. Ortega, F. Pattyn, N. Souverijns, G. Van Achter, S. Vanden Broucke, A. Vanhulle, D. Verfaillie, L. Zipf.
_Geoscientific Model Development Discussions_ (in review as of 04/10/2021);
[10.5194/gmd-2021-315](https://doi.org/10.5194/gmd-2021-315)

These sources are distribued in two places:

  - full version : a tarball accessible to [CLM-Community members](https://wiki.coast.hzg.de/clmcom/) (free for all research applications), accessible once registered to the [RedC](https://redc.clm-community.eu/) via "COSMO-CLM" -> "Downloads".
  - version without COSMO (**this version**): a Zenodo deposit (DOI [10.5281/zenodo.5337509](https://doi.org/10.5281/zenodo.5337509)) which does not include the COSMO (atmosphere) part of PARASO due to licensing.

# Compilation

This deposit provides all PARASO sources. For the compilation, architecture and configuration files matching the following modules are also provided:

```
  1) GCCcore/4.9.3
  2) binutils/2.25-GCCcore-4.9.3
  3) icc/2016.3.210-GCC-4.9.3-2.25
  4) ifort/2016.3.210-GCC-4.9.3-2.25
  5) iccifort/2016.3.210-GCC-4.9.3-2.25
  6) impi/5.1.3.181-iccifort-2016.3.210-GCC-4.9.3-2.25
  7) iimpi/2016.03-GCC-4.9.3-2.25
  8) imkl/11.3.3.210-iimpi-2016.03-GCC-4.9.3-2.25
  9) intel/2016a
 10) zlib/1.2.8-GCCcore-4.9.3
 11) Szip/2.1-GCCcore-4.9.3
 12) HDF5/1.8.16-intel-2016a
 13) cURL/7.49.1-GCCcore-4.9.3
 14) netCDF/4.4.0-intel-2016a
 15) netCDF-Fortran/4.4.4-intel-2016a
```

Other combinations may be possible through editing the various architecture and configuration files in the `build_paraso` subdirectory.

Compilation is done through calling `build_paraso/build_paraso.sh`, once the requisite modules have been loaded.

The first compilation is slow, since CESM will download external data through a subversion deposit.
Once this has been done, compilating anew should take about 15 - 20min.

## f.ETISh compilation

f.ETISh (ice-sheet model) is written in MATLAB. It should be compiled from within MATLAB, using matching MATLAB module versions with the MATLAB_Runtime (MCR) module used for running PARASO. In this deposit, we provide:

  - full f.ETISh sources in the `fetish/src` directory;
  - a compiled version of f.ETISh in the `fetish/bin` directory, compiled with MCR/2018b.
  
The cluster on which the PARASO experiments ran did not feature MATLAB, hence the pre-compiled version (compiled from another MATLAB-including computer) was used. The sources are provided for the sake of transparency.

# Running an experiment

  1. Load the modules you used for the NEMO-CCLM compilation.
  
  2. `source cclm-nemo-f-antarctica/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh`

  3. `mkdir -p exp/paraso_run; cd exp/paraso_run; coral init breniac_coupledf`.
  
  4. Input file paths (to be potentially edited) are present in `data.cfg`, `namelists/cosmo_clm_parameter` (`ydirini` and `ydirbd`).
  
  5. The model will run in the `out_dir` specified in `experiment.cfg`. Be sure to have lots of space and write permission there.
  
  6. `coral build_coupledf; qsub run.sh`


# Licenses

PARASO has been built from distinct models and tools, whose specific licenses are given below.

## Atmosphere: COSMO v5.0

The COSMO model is free of charge for all research applications; however, access is license-restricted (see [here](http://www.cosmo-model.org/content/consortium/licencing.htm), last access: August 24th 2021). To download, the user needs to become a member of the CLM-Community or the respective institute needs to hold an institutional license. The documentation of the COSMO model is developed by the COSMO-Consortium, consists of scientific documentation and a user guide, and is permanently available [here](https://www.dwd.de/EN/ourservices/cosmo_documentation/cosmo_documentation.html) (last access: August 24th 2021).

## Land: CLM v4.5

The [Community Land Model](https://www.cesm.ucar.edu/models/clm/) is the land model for the Community Earth System Model [(CESM)](https://www.cesm.ucar.edu).
It is an open and community-driven model. The CESM project is supported primarily by the National Science Foundation.

The PARASO version of CCLM (COSMO and CLM coupled together) is based on COSMO v5.0 and CLM v4.5, and it includes specific developments which are included herein.

## Ocean, sea ice and their I/O: NEMO-LIM v3.6, XIOS v1.0

NEMO, LIM and XIOS (a NEMO-compatible I/O library) are developed by the NEMO consortium, and distributed under the CeCILL license.
The NEMO-LIM version used in PARASO has been built from the standard 3.6 version, with the ice-shelf following two modifications:
  1. an undocumented lateral sea-ice melt scheme (J. Raulier, UCLouvain);
  2. the ice-shelf coupling module from the [revision 11248](https://forge.ipsl.jussieu.fr/nemo/browser/branches/UKMO/dev_isf_gm_UKESM_GO6package_r9321?rev=11247) of the [dev\_isf\_remapping\_UKESM\_GO6package\_r9314 NEMO development branch](https://forge.ipsl.jussieu.fr/nemo/browser/branches/UKMO/dev_isf_gm_UKESM_GO6package_r9321).
Complete NEMO documentation is available from the [NEMO consortium website](www.nemo-ocean.eu).

## Ice sheet: f.ETISh v1.7

f.ETISh (Fast Elementary Thermomechanical Ice Sheet model of intermediate complexity v1.7) has been developed by F. Pattyn and co-workers (Pattyn _et al._, 2017, doi [10.5194/tc-11-1851-2017](https://doi.org/10.5194/tc-11-1851-2017)).

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.


## Coupler: OASIS3-MCT_2.0

The OASIS-MCT coupler is developed by the CERFACS (Toulouse, France) and the CNRS (Paris, France). It is distributed under the Lesser GNU General Public License (LGPL). License details are provided in `./oasis3-mct/LICENSE`.

OASIS-MCT_2.0 can be downloaded from the [its official website](https://portal.enes.org/oasis) (last access: June 2019).

## Job submission tool: Coral

Coral has been created by P.Y. Barriat and F. Damiens (CISM, UCLouvain, Belgium), and adapted to PARASO. It is distributed under the Creative Commons CC0 1.0 Universal license, provided in `nemo/NEMO_DEV/EXTERNAL/coral/LICENSE`.

## NEMO - f.ETISh coupling routines

The ocean -- ice sheet coupling interface also relies on:

  - the Climate Data Operators ([CDO](https://code.mpimet.mpg.de/projects/cdo/)) version 1.9.5, developed at the Max-Planck-Institute for Meteorology and released under the terms of the GNU General Public License v2 (GPL);
  - the netCDF Operators ([NCO](http://nco.sourceforge.net)) version 4.7.6, developed at the University of California, Irvine, and distributed under the GNU GPL v2 license.
