# Installing Coupled `NEMO-CCLM` and `CCLM-Standalone`

## About
This repository walks you through easy steps to build the CCLM (and its dependencies) on any VSC cluster by following few simple steps. The source-tree and the build scripts are all adapted from the version passed by Francois Klein (UCL).

## Why bother?
The idea of the current repo is to
- have a single point of reference for the software that the team uses (as much as possible)
- allowing the team to contribute to the software in a single repository, and avoiding to have multiple copies spread around
- allowing to build the software flexibly on VSC machines by just changing at least two lines in build scripts (`build-cclm-standalone.sh` and `build-nemo-cclm.sh`)
- to separate the source codes from the compiled versions (to allow multiple builds on various architectures), and facilitate concrete benchmarking tests

## Before you start
The current build workflow copies few template `Makefile/Fopts/.xml/.cfg` files in relevant paths, and overrides automatically some preset values therein (based on machine, architecture, and the toolchain that the user chooses). Hence, Only one compilation instance must be launched by the user at any given time; else, you will fall into an uncharted territory!

## Content
This repository contains the following bash scripts and configuration files (inside `build_cclm`):
- `source-codes.tar.gz` which bundles the raw source tree for `libgrib` and `nemo_cclm`
- `build-{libgrib,nemo_cclm,cclm-standalone}.sh`: corresponding scripts which build each of the tools. Calling each of these tools requires specifying two arguments: 
```bash
dir_sources=</path/to/toplevel/nemo_cclm>
dir_apps=</path/to/installed/folders>
```
`dir_sources` is the **full path** to a directory where the [CCLM-NEMO-F](https://gitlab.kuleuven.be/rcs/cclm/cclm-nemo-f-antarctica) repository is cloned into, and `dir_apps` is the **full path** to where the installed software would be stored; please avoid using relative paths.
- configuration template files: these files fix the hard-coded paths and variables, and allow more flexible and generic compilation of the software. The variables that need modification are normally highlighted as e.g. `<set>`. 
- The relavant config files are: `Makefile.libgrib`, `Makefile.COSMO`, `Fopts.COSMO`, `make.inc.OASIS3`, `config_compilers.xml.CESM`, `config_machines.xml.CESM`, `arch-intel_oasis.path.XIOS`, `config.fcm.XIOS`, `arch-xios1_oasis3_2.fcm.NEMO`

## How to compile the software
Simply, take the following steps:

- clone this repository (somewhere) in your `$VSC_DATA`, e.g.
``` bash
cd $VSC_DATA
git clone git@gitlab.kuleuven.be:rcs/cclm/cclm-nemo-f-antarctica.git
cd cclm-nemo-f-antarctica/build_cclm
```

- open `build-{nemo-cclm,cclm-standalone}.sh`:
  - if you would like to experiment with new toolchains, adapt the `dependencies` block
  - set `dir_sources` to point to the folder where the source codes for `libgrib` and `nemo_cclm` therein reside
  - set `dir_apps` to where all these applications will be stored after installation. It is recommended to separate the source codes from the linked binaries. The two VSC-specific environment variables `${VSC_INSTITUTE_CLUSTER}` and `${VSC_ARCH_LOCAL}` specify the cluster name and CPU architecture for which the software is being built; additionally, you can append toolchain year to the folder name, e.g. `dir_apps=$VSC_DATA/compiled/${VSC_INSTITUTE_CLUSTER}.${VSC_ARCH_LOCAL}.${toolchain}`. This can end up putting the compiled binaries into `$VSC_DATA/compiled/genius.skylake.2019b`

- Log in to a compute node using
``` bash
qsub -I -lnodes=1:ppn=28:skylake -A project_id
cd $VSC_DATA/cclm-nemo-f-antarctica/build_cclm
```
It ensures you to build the code using nodes with the same architecture (skylake) as the nodes used for executing the code afterwards.

- execute the build script, e.g.
``` bash
bash ./build-cclm-standalone.sh 2>&1 | tee logs
```

## Runtime Environment Variables
Apparently, `libgrib` has a way of controlling the number of bits needed to store a piece of "word" in memory. So, to comply with how the codes were built, before using `CCLM`, make sure you specify (e.g. in your bash prompt for interactive jobs, or in your job script) the following line:
``` bash
export LIBDWD_FORCE_CONTROLWORDS=1
```

# Launching Experiments
## Running Coupled model
To launch a 1 day-long test experiment using coral (e.g. `{coupled|coupledf}` model):
```bash
cd </path/to/coupled/experiment/folder>
mkdir coupled_run; cd coupled_run;
coral init *machine*_coupled
coral build_coupled
coral submit
```
where `machine` is either of `genius` or `breniac`.

## Running COSMO-CLM in standalone
Launch the experiment using coral:
```bash
cd </path/to/cclm/experiment/folder>
mkdir cclm_run; cd cclm_run;
coral init *machine*_cclm
coral build_cclm
coral submit
```
where `machine` is either of `genius` or `breniac`.
