#!/bin/bash

set -e
umask 0002

############################################################ 
# Only Change dependencies and paths if you know what you're doing

# which
machine=$VSC_INSTITUTE_CLUSTER
arch=$VSC_ARCH_LOCAL
toolchain='2016a'
paraso_version='v1.4.1'
adj_rout_ver='samuel' # choices are {samuel, niels_baseline, niels_paper, alexandra}

build_git_tag=true # if "true", build PARASO git tagged version specified by paraso_version; else, build current sources on disk (whatever they are)

[[ $machine == 'genius' && $arch == 'cascadelake' ]] && arch='skylake' || true

# paths
cwd=$(pwd)
dir_sources=$(dirname $PWD)
dir_apps="$VSC_DATA/apps/${machine}.${arch}.${toolchain}.${paraso_version}/cclm_standalone_${adj_rout_ver}_adj_rout_debug"
############################################################ 

# Please do not modify the rest of this file

# source paths
dir_nemo_cclm=$dir_sources
dir_oasis=$dir_nemo_cclm/oasis3-mct
dir_cesm=$dir_nemo_cclm/cesm1_2_0
dir_cosmo=$dir_nemo_cclm/cosmo_src
dir_xios=$dir_nemo_cclm//nemo/NEMO_DEV/EXTERNAL/xios-1.0
dir_nemo=$dir_nemo_cclm/nemo/NEMO_DEV

# destination paths
dir_install_oasis=$dir_apps/OASIS3-MCT_2.0
dir_install_cesm=$dir_apps/cesm
dir_bin_cesm=$dir_install_cesm/bin
dir_install_cosmo=$dir_apps/cosmo
dir_bin_cosmo=$dir_install_cosmo/bin

############################################################ 

# dependencies
source dependencies.sh

# prepare
if [[ ! -d $dir_nemo_cclm ]]; then
  cd $dir_sources
  git clone git@gitlab.kuleuven.be:rcs/cclm/cclm-nemo-f-antarctica.git
fi
cd $dir_nemo_cclm

if [ "${build_git_tag}" == "true" ]; then
  git fetch --all --tags
  git checkout tags/$paraso_version
  git pull origin master
fi  

source $dir_nemo_cclm/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh

############################################################ 

###################
# Compile OASIS3-MCT_2.0
###################
cd $cwd
make_oasis_bak=$cwd/make.inc.OASIS3
make_oasis=$dir_oasis/util/make_dir/make.inc
test -f $make_oasis_bak || { echo "Error: file not found: $make_oasis_bak"; exit 12; }
cp $make_oasis_bak $make_oasis

orig='COUPLE = <set>'
update="COUPLE          = $dir_oasis"
sed -i "s|$orig|$update|g" $make_oasis

orig='ARCHDIR = <set>'
update="ARCHDIR         = $dir_install_oasis"
sed -i "s|$orig|$update|g" $make_oasis

orig='F90FLAGS_1 = <set>'
update='F90FLAGS_1 = -g -check all -CB -traceback -vec-report0 -O0 -fno-alias -ip -align -msse3'
sed -i "s|$orig|$update|g" $make_oasis

orig='CCFLAGS_1 = <set>'
update='CCFLAGS_1 = -vec-report0 -O0 -g -traceback -fno-alias -align -fp-model precise'
sed -i "s|$orig|$update|g" $make_oasis

orig='LDFLAGS = <set>'
update='LDFLAGS = -O0'
sed -i "s|$orig|$update|g" $make_oasis

cd $dir_oasis/util/make_dir
make realclean -f TopMakefileOasis3
make -f TopMakefileOasis3
test $? -eq 0 || { echo "Error: make -f TopMakefileOasis3 failed"; exit 13; }
cd $dir_install_oasis/lib
rm -rf libmct_oasis.a libmpeu_oasis.a; ln -s libmct.a libmct_oasis.a; ln -s libmpeu.a libmpeu_oasis.a
printf "\nInfo: OASIS3-MCT-2.0 compiled successfully \n\n"

###################
# Compile CESM
###################
cd $dir_cesm
config_cesm_machine_bak="$cwd/config_machines.xml.CESM"
config_cesm_machine="$dir_cesm/scripts/ccsm_utils/Machines/config_machines.xml"
config_cesm_compilers_bak="$cwd/config_compilers.xml.CESM"
config_cesm_compilers="$dir_cesm/scripts/ccsm_utils/Machines/config_compilers.xml"
test -f $config_cesm_machine_bak || { echo "Error: file not found: $config_cesm_machine_bak"; exit 14; }
cp -v $config_cesm_machine_bak $config_cesm_machine
test -f $config_cesm_compilers_bak || { echo "Error: file not found: $config_cesm_compilers_bak"; exit 14; }
cp -v $config_cesm_compilers_bak $config_cesm_compilers
mkdir -p $VSC_SCRATCH/cesm/inputdata

sed -i "s|TOPDIR_NEMO_CCLM|$dir_nemo_cclm|g" $config_cesm_machine
sed -i "s|VSC_SCRATCH|$VSC_SCRATCH|g" $config_cesm_machine

orig='<FFLAGS>-I${HOME}/modeles/nemo_cclm/oasis3-mct/'"$machine"
update="<FFLAGS>-g -O0 -traceback -I${dir_install_oasis}"
sed -i "s|$orig|$update|g" $config_cesm_compilers

orig='-I${HOME}/modeles/nemo_cclm/oasis3-mct/'"$machine"
update="-I${dir_install_oasis}"
sed -i "s|$orig|$update|g" $config_cesm_compilers

orig='<LDFLAGS>-L${HOME}/modeles/nemo_cclm/oasis3-mct/'"$machine"
update="<LDFLAGS>-g -O0 -traceback -L${dir_install_oasis}"
sed -i "s|$orig|$update|g" $config_cesm_compilers

machfile=$dir_cesm/scripts/ccsm_utils/Machines/env_mach_specific.${machine}
printf '%s\n' '#!/bin/tcsh' 'source /etc/csh.cshrc' > $machfile

# choose the relevant compile_cesm.sh script
adj_rout_tarball="adjusted_routines_${adj_rout_ver}.tar.gz"
case $adj_rout_ver in
  'niels_baseline' )
    test -d $dir_cesm/adjusted_routines_niels_baseline || tar axf $cwd/$adj_rout_tarball -C $dir_cesm
    cp -v $cwd/compile_cesm.sh.niels_adj_rout ./compile_cesm.sh
    sed -i 's|<adj-rout-dirname>|adjusted_routines_niels_baseline|g' compile_cesm.sh
  ;;
  'niels_paper' )
    test -d $dir_cesm/adjusted_routines_AEROCLOUD_30y || tar axf $cwd/$adj_rout_tarball -C $dir_cesm
    cp -v $cwd/compile_cesm.sh.niels_adj_rout ./compile_cesm.sh
    sed -i 's|<adj-rout-dirname>|adjusted_routines_AEROCLOUD_30y|g' compile_cesm.sh
  ;;
  'samuel' )
    test -d $dir_cesm/adjusted_routines_AEROCLOUD_30y_with_density || tar axf $cwd/$adj_rout_tarball -C $dir_cesm
    cp -v $cwd/compile_cesm.sh.samuel_adj_rout ./compile_cesm.sh
  ;;
  'alexandra' )
    test -d $dir_cesm/adjusted_routines_bs2 || tar axf $cwd/$adj_rout_tarball -C $dir_cesm
    cp -v $cwd/compile_cesm.sh.alexandra_adj_rout ./compile_cesm.sh
  ;;
  'default' )
    echo "Error: You must set adj_rout_var to {alexandra, niels, samuel}; \"default\" is not accepted here!"
    exit 15
  ;;
  * )
    echo "Error: Wrong adj_rout_var specified; choose from: {niels, samuel}"
    exit 15
  ;;
esac

./compile_cesm.sh $machine clean
test $? -eq 0 || { echo "Error: ./compile_cesm.sh clean failed"; exit 15; }

./compile_cesm.sh $machine build
test $? -eq 0 || { echo "Error: ./compile_cesm.sh build failed"; exit 15; }

test -d $dir_bin_cesm && rm -rf $dir_bin_cesm
mkdir -p $dir_bin_cesm
cp -v $dir_cesm/scripts/clm4_5_${machine}/bld/cesm.exe $dir_bin_cesm

printf "\nInfo: CESM compiled successfully\n\n"

###################
# Compile libgrib
###################
cd $cwd
test -f $dir_apps/libgrib/libgrib1.a || ./build-libgrib.sh $cwd $dir_apps

###################
# Compile COSMO
###################
cd $dir_cosmo
fopts_cosmo_bak=$cwd/Fopts.COSMO.nonemo
fopts_cosmo=$dir_cosmo/Fopts.${machine}.nonemo
test -f $fopts_cosmo && rm -f $fopts_cosmo
test -f $fopts_cosmo_bak || { echo "Error: file not found: $fopts_cosmo_bak"; exit 16; }
cp -v $fopts_cosmo_bak $fopts_cosmo
test -d $dir_bin_cosmo && rm -rf $dir_bin_cosmo
mkdir -p $dir_bin_cosmo

update="HOME_LIB = $dir_cosmo"
sed -i "s|HOME_LIB = <set>|$update|g" $fopts_cosmo

update="GRIB_LIB = $dir_apps/libgrib/libgrib1.a"
sed -i "s|GRIB_LIB = <set>|$update|g" $fopts_cosmo

update="OASDIR = $dir_install_oasis"
sed -i "s|OASDIR = <set>|$update|g" $fopts_cosmo

update="PROGRAM = $dir_bin_cosmo/cclm"
sed -i "s|PROGRAM = <set>|$update|g" $fopts_cosmo

sed -i "s|-O2 -xHost|-O0 -g -traceback|g" $fopts_cosmo
sed -i "s|-O2 -g|-O0 -g -traceback|g" $fopts_cosmo
sed -i "s|-O2 -DNETCDF|-O0 -DNETCDF|g" $fopts_cosmo

makefile_bak=$cwd/Makefile.COSMO
makefile=$dir_cosmo/Makefile
test -f $makefile && rm -f $makefile
cp -v $makefile_bak $makefile
sed -i 's|<set>|src_cp4|g' $makefile

rm -rf ObjDependencies; ln -s ObjDependencies_src_paraso ObjDependencies
rm -rf ObjFiles; ln -s ObjFiles_src_paraso ObjFiles
rm -rf Fopts; ln -s Fopts.${machine}.nonemo Fopts

make clean 
make -j 4
test $? -eq 0 || { echo "Error make for COSMO failed"; exit 17; }
printf "\nInfo: COSMO compiled successfully\n\n"

printf "\nInfo: CCLM-Standalone is compiled successfully in %s\n\n" $dir_apps
