#!/bin/bash

set -e
umask 0002

usage='./build-libgrib.sh </path/to/sources> </path/to/apps>'
test $# -eq 2 || { echo $usage; exit 1; } 

# which
version='061107'

# paths
dir_sources=$1
dir_apps=$2
cwd=$(pwd)
dir_grib="$dir_sources/DWD-libgrib1_$version"
dir_inc="$dir_grib/include"
dir_src="$dir_grib/source"
dir_install="$dir_apps/libgrib"

# files
tarball="$cwd/libgrib_061107.tar.gz"
libfile="$dir_install/libgrib1.a"
makefile="$dir_src/Makefile"
makefile_bak="$cwd/Makefile.libgrib"

# checks
test -f $tarball  || { echo "Error: tarball $tarball not found"; exit 11; }
test -d $dir_grib || tar axf $tarball -C $dir_sources
test -d $dir_install && rm -rf $dir_install
mkdir -p $dir_install
test -f $dir_grib/libgrib1.a && rm -f $dir_grib/libgrib1.a
test -f $libfile && rm -f $libfile
if [[ -f $makefile_bak ]]; then 
  cp $makefile_bak $makefile
else
  echo "Error ($0): $makefile_bak not found"
  exit 1
fi

# prepare
cd $dir_src

from='LIBPATH = <set>'
to="LIBPATH = $dir_install/"
sed -i "s|$from|$to|g" $makefile

# build
make clean
make -j 4
test $? -eq 0 || { echo "Error: make failed"; exit 2; }

cp -r $dir_inc $dir_install

echo "Info: libgrib successfully installed in $dir_install"

