#!/bin/bash

set -e
umask 0002

############################################################ 
# Only Change these variables and paths specific to your project

# which
machine=$VSC_INSTITUTE_CLUSTER
arch=$VSC_ARCH_LOCAL
toolchain='2016a'
paraso_version='v1.4.1'

build_git_tag=false # if "true", build PARASO git tagged version specified by paraso_version; else, build current sources on disk (whatever they are)

[[ $machine == 'genius' && $arch == 'cascadelake' ]] && arch='skylake' || true

# paths
cwd=$(pwd)
dir_sources=$(dirname $PWD)
dir_apps="$VSC_DATA/apps/${machine}.${arch}.${toolchain}.${paraso_version}/nemo_cclm"
scratch_nemo_cclm="${VSC_SCRATCH}"
############################################################ 

# Please do not modify the rest of this file

# source paths
dir_nemo_cclm=$dir_sources
dir_oasis=$dir_nemo_cclm/oasis3-mct
dir_cesm=$dir_nemo_cclm/cesm1_2_0
dir_cosmo=$dir_nemo_cclm/cosmo_src
dir_xios=$dir_nemo_cclm//nemo/NEMO_DEV/EXTERNAL/xios-1.0
dir_nemo=$dir_nemo_cclm/nemo/NEMO_DEV
dir_fetish=$dir_nemo_cclm/fetish

# destination paths
dir_install_oasis=$dir_apps/OASIS3-MCT_2.0
dir_install_cesm=$dir_apps/cesm
dir_bin_cesm=$dir_install_cesm/bin
dir_install_cosmo=$dir_apps/cosmo
dir_bin_cosmo=$dir_install_cosmo/bin
dir_install_nemo_cclm=$dir_apps/nemo 
dir_install_xios=$dir_apps/xios-1.0
dir_install_fetish=$dir_apps/fetish

############################################################ 

# dependencies
source dependencies.sh

# prepare
if [[ ! -d $dir_nemo_cclm ]]; then
  cd $dir_sources
  git clone git@gitlab.kuleuven.be:rcs/cclm/cclm-nemo-f-antarctica.git
fi
cd $dir_nemo_cclm

if [ "${build_git_tag}" == true ]; then
  git fetch --all --tags
  git checkout tags/$paraso_version
  git pull origin master
fi  

source $dir_nemo_cclm/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh

############################################################ 

###################
# Compile OASIS3-MCT_2.0
###################
cd $cwd
make_oasis_bak=$cwd/make.inc.OASIS3
make_oasis=$dir_oasis/util/make_dir/make.inc
test -f $make_oasis_bak || { echo "Error: file not found: $make_oasis_bak"; exit 12; }
cp $make_oasis_bak $make_oasis

orig='COUPLE = <set>'
update="COUPLE          = $dir_oasis"
sed -i "s|$orig|$update|g" $make_oasis

orig='ARCHDIR = <set>'
update="ARCHDIR         = $dir_install_oasis"
sed -i "s|$orig|$update|g" $make_oasis

orig='F90FLAGS_1 = <set>'
update='F90FLAGS_1 = -O2 -xHost'
sed -i "s|$orig|$update|g" $make_oasis

orig='CCFLAGS_1 = <set>'
update='CCFLAGS_1 = -O2 -xHost'
sed -i "s|$orig|$update|g" $make_oasis

orig='LDFLAGS = <set>'
update='LDFLAGS = -O2 -xHost'
sed -i "s|$orig|$update|g" $make_oasis

cd $dir_oasis/util/make_dir
make realclean -f TopMakefileOasis3
make -f TopMakefileOasis3
test $? -eq 0 || { echo "Error: make -f TopMakefileOasis3 failed"; exit 13; }
cd $dir_install_oasis/lib
rm -rf libmct_oasis.a libmpeu_oasis.a; ln -s libmct.a libmct_oasis.a; ln -s libmpeu.a libmpeu_oasis.a
printf "\nInfo: OASIS3-MCT-2.0 compiled successfully \n\n"

###################
# Compile CESM
###################
cd $dir_cesm
config_cesm_machine_bak="$cwd/config_machines.xml.CESM"
config_cesm_machine="$dir_cesm/scripts/ccsm_utils/Machines/config_machines.xml"
config_cesm_compilers_bak="$cwd/config_compilers.xml.CESM"
config_cesm_compilers="$dir_cesm/scripts/ccsm_utils/Machines/config_compilers.xml"
test -f $config_cesm_machine_bak || { echo "Error: file not found: $config_cesm_machine_bak"; exit 14; }
cp -v $config_cesm_machine_bak $config_cesm_machine
test -f $config_cesm_compilers_bak || { echo "Error: file not found: $config_cesm_compilers_bak"; exit 14; }
cp -v $config_cesm_compilers_bak $config_cesm_compilers
mkdir -p $scratch_nemo_cclm/cesm/inputdata

sed -i "s|TOPDIR_NEMO_CCLM|$dir_nemo_cclm|g" $config_cesm_machine
sed -i "s|SCRATCH_NEMO_CCLM|$scratch_nemo_cclm|g" $config_cesm_machine

orig='<FFLAGS>-I${HOME}/modeles/nemo_cclm/oasis3-mct/'"$machine"
update="<FFLAGS>-I${dir_install_oasis}"
sed -i "s|$orig|$update|g" $config_cesm_compilers

orig='-I${HOME}/modeles/nemo_cclm/oasis3-mct/'"$machine"
update="-I${dir_install_oasis}"
sed -i "s|$orig|$update|g" $config_cesm_compilers

orig='<LDFLAGS>-L${HOME}/modeles/nemo_cclm/oasis3-mct/'"$machine"
update="<LDFLAGS>-L${dir_install_oasis}"
sed -i "s|$orig|$update|g" $config_cesm_compilers

machfile=$dir_cesm/scripts/ccsm_utils/Machines/env_mach_specific.${machine}
printf '%s\n' '#!/bin/tcsh' 'source /etc/csh.cshrc' > $machfile

# choose the relevant compile_cesm.sh script
cp -v $cwd/compile_cesm.sh.default ./compile_cesm.sh

./compile_cesm.sh $machine clean
test $? -eq 0 || { echo "Error: ./compile_cesm.sh clean failed"; exit 15; }

./compile_cesm.sh $machine build
test $? -eq 0 || { echo "Error: ./compile_cesm.sh build failed"; exit 15; }

test -d $dir_bin_cesm && rm -rf $dir_bin_cesm
mkdir -p $dir_bin_cesm
cp -v $dir_cesm/scripts/clm4_5_${machine}/bld/cesm.exe $dir_bin_cesm

printf "\nInfo: CESM compiled successfully\n\n"

###################
# Compile libgrib
###################
cd $cwd
test -f $dir_apps/libgrib/libgrib1.a || ./build-libgrib.sh $cwd $dir_apps

###################
# Compile COSMO
###################
cd $dir_cosmo
fopts_cosmo_bak=$cwd/Fopts.COSMO.nemo
fopts_cosmo=$dir_cosmo/Fopts.${machine}
test -f $fopts_cosmo && rm -f $fopts_cosmo
test -f $fopts_cosmo_bak || { echo "Error: file not found: $fopts_cosmo_bak"; exit 16; }
cp -v $fopts_cosmo_bak $fopts_cosmo
test -d $dir_bin_cosmo && rm -rf $dir_bin_cosmo
mkdir -p $dir_bin_cosmo

update="HOME_LIB = $dir_cosmo"
sed -i "s|HOME_LIB = <set>|$update|g" $fopts_cosmo

update="GRIB_LIB = $dir_apps/libgrib/libgrib1.a"
sed -i "s|GRIB_LIB = <set>|$update|g" $fopts_cosmo

update="OASDIR = $dir_install_oasis"
sed -i "s|OASDIR = <set>|$update|g" $fopts_cosmo

update="PROGRAM = $dir_bin_cosmo/cclm"
sed -i "s|PROGRAM = <set>|$update|g" $fopts_cosmo

makefile_bak=$cwd/Makefile.COSMO
makefile=$dir_cosmo/Makefile
test -f $makefile && rm -f $makefile
cp -v $makefile_bak $makefile
sed -i 's|<set>|src_paraso|g' $makefile

rm -rf ObjDependencies; ln -s ObjDependencies_src_paraso ObjDependencies
rm -rf ObjFiles; ln -s ObjFiles_src_paraso ObjFiles
rm -rf Fopts; ln -s Fopts.$machine Fopts

make clean 
make -j 4
test $? -eq 0 || { echo "Error make for COSMO failed"; exit 17; }
printf "\nInfo: COSMO compiled successfully\n\n"

###################
# Compile XIOS1 OR 2
###################
cd $dir_xios

conf_xios_bak="$cwd/config.fcm.XIOS"
conf_xios="$dir_xios/config.fcm"
test -f $conf_xios_bak || { echo "Error: file not found: $conf_xios_bak"; exit 18; }
cp -v $conf_xios_bak $conf_xios

sed -i "s|<netcdf-path>|$EBROOTNETCDF|g" $conf_xios
sed -i "s|<oasis-path>|$dir_install_oasis|g" $conf_xios

arch_path_bak="$cwd/arch-intel_oasis.path.XIOS"
arch_path="$dir_xios/arch/arch-intel_${machine}_oasis.path"
test -f $arch_path_bak || { echo "Error: file not found: $arch_path_bak"; exit 20; }
cp -v $arch_path_bak $arch_path

sed -i "s|<oasis-path>|$dir_install_oasis|g" $arch_path

bld_cfg_bak="$cwd/parsed_bld.cfg.XIOS"
bld_cfg="$dir_xios/cfg/parsed_bld.cfg"
test -f $bld_cfg_bak || { echo "Error: file not found: $bld_cfg_bak"; exit 19; }
cp -v $bld_cfg_bak $bld_cfg

sed -i "s|<xios-path>|$dir_xios|g" $bld_cfg
sed -i "s|<oasis-path>|$dir_install_oasis|g" $bld_cfg
sed -i "s|<netcdf-path>|$EBROOTNETCDF|g" $bld_cfg
# sed -i "s|<netcdf-src-path>|<netcdf-src-path>|g" $bld_cfg  # this is problematic!

[[ -d $dir_xios/.void_dir ]] || mkdir -p $dir_xios/.void_dir
[[ ! -L $dir_xios/extern/netcdf4 ]] && ln -s $dir_xios/.void_dir/ $dir_xios/extern/netcdf4

test -L $dir_xios/arch.fcm && ./tools/FCM/bin/fcm build --clean
./make_xios --arch intel_${machine}_oasis  --use_oasis oasis3_mct --netcdf_lib netcdf4_par --job 6
test $? -eq 0 || { echo "Error building Xios1 failed"; exit 18; }

test -d $dir_install_xios && rm -rf $dir_install_xios
mkdir -p $dir_install_xios
cp -rv $dir_xios/bin $dir_xios/inc $dir_xios/lib $dir_install_xios

printf "\nInfo: Xios1 compiled successfully\n\n"

###################
# Compile NEMO
###################
cd $dir_nemo/CONFIG/PARASO/
# fix possible broken symlink
[[ -e cpp_PARASO.fcm ]] || ln -s cpp_PARASO.fcm_xios1 cpp_PARASO.fcm

cd $cwd
fcm_bak="$cwd/arch-xios1_oasis3_2.fcm.NEMO"
fcm="$dir_nemo/ARCH/arch-${machine}_xios1_oasis3_2.fcm"
test -f $fcm_bak || { echo "Error: $fcm_bak not found!"; exit 19; }
cp -v $fcm_bak $fcm

from='<path-xios>'
to="$dir_install_xios"
sed -i "s|$from|$to|g" $fcm

from='<path-oasis>'
to="$dir_install_oasis"
sed -i "s|$from|$to|g" $fcm

cd $dir_nemo/CONFIG
./makenemo -n PARASO -m ${machine}_xios1_oasis3_2 -j6

test -d $dir_install_nemo_cclm && rm -rf $dir_install_nemo_cclm
mkdir -p $dir_install_nemo_cclm/CONFIG/SHARED
cp -rv $dir_nemo/CONFIG/PARASO/BLD/bin $dir_install_nemo_cclm
cp -rv $dir_nemo/CONFIG/SHARED/* $dir_install_nemo_cclm/CONFIG/SHARED

printf "\nInfo: NEMO is compiled successfully in %s\n\n" $dir_install_nemo

###################
# Copy f.ETISh
###################

mkdir -p $dir_install_fetish
printf '\nInfo: copying pre-compiled f.ETISh executable\n\n'
cp -rv ${dir_fetish}/bin $dir_install_fetish
cp -rv ${dir_fetish}/launch $dir_install_fetish

printf '\nInfo: fETISh successfully built\n\n'

###################
# Compile f.ETISh - NEMO coupling tools
###################

cd $dir_nemo/TOOLS
./maketools -n REBUILD_NEMO -m breniac_xios1_oasis3_2

mkdir -p $dir_install_nemo_cclm/EXTERNAL 
cp -rv $dir_nemo/EXTERNAL/isfcpl_scripts $dir_install_nemo_cclm/EXTERNAL
mkdir -p $dir_install_nemo_cclm/TOOLS
cp -rv $dir_nemo/TOOLS/REBUILD_NEMO $dir_install_nemo_cclm/TOOLS
rm -rf $dir_install_nemo_cclm/TOOLS/BLD
rm -rf $dir_install_nemo_cclm/TOOLS/src

dir_rebuild_nemo=$dir_install_nemo_cclm/TOOLS/REBUILD_NEMO
rm -f $dir_rebuild_nemo/rebuild_nemo.exe # symlink
cp $dir_nemo/TOOLS/REBUILD_NEMO/BLD/bin/rebuild_nemo.exe $dir_rebuild_nemo

cat > $dir_rebuild_nemo/fcm_env.sh << EOF
#!/bin/bash
export PATH=${dir_install_nemo_cclm}/TOOLS/REBUILD_NEMO:\$PATH
EOF

ln -s $dir_rebuild_nemo/fcm_env.sh $dir_rebuild_nemo/fcm_env.ksh 

printf '\nInfo: f.ETISH support is successfully built\n\n'

###################
# Done!
###################

printf "\nInfo: NEMO-CCLM is compiled successfully in %s\n\n" $dir_apps

