#! /bin/bash -l

if [ $# -gt 2 ] || [ $# -lt 2 ]; then
    echo "Usage:"
    echo "./compile_cesm.sh machine action"
    echo ""    
    echo "where machine="genius" or "lemaitre3""
    echo "where action="clean" or "build""
  exit
fi

echo CCSMROOT set to $CCSMROOT

CLM_CUT_OUT=0
expname="prod"
CESMV=cesm1_2_0

ADJUSTED_ROUTINES="$PWD/adjusted_routines"
OAS_INT_DIR="$PWD/oas_interface_cesm/$CESMV"
export CCSMROOT="$PWD"
BUILDDIR="${CCSMROOT}/scripts"

cd $BUILDDIR

MACH=$1
RES=1.9x2.5_gx1v6
COMP=ITEST
PHYS=4_5
EXP=clm$PHYS'_'$MACH
COMPILER=intel

# Remove the experiment dir
rm -rf $EXP

echo "create_newcase -v -res $RES -compset $COMP -mach $MACH -compiler $COMPILER -case $BUILDDIR/$EXP"
./create_newcase -v -res $RES -compset $COMP -mach $MACH -compiler $COMPILER -case $BUILDDIR/$EXP

cd $BUILDDIR/$EXP

./xmlchange RTM_MODE="NULL"
./xmlchange -file env_build.xml -id CLM_CONFIG_OPTS -val "-phys clm$PHYS"

./cesm_setup

export oas_clm_define=$OAS_INT_DIR/src/oas/oas_clm_define_ED.F90

ln -sf $ADJUSTED_ROUTINES/histFileMod.F90 SourceMods/src.clm/
ln -sf $OAS_INT_DIR/src/oas/* SourceMods/src.drv/
rm  SourceMods/src.drv/oas_clm_vardef.F90
rm  SourceMods/src.drv/oas_clm_define_*.F90
ln -sf $oas_clm_define SourceMods/src.drv/oas_clm_define.F90
ln -sf $OAS_INT_DIR/src/drv/* SourceMods/src.drv/
ln -sf $OAS_INT_DIR/src/oas/oas_clm_vardef.F90 SourceMods/src.share/
ln -sf $OAS_INT_DIR/src/datm/* SourceMods/src.datm/

echo "PWD: $PWD"
echo "EXP.build= $EXP.build"

if [ "$2" = "build" ]; then
   ./$EXP.clean_build all
   ./$EXP.build
   if [ $? -ne 0 ]; then
      echo "ERROR building CESM"
      exit 1
   else
      echo ""
      echo "CESM successfully built in $BUILDDIR/$EXP/bld/cesm.exe"
      echo ""
   fi
elif [ "$2" = "clean" ]; then
   ./$EXP.clean_build all
else
   echo "Arg action should be "build" or "clean""
fi
