   SUBROUTINE oas_clm_rcv( kid, kstep, pdata, kinfo )

      !!---------------------------------------------------------------------
      !!              ***  ROUTINE oas_clm_rcv  ***
      !!
      !! ** Purpose : - At each coupling time-step,this routine calls fields
      !!                from the coupler or remote application.
      !!----------------------------------------------------------------------

      USE shr_kind_mod , only : r8 => shr_kind_r8
      USE oas_clm_vardef
      USE mod_prism

      IMPLICIT NONE

      INTEGER,                          INTENT( IN  )   :: kid    ! variable intex in the array
      INTEGER,                          INTENT( IN  )   :: kstep  ! time-step in seconds
      REAL(r8), DIMENSION(grid_size),   INTENT( OUT )   :: pdata
      INTEGER,                          INTENT( OUT )   :: kinfo  ! OASIS info argument
      !!
      LOGICAL                                           :: llaction
      REAL(r8), DIMENSION(grid_size)                    :: ztmp1
      !!--------------------------------------------------------------------
      !
      !  Masked point are not modified by OASIS
      !  = buffer set to 0 before calling oasis
      ztmp1=0.
      !
      ! receive data from OASIS
      !
      CALL prism_get_proto( srcv(kid)%nid, kstep, ztmp1, kinfo )         

      llaction = .false.
      IF( kinfo == PRISM_Recvd   .OR. kinfo == PRISM_FromRest .OR.   &
          kinfo == PRISM_RecvOut .OR. kinfo == PRISM_FromRestOut .OR. &
          kinfo == PRISM_Input )   llaction = .TRUE.

      IF ( IOASISDEBUGLVL == 1 ) THEN
         WRITE(NULOUT,*) "llaction, kinfo, kstep, ivarid: " , llaction, kinfo, kstep, srcv(kid)%nid
         CALL FLUSH(NULOUT)
      ENDIF

      IF ( llaction ) THEN

         ! Declare to calling routine that OASIS provided coupling field
         kinfo = OASIS_Rcv

         ! Update array which contains coupling field (only on valid shape)
         pdata(:) = ztmp1(:)
         
      ELSE
         ! Declare to calling routine that OASIS did not provide coupling field
         kinfo = OASIS_idle     
      ENDIF


   END SUBROUTINE oas_clm_rcv
