MODULE oas_clm_vardef

!**** *oas_clm_vardef * - Controls, definitions and variables
!                   for OASIS communications
!
!     AUTHOR.
!     -------
!     2007-1-30 Kristian Mogensen

!     MODIFICATIONS.
!     --------------
!     2010-3  Eric Maisonnave: OASIS4
!     2010-11 Eric Maisonnave: COSMO-CLM coupling
!

   USE shr_kind_mod , only : r8 => shr_kind_r8
   USE mpi

   IMPLICIT NONE

   SAVE

   ! Debug level of OASIS
   !     0 : Minimum debugging
   !     1 : Debugging
   !     2 : CLM produces mask file

   INTEGER :: IOASISDEBUGLVL = 2

   ! Variable ids

   INTEGER            :: ncomp_id           ! id returned by prism_init_comp
   INTEGER            :: kl_comm            ! Local communicator 
   INTEGER            :: nerror             ! return error code
   INTEGER, PUBLIC    :: OASIS_Rcv  = 1     ! return code if received field
   INTEGER, PUBLIC    :: OASIS_idle = 0     ! return code if nothing done by oasis
   INTEGER, PUBLIC    :: PRISM_Success = 0  ! return code if no error in oasis

   INTEGER, PARAMETER :: nmaxfld=40    ! Maximum number of coupling fields
   INTEGER            :: ksnd=6, krcv=16    ! Number of send/received coupling fields

   TYPE, PUBLIC ::   FLD_CPL            ! Type for coupling field information
      LOGICAL            ::   laction   ! To be coupled or not
      CHARACTER(len =12) ::   clname    ! Name of the coupling field
      CHARACTER(len = 1) ::   clgrid    ! Grid type
      INTEGER            ::   nid       ! Id of the field
   END TYPE FLD_CPL

   INTEGER :: grid_size = 1
   INTEGER, PARAMETER :: NULOUT=6
   INTEGER :: global_2d_dim(2)

   TYPE(FLD_CPL), DIMENSION(nmaxfld), PUBLIC :: srcv, ssnd   ! Coupling fields

   REAL(r8), DIMENSION(:,:), ALLOCATABLE :: exfld  ! Temporary buffer for sending and receiving
   INTEGER, DIMENSION(:), ALLOCATABLE :: oasis_mask  ! Model Mask for oasis
   REAL(r8), DIMENSION(:), ALLOCATABLE :: oasis_lat  ! Model Latitudes for oasis
   REAL(r8), DIMENSION(:), ALLOCATABLE :: oasis_lon  ! Model Longitudes for oasis
   REAL(r8), DIMENSION(:), ALLOCATABLE :: oasis_areas  ! Model areas for oasis

   !  Received fields index

   INTEGER, PARAMETER ::   jps_t   =  1            ! temperature
   INTEGER, PARAMETER ::   jps_u   =  2            ! u wind
   INTEGER, PARAMETER ::   jps_v   =  3            ! v wind
   INTEGER, PARAMETER ::   jps_q   =  4            ! specific water vapor content
   INTEGER, PARAMETER ::   jps_th  =  5            ! thickness of lowest level (m)
   INTEGER, PARAMETER ::   jps_pr  =  6            ! surface pressure (Pa)
   INTEGER, PARAMETER ::   jps_rs  =  7            ! direct shortwave downward radiation (W/m2)
   INTEGER, PARAMETER ::   jps_fs  =  8            ! diffuse shortwave downward radiation (W/m2)
   INTEGER, PARAMETER ::   jps_lw  =  9            ! longwave downward radiation (W/m2) 
   INTEGER, PARAMETER ::   jps_cr  = 10            ! convective rain precipitation      (kg/m2*s)
   INTEGER, PARAMETER ::   jps_cs  = 11            ! convective snow precipitation      (kg/m2*s)
   INTEGER, PARAMETER ::   jps_gr  = 12            ! gridscale rain precipitation
   INTEGER, PARAMETER ::   jps_gs  = 13            ! gridscale snow precipitation
   INTEGER, PARAMETER ::   jps_gg  = 14            ! gridscale graupel precipitation
   INTEGER, PARAMETER ::   jps_cp  = 15            ! total convective precipitation
   INTEGER, PARAMETER ::   jps_gp  = 16            ! total gridscale precipitation


   !  Send fields index

   INTEGER, PARAMETER ::   jps_taux   =  1    !  zonal wind stress
   INTEGER, PARAMETER ::   jps_tauy   =  2    !  meridional wind stress
   INTEGER, PARAMETER ::   jps_lat    =  3    !  total latent heat flux (W/m**2)
   INTEGER, PARAMETER ::   jps_sens   =  4    !  total sensible heat flux (W/m**2)
   INTEGER, PARAMETER ::   jps_ir     =  5    !  emitted infrared (longwave) radiation (W/m**2)
   INTEGER, PARAMETER ::   jps_alb    =  6    !  albedo

END MODULE oas_clm_vardef
