   SUBROUTINE receive_fld_2cos(x, kt, lcoupled)

   !!---------------------------------------------------------------------
   !!              ***  ROUTINE send_fld_2cos  ***
   !!
   !! ** Purpose : - Prepare and send coupling fields
   !!      to COSMO atmosphere model
   !!----------------------------------------------------------------------

   USE oas_clm_vardef
   USE shr_kind_mod ,            only : r8 => shr_kind_r8
   USE spmdMod ,                 only : masterproc, SPMD_MPI_LOGICAL => MPI_LOGICAL

   USE netcdf

   IMPLICIT NONE

   INTEGER, intent(in)          :: kt              ! time step in seconds
   LOGICAL, intent(out)         :: lcoupled(krcv)  ! TRUE if field coupled
   REAL(kind=r8), intent(inout) :: x(grid_size,krcv)  ! coupling fields array

   INTEGER :: jn, ier
   INTEGER , DIMENSION(krcv) ::   nrcvinfo           ! OASIS info argument
   INTEGER , DIMENSION(krcv) ::   il_var_id
   INTEGER :: ji, jj
   INTEGER :: il_file_id, mype, npes, ib, dimids(2)

   REAL(kind=r8), DIMENSION(grid_size) ::   ztmp1   

   ! Default value (changed with namcouple OASIS parameter file)
   lcoupled(:) = srcv(:)%laction

   ! Default value (changed when coupling)
   nrcvinfo = OASIS_idle

   DO jn = 1, krcv
      !
      IF( srcv(jn)%laction ) THEN

         CALL oas_clm_rcv( jn, kt, ztmp1(:), nrcvinfo(jn) )

         ! Update arrays which contains coupling fields only at coupling time step
         ! exfld array keep coupling field at other time steps
         IF( nrcvinfo(jn) == OASIS_Rcv ) exfld(:,jn)=ztmp1(:)

      ENDIF

   ENDDO

   ! Write coupling fields for new mask setting
   IF ( ( IOASISDEBUGLVL == 2 ) .and. masterproc ) then

      nerror =  nf90_create("mask_clm.nc", NF90_CLOBBER, il_file_id)
      nerror =  nf90_def_dim(il_file_id, "ni", global_2d_dim(1), dimids(1))
      nerror =  nf90_def_dim(il_file_id, "nj", global_2d_dim(2), dimids(2))
      nerror =  nf90_def_var(il_file_id, "mask", NF90_INT, dimids, il_var_id(1))
      nerror =  nf90_enddef(il_file_id)

      ! mask production case: invert received field
      ! following CLM convention:
      ! 0: masked
      ! 1: unmasked

      where ( exfld(:,1) .gt. .5 ) 
         exfld(:,1) = 1.1
      elsewhere
         exfld(:,1) = 0.1
      endwhere
      nerror = nf90_put_var(il_file_id, il_var_id(1), INT(RESHAPE(exfld(:,1),global_2d_dim)))
      nerror =  nf90_close(il_file_id)

      write(6,*) 'CLM produces mask on mask_clm.nc file and stops'
      call flush(6) 
      STOP

   ENDIF

   ! Update x array at each timestep
   ! - with OASIS received coupling field if coupling time step   
   ! - with saved exfld array if not (value of previous coupling step)   
   x(:,:) = exfld(:,:)

! Uncomment for debug only:
! incoming fields are made constant for CLM point of view
!     x(:,1) = 288.15
!     x(:,2) = sqrt(2.)
!     x(:,3) = sqrt(2.)
!     x(:,4) = 0.009
!     x(:,5) is never updated 
!     x(:,6) = 102360.
!     x(:,7) = 0.
!     x(:,8) = 0.
!     x(:,9) = 365.
!     x(:,10) = 0.
!     x(:,11) = 0.
!     x(:,12) = 0.
!     x(:,13) = 0.
!     x(:,14) = 0.
!     x(:,15) = 0.
!     x(:,16) = 0.

   END SUBROUTINE receive_fld_2cos
