function [par]=FetishInputParamsPARAMOUR(m)


%-----------------------------------
% General control parameters
%-----------------------------------

par.plotcolor='*RdBu'; % Brewermap colorscale
par.plotH=0; % Plot evolution of ice sheet instead of ice-sheet changes


%-----------------------------------
% Numerical control parameters
%-----------------------------------

par.maxspeed=20e3; % maximum ice speed limit (m/a)
par.upstream=1; % upstream differences in solution of ice thickness equation
par.omega=2.5; % Crank-Nicholson scale factor (0=explicit; 1=implicit; >1 over-implicit)
par.secperyear=31556926;


%-----------------------------------
% Subglacial characteristics
%-----------------------------------

par.u0=100.; % (Coulomb)
par.phi=30; % nominal value of till friction angle (degrees)
par.PoreFrac=0.96; % Fraction of water pressure to balance ice pressure (0.96)
par.longcoupwater=5; % distance in number of ice thicknesses over which
                 % hydraulic gradient coupling takes place
par.dirpp_war=[9 8 7 6 5 4 3 2 1];
par.waterviscosity=1.8e-3/par.secperyear;
par.NeffScale=0.5e7; % scale factor for Effective Pressure
par.Wdmin=1e-8; % minimum value for Wd and Wtil
par.Wdmax=0.015; % maximum value for Wd
par.Wmax=2; % maximum value for Wtil (2 m)
par.flw0=1e5; % maximum value for subglacial water flux
par.Cdr=1e-3; % background till drainage rate
par.Cc=0.12; % till compressibility (Tulaczyk et al., 2000a)
par.e0=0.69; % reference void ratio at N0(Tulaczyk et al., 2000a)
par.N0=1e3; % reference effective pressure (Tulaczyk et al., 2000a)
par.sigmat=0.02; % Ntil lower bound, as fraction of overburden pressure

%-----------------------------------
% Ice dynamics
%-----------------------------------

par.SuctionCheck=0;
par.ShelfPinning=1; % sub-shelf pinning of ice shelves based on bedrock variability
par.g=9.81; % gravitational acceleration
par.rho=917.; % ice density
par.rhow=1026.; % sea water density
par.rhom=3370.; % lithosphere density
par.n=3; % flow law exponent
par.visciter=50; % max number of iterations in SSA solution
par.Qo=0.61; % Numerical constant in Tsai boundary condition


%-----------------------------------
% Ice-ocean interactions
%-----------------------------------

par.Latent=3.35e5; % Latent heat of freezing
par.cp0=3974.; % Heat capacity of ocean water
par.Soi=34.5; % ocean salinity for initialization
par.Toi=-1.7; % Ocean temperature for initialization
par.SeaIceThickness=0.1;
par.ArcOcean=0; % include Ocean Arc to control Melt and calving
    
%-----------------------------------
% PICO and plume model parameters
%-----------------------------------

par.nbox=5; % max number of ocean boxes
par.alphao=7.5e-5;
par.betao=7.7e-4;
par.rhoref=1033;
par.C=1e6;

par.lambda1=-5.73e-2;
par.lambda2=8.32e-2;
par.lambda3=7.61e-4;
par.gamma0=1.447733676e4;
par.gamma1=0.545;
par.gamma2=3.5e-5;
par.CdGamT=1.1e-3;
par.CdGamTS0=6e-4;
par.Eo=3.6e-2;
par.Cd=2.5e-3;
par.x0=0.56;

% Melt function coefficients (Lazeroms)
par.pcof=[0.1371330075095435
    5.527656234709359e1
    -8.951812433987858e2
    8.927093637594877e3
    -5.563863123811898e4
    2.218596970948727e5
    -5.820015295669482e5
    1.015475347943186e6
    -1.166290429178556e6
    8.466870335320488e5
    -3.520598035764990e5
    6.387953795485420e4];

%-----------------------------------
% Isostasy
%-----------------------------------

par.FlexRigid=1e25; % Flexural rigidity
par.bedrelax=3000.; % relaxation time astenosphere
par.flexdist=500e3; % Flexural distance lithosphere
par.nuB=0.25; % Poisson ratio in flexural rigidity
par.Bclass=8; % Number of flexural rigidity classes if Db is imported


%-----------------------------------
% Local sea level (fingerprints)
%-----------------------------------

par.Re=6.3781e6; % Radius Earth
par.Me=5.972e24; % Mass Earth
par.Aoc=3.618e14; % ocean surface
par.geoidist=5000e3; % size of convolution filter
par.rhof=1000; % fresh water density
par.rhoL=2830; % mean lithosphere density
par.SLref=0; % reference sea level

%-----------------------------------
% Model initialization
%-----------------------------------

par.RignotInverse=1; % Use Rignot velocities for calculating Theta in shelf
par.SmoothInv=1;
par.SmoothSpan=48; % Span of Savitsky-Golay filter in km (150)
par.invmin=1.e-10; % values valid for m=2; scaled with 150kPa for other m
par.invmax=1.e-5;
par.AsFroz=1e-11;
par.AsScale=1e5^(2-m);

%-----------------------------------
% Thermodynamics
%-----------------------------------

par.T0=273.15; % absolute temperature
par.K=2.1; % thermal conductivity
par.kdif=1.1487e-6; % kdif=K/(rho*cp), cp=2009; value of EISMINT
par.pmp=8.66e-4; % Clausius Clapeyron (Payne, 2000)
par.atune=1.0; % tuning factor in Arrhenius
par.a1=3.985e-13*par.secperyear; % Greve and Blatter (correction from Huybrechts)
par.a2=1.916e3*par.secperyear;
par.Q1=60e3;
par.Q2=139e3;
par.R=8.314; % gas constant
par.udfrac=0.25;
par.intT=1; % number of iterations for which tmp is calculated
par.TrTemp=-10.0; % Basal temperature for which ice is frozen to bed

%-----------------------------------
% PDD model parameters
%-----------------------------------

par.snowfac=0.003; % PDD factor for snow melt
par.icefac=0.008; % PDD factor for ice melt
par.refreeze=2; % Maximum depth of refreezing of percolating meltwater (m)
par.Tlapse=-0.008; % Lapse rate for temperature correction with height
par.Tsigma=5;
par.PDDsteps=12;

end