function create_NetCDF(FileNameIn,FileNameOut)

load([FileNameIn '_toto'],'X','Y','lat','lon','Bor','H','HB','sn','MASK','ctr','par');

MASK(MASK==0 & H>par.SeaIceThickness)=3;
H(MASK==0 & H==par.SeaIceThickness)=0;

% make sure MASK is constant
    load([FileNameIn '_toto'],'MASKo');
    H(MASKo==0 & MASK~=0)=0;
    HB(MASKo==0 & MASK~=0)=0;
    sn(MASKo==0 & MASK~=0)=0;
    MASK(MASKo==0 & MASK~=0)=0;
    H(MASKo~=0 & MASK==0)=10*par.rhow/par.rho;  % 10m = min. ice shelf draft for NEMO
    HB(MASKo~=0 & MASK==0)=max(-par.rho/par.rhow*H(MASKo~=0 & MASK==0),Bor(MASKo~=0 & MASK==0));
    sn(MASKo~=0 & MASK==0)=H(MASKo~=0 & MASK==0)+HB(MASKo~=0 & MASK==0);
    MASK(MASKo~=0 & MASK==0)=3;
    MASK(MASKo~=0 & MASK==0 & HB==Bor)=1;


HB=-HB;
HB(MASK==1)=-1;

X=(X-2800)*1000;
Y=(Y-2800)*1000;

ncid = netcdf.create([FileNameOut '.nc'],'CLASSIC_MODEL');

%Define the dimensions
dimix = netcdf.defDim(ncid,'x',ctr.imax);
dimiy = netcdf.defDim(ncid,'y',ctr.jmax);

%create global attributes
varid = netcdf.getConstant('GLOBAL');
netcdf.putAtt(ncid,varid,'Explanation',['fETISh geometry from ' FileNameIn]);

%Define variables:

% X:
    X_ID=netcdf.defVar(ncid,'X','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,X_ID,X);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'X');
    netcdf.putAtt(ncid,varid,'long_name','polar stereographic x coordinate');
    netcdf.putAtt(ncid,varid,'unit','meters "east" of south pole (following 90E Meridian)');
    netcdf.putAtt(ncid,varid,'resolution',[num2str(ctr.delta/1000) ' km']);

% Y:
    Y_ID=netcdf.defVar(ncid,'Y','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,Y_ID,Y);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'Y');
    netcdf.putAtt(ncid,varid,'long_name','polar stereographic y coordinate');
    netcdf.putAtt(ncid,varid,'unit','meters "north" of south pole (following Prime Meridian)');
    netcdf.putAtt(ncid,varid,'resolution',[num2str(ctr.delta/1000) ' km']);
    
% lat:
    lat_ID=netcdf.defVar(ncid,'lat','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,lat_ID,lat);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'lat');
    netcdf.putAtt(ncid,varid,'long_name','Latitude');

% lon:
    lon_ID=netcdf.defVar(ncid,'lon','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,lon_ID,lon);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'lon');
    netcdf.putAtt(ncid,varid,'long_name','Longitude');

% Bed:
    B_ID=netcdf.defVar(ncid,'B','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,B_ID,Bor);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'B');
    netcdf.putAtt(ncid,varid,'long_name','Bathymetry');
    netcdf.putAtt(ncid,varid,'unit','meters above sea level');
    netcdf.putAtt(ncid,varid,'explanation','MEaSUREs BedMachine Antarctica, Version 1; M. Morlighem et al. ');
    netcdf.putAtt(ncid,varid,'link','https://nsidc.org/data/nsidc-0756');
    netcdf.putAtt(ncid,varid,'citation','Morlighem, M., Rignot, E., Binder, T. et al. Deep glacial troughs and stabilizing ridges unveiled beneath the margins of the Antarctic ice sheet. Nat. Geosci. 13, 132137 (2020). https://doi.org/10.1038/s41561-019-0510-8');
    
% MASK:    
    MASK_ID=netcdf.defVar(ncid,'MASK','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,MASK_ID,MASK);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'MASK');
    netcdf.putAtt(ncid,varid,'explanation','0=ocean, 1=grounded_ice, 3=ice_shelf');
    
% ice thickness:
    H_ID=netcdf.defVar(ncid,'thickness','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,H_ID,H);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'thickness');
    netcdf.putAtt(ncid,varid,'long_name','ice thickness');
    netcdf.putAtt(ncid,varid,'unit','meters');

% ice shelf draft:
    HB_ID=netcdf.defVar(ncid,'draft','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,HB_ID,HB);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'draft');
    netcdf.putAtt(ncid,varid,'long_name','ice shelf draft');
    netcdf.putAtt(ncid,varid,'unit','meters below sea level');
    netcdf.putAtt(ncid,varid,'explanation','ice shelf draft in meters below sea level (based on floatation criterion)');
    netcdf.putAtt(ncid,varid,'open_ocean','draft = 0');
    netcdf.putAtt(ncid,varid,'grounded_ice','draft = -1');
    netcdf.putAtt(ncid,varid,'ice_shelves','draft = ice_thickness * (rho_ice / rho_seawater)');
    netcdf.putAtt(ncid,varid,'used_densities',['rho_ice = ' num2str(par.rho) ' kg/m^3; rho_seawater = ' num2str(par.rhow) ' kg/m^3']);

% surface elevation:   
    Surf_ID=netcdf.defVar(ncid,'surface','double',[dimix dimiy]);
    netcdf.endDef(ncid);
    netcdf.putVar(ncid,Surf_ID,sn);
    netcdf.reDef(ncid);
    varid = netcdf.inqVarID(ncid,'surface');
    netcdf.putAtt(ncid,varid,'long_name','surface elevation');
    netcdf.putAtt(ncid,varid,'unit','meters above sea level');
    
        
%We're done, close the netcdf
netcdf.close(ncid)

end