function fETISh_Coupling_PARAMOUR_V171

% read in fETISh definitions file to define file names of relaxed fETISh
% input file and tracking file
[fETIShDef]=read_fETIShDef('definitions.fETISh');

% load coupling variables from tracking_file
[trackDef]=read_trackVars(fETIShDef.tracking_file);

if trackDef.ice_atm_cpl==1
    % create COSMO input name
    COSMOinput=[trackDef.run_name '_atmosphere2ice_' num2str(trackDef.date_beg_isfcpl) '-' num2str(trackDef.date_end_isfcpl) '.nc'];
    % convert COSMO input to SMB data on fETISh grid
    COSMOsmb=convertCosmoSMB(COSMOinput,fETIShDef.fETIShRelaxFile,trackDef.n_month_isfcpl);
else
    % create dummy COSMO input for no COSMO coupling
    load([fETIShDef.fETIShRelaxFile '_toto'],'ctr');
    COSMOsmb=zeros(ctr.imax,ctr.jmax,trackDef.n_month_isfcpl);
    clear ctr
end

% create NEMO input name
NEMOinput=[trackDef.run_name '_ocean2ice_' num2str(trackDef.date_beg_isfcpl) '-' num2str(trackDef.date_end_isfcpl) '.nc'];

% convert NEMO input to Melt data on fETISh grid
NEMOmelt=convertNemoMelt(NEMOinput,fETIShDef.fETIShRelaxFile,trackDef.n_month_isfcpl);

trackDef.DateOut=floor(trackDef.date_beg_isfcpl/100);

if isequal(trackDef.initial_date,trackDef.date_beg_isfcpl)
    trackDef.DateIn=trackDef.DateOut;
    [ctr,fc,infile,outfile]=createRelaxedStartFile(fETIShDef,trackDef,NEMOmelt(:,:,1),COSMOsmb(:,:,1)); % create relaxed start file
else
    if trackDef.DateOut-floor(trackDef.DateOut/100)*100==1 % if January
        trackDef.DateIn=trackDef.DateOut-89;
    else
        trackDef.DateIn=trackDef.DateOut-1;
    end
    [ctr,fc,infile,outfile]=createTotoFile(trackDef,NEMOmelt(:,:,1),COSMOsmb(:,:,1)); % create toto start file for next coupling step
end
FetishModel2020julyLZbPARAMOUR(ctr,fc,infile,outfile);

if trackDef.n_month_isfcpl>1
    for i=2:trackDef.n_month_isfcpl
        trackDef.DateIn=trackDef.DateOut;
        if trackDef.DateIn-floor(trackDef.DateIn/100)*100==12 % if December
            trackDef.DateOut=trackDef.DateIn+89;
        else
            trackDef.DateOut=trackDef.DateIn+1;
        end
        [ctr,fc,infile,outfile]=createTotoFile(trackDef,NEMOmelt(:,:,i),COSMOsmb(:,:,i)); % create toto start file for next coupling step
        FetishModel2020julyLZbPARAMOUR(ctr,fc,infile,outfile);
    end
end

% create fETISh output nc file
create_NetCDF(outfile,[trackDef.run_name '_ice2ocean_' num2str(trackDef.date_end_isfcpl)]);

end