# namelist.nemo-ORCA1L75.cfg.sh writes the NEMO namelist for ORCA1L75 in
# This namelist will overwrite the reference namelist (namelist.nemo.ref.sh). 
#
if $leg_is_restart || $special_restart || ${special_rs_nemo}
then
    nemo_restart=".TRUE."
else
    nemo_restart=".FALSE."
fi

if $special_restart || ${special_rs_nemo}
then
    check_restart_date=0
else
    check_restart_date=1
fi


cat << EOF
!!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!! NEMO/OPA  Configuration namelist : used to overwrite defaults values defined in SHARED/namelist_ref
!!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!
!-----------------------------------------------------------------------
&namrun        !   parameters of the run
!-----------------------------------------------------------------------
   cn_exp        =  "${exp_name}" !  experience name
   nn_it000      =  $(( leg_start_sec / nem_time_step_sec + 1 )) !  first time step
   nn_itend      =  $(( leg_end_sec / nem_time_step_sec ))       !  last  time step (std 5475)
   nn_date0      =  ${leg_start_date_yyyymmdd} !  date at nit_0000 (format yyyymmdd)
                                  !    used if ln_rstart=F or
                                  !    (ln_rstart=T and nn_rstctl=0 or 1)
   ln_rstart     = ${nemo_restart} !  start from rest (F) or from a restart file (T)
   nn_rstctl     = ${check_restart_date}     !  restart control ==> activated only if ln_rstart=T
                                  !  = 0 nn_date0 read in namelist ; nn_it000 : read in namelist
                                  !  = 1 nn_date0 read in namelist ; nn_it000 : check consistancy between namelist and restart
                                  !  = 2 nn_date0 read in restart  ; nn_it000 : check consistancy between namelist and restart
   cn_ocerst_in  = "restart_oce"  !  suffix of ocean restart name (input)
   cn_ocerst_out = "restart_oce"  !  suffix of ocean restart name (output)
   nn_stock      =  0             !  frequency of creation of a restart file (modulo referenced to 1)
   nn_write      = -1             !  frequency of write in the output file   (modulo referenced to nn_it000)
   ln_iscpl = .false.
   ln_clobber = .true.
   nn_istate = 0
   nn_leapy  = ${leap_years}
/
!-----------------------------------------------------------------------
&namcfg        !   parameters of the configuration
!-----------------------------------------------------------------------
   cp_cfg      =  "paraso"               !  name of the configuration
   jp_cfg      =     025               !  resolution of the configuration
   jpidta      =     1442               !  1st lateral dimension ( >= jpi )
   jpjdta      =     559               !  2nd    "         "    ( >= jpj )
   jpkdta      =      75               !  number of levels      ( >= jpk )
   jpiglo      =     1442               !  1st dimension of global domain --> i =jpidta
   jpjglo      =     559               !  2nd    -                  -    --> j  =jpjdta
   jperio      =       1               !  6 cyclic East-West AND North fold F-point pivot
   ln_use_jattr = .true.               !  use (T) the file attribute: open_ocean_jstart if present
/
!-----------------------------------------------------------------------
&namzgr        !   vertical coordinate
!-----------------------------------------------------------------------
   ln_isfcav = .true.
   ln_zps = .true.
/
!-----------------------------------------------------------------------
&namzgr_sco    !   s-coordinate or hybrid z-s-coordinate
!-----------------------------------------------------------------------
   ln_s_sh94   = .true.    !  Song & Haidvogel 1994 hybrid S-sigma   (T)|
   ln_s_sf12   = .false.   !  Siddorn & Furner 2012 hybrid S-z-sigma (T)| if both are false the NEMO tanh stretching is applied
   ln_sigcrit  = .false.   !  use sigma coordinates below critical depth (T) or Z coordinates (F) for Siddorn & Furner stretch
                           !  stretching coefficients for all functions
   rn_sbot_min =   10.0    !  minimum depth of s-bottom surface (>0) (m)
   rn_sbot_max = 7000.0    !  maximum depth of s-bottom surface (= ocean depth) (>0) (m)
   rn_hc       =  150.0    !  critical depth for transition to stretched coordinates
                        !!!!!!!  Envelop bathymetry
   rn_rmax     =    0.3    !  maximum cut-off r-value allowed (0<r_max<1)
                        !!!!!!!  SH94 stretching coefficients  (ln_s_sh94 = .true.)
   rn_theta    =    6.0    !  surface control parameter (0<=theta<=20)
   rn_bb       =    0.8    !  stretching with SH94 s-sigma
                        !!!!!!!  SF12 stretching coefficient  (ln_s_sf12 = .true.)
   rn_alpha    =    4.4    !  stretching with SF12 s-sigma
   rn_efold    =    0.0    !  efold length scale for transition to stretched coord
   rn_zs       =    1.0    !  depth of surface grid box
                           !  bottom cell depth (Zb) is a linear function of water depth Zb = H*a + b
   rn_zb_a     =    0.024  !  bathymetry scaling factor for calculating Zb
   rn_zb_b     =   -0.2    !  offset for calculating Zb
                        !!!!!!!! Other stretching (not SH94 or SF12) [also uses rn_theta above]
   rn_thetb    =    1.0    !  bottom control parameter  (0<=thetb<= 1)
/
!-----------------------------------------------------------------------
&namzgr_isf    !   isf cavity geometry definition
!-----------------------------------------------------------------------
   rn_isfdep_min    = 10.         ! minimum isf draft tickness (if lower, isf draft set to this value)
   rn_glhw_min      = 1.e-3       ! minimum water column thickness to define the grounding line
   rn_isfhw_min     = 40.         ! minimum water column thickness in the cavity once the grounding line defined.
   ln_isfchannel    = .false.     ! remove channel (based on 2d mask build from isfdraft-bathy)
   ln_isfconnect    = .false.     ! force connection under the ice shelf (based on 2d mask build from isfdraft-bathy)
   nn_kisfmax       = 999         ! limiter in level on the previous condition. (if change larger than this number, get back to value before we enforce the connection)
   rn_zisfmax       = 7000.       ! limiter in m     on the previous condition. (if change larger than this number, get back to value before we enforce the connection)
   ln_isfcheminey   = .true.     ! close cheminey
   ln_isfsubgl      = .true.      ! remove subglacial lake created by the remapping process
   rn_isfsubgllon   =    0.0      !  longitude of the seed to determine the open ocean
   rn_isfsubgllat   =    -50.0      !  latitude  of the seed to determine the open ocean
/
!-----------------------------------------------------------------------
&namdom        !   space and time domain (bathymetry, mesh, timestep)
!-----------------------------------------------------------------------
   nn_msh      =    0                  !  create (=1) a mesh file or not (=0)
   rn_hmin     =    20.                !  min depth of the ocean (>0) or min number of ocean level (<0)
   rn_rdt      = ${nem_time_step_sec} !  time step for the dynamics (and tracer if nn_acc=0)
   ppglam0     =  999999.0             !  longitude of first raw and column T-point (jphgr_msh = 1)
   ppgphi0     =  999999.0             !  latitude  of first raw and column T-point (jphgr_msh = 1)
   ppe1_deg    =  999999.0             !  zonal      grid-spacing (degrees)
   ppe2_deg    =  999999.0             !  meridional grid-spacing (degrees)
   ppe1_m      =  999999.0             !  zonal      grid-spacing (degrees)
   ppe2_m      =  999999.0             !  meridional grid-spacing (degrees)
   ppsur       =  -3958.951371276829   !  ORCA r4, r2 and r05 coefficients
   ppa0        =    103.95300960000000 ! (default coefficients)
   ppa1        =      2.41595126900000 !
   ppkth       =     15.35101370000000 !
   ppacr       =      7.0              !
   ppdzmin     =  999999.              !  Minimum vertical spacing
   pphmax      =  999999.              !  Maximum depth
   ppa2        =  100.7609285000000    !  Double tanh function parameters
   ppkth2      =  48.02989372000000    !
   ppacr2      =  13.00000000000       !
/
!-----------------------------------------------------------------------
&namsplit      !   time splitting parameters  ("key_dynspg_ts")
!-----------------------------------------------------------------------
    rn_bt_cmax = .8
!-----------------------------------------------------------------------
&namcrs        !   Grid coarsening for dynamics output and/or
               !   passive tracer coarsened online simulations
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namc1d        !   1D configuration options                             ("key_c1d")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namtsd    !   data : Temperature  & Salinity
!-----------------------------------------------------------------------
!          !  file name                                            ! frequency (hours) ! variable  ! time interp. !  clim  ! 'yearly'/ ! weights                                         ! rotation ! land/sea mask !
!          !                                                       !  (if <0  months)  !   name    !   (logical)  !  (T/F) ! 'monthly' ! filename                                        ! pairing  ! filename      !
    sn_tem  = 'so025_noiscpl-40y_Mm_misstonn_smooth9_tempsal_1990-2009.nc',                          -1        ,'thetao' ,    .true.    , .true. , 'yearly'  ,'',   ''     ,    ''
   sn_sal  = 'so025_noiscpl-40y_Mm_misstonn_smooth9_tempsal_1990-2009.nc'       ,                   -1        ,'so' ,    .true.    , .true. , 'yearly'  ,'',   ''     ,    ''
   ln_tsd_tradmp = .false.  !  damping of ocean T & S toward T &S input data (T) or not (F)
/
!-----------------------------------------------------------------------
&namsbc        !   Surface Boundary Condition (surface module)
!-----------------------------------------------------------------------
   nn_fsbc     = $(( lim_time_step_sec / nem_time_step_sec )) !  frequency of surface boundary condition computation
                           !     (also = the frequency of sea-ice model call)
   nn_isf      = 1         !  ice shelf melting/freezing                (/=0 => fill namsbc_isf)
                           !  0 =no isf                  1 = presence of ISF
                           !  2 = bg03 parametrisation   3 = rnf file for isf
                           !  4 = ISF fwf specified
                           !  option 1 and 4 need ln_isfcav = .true. (domzgr)
   nn_fwb      = 0         !  FreshWater Budget: =0 unchecked
                           !     =1 global mean of e-p-r set to zero at each time step
                           !     =2 annual global mean of e-p-r set to zero

ln_rnf = .true.
ln_ssr = .false.
ln_blk_core = .true.    ! FK  CORE bulk formulation                     (T => fill namsbc_core)
ln_cpl      = .true.   ! FK  atmosphere coupled   formulation          ( requires key_oasis3 )
nn_limflx   = 3   ! FK 0 ou 2 si couplage SIC
ln_mixcpl = .true. ! FK forced-coupled mixed formulation ( requires key_oasis3 )
/
!-----------------------------------------------------------------------
&namsbc_ana    !   analytical surface boundary condition
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsbc_flx    !   surface boundary condition : flux formulation
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsbc_clio   !   namsbc_clio  CLIO bulk formulae
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsbc_core   !   namsbc_core  CORE bulk formulae
!-----------------------------------------------------------------------
!              !  file name              ! frequency (hours) ! variable ! time interp. !  clim   ! 'yearly'/ ! weights                                     ! rotation ! land/sea mask !
!              !                         !  (if <0  months)  !   name   !   (logical)  !  (T/F)  ! 'monthly' ! filename                                    ! pairing  ! filename      !
 

  sn_wndi     = 'ERA5_29s_3h_drown-smooth_uas'  ,         3         , 'uas'       ,   .true.     , .false. , 'yearly'  , 'weights_bicub_ERA5-29S_to_eORCA025-SO.nc' , 'U1'     , ''
   sn_wndj     = 'ERA5_29s_3h_drown-smooth_vas'  ,         3        , 'vas'       ,   .true.     , .false. , 'yearly'  , 'weights_bicub_ERA5-29S_to_eORCA025-SO.nc' , 'V1'     , ''
   sn_qsr      = 'ERA5_29s_3h_drown-smooth_rsds' ,         3         , 'rsds' ,   .true.     , .false. , 'yearly'  , 'weights_bilin_ERA5-29S_to_eORCA025-SO.nc' , ''       , ''
   sn_qlw      = 'ERA5_29s_3h_drown-smooth_rlds' ,         3         , 'rlds' ,   .true.     , .false. , 'yearly'  , 'weights_bilin_ERA5-29S_to_eORCA025-SO.nc' , ''       , ''
   sn_tair     = 'ERA5_29s_3h_drown-smooth_tas'   ,         3         , 'tas'        ,   .true.     , .false. , 'yearly'  , 'weights_bilin_ERA5-29S_to_eORCA025-SO.nc' , ''       , ''
   sn_humi     = 'ERA5_29s_3h_drown-smooth_huss'  ,         3        , 'huss'       ,   .true.     , .false. , 'yearly'  , 'weights_bilin_ERA5-29S_to_eORCA025-SO.nc' , ''       , ''
   sn_prec     = 'ERA5_29s_3h_drown-smooth_pr' ,         3        , 'pr' ,   .true.     , .false. , 'yearly'  , 'weights_bilin_ERA5-29S_to_eORCA025-SO.nc' , ''       , ''
   sn_snow     = 'ERA5_29s_3h_drown-smooth_prsn' ,         3         , 'prsn' ,   .true.     , .false. , 'yearly'  , 'weights_bilin_ERA5-29S_to_eORCA025-SO.nc' , ''       , ''
   
   rn_zqt      = 2.        !  Air temperature and humidity reference height (m)
   rn_pfac = 1. ! facteur pour les precip
/ 
!-----------------------------------------------------------------------
&namsbc_mfs   !   namsbc_mfs  MFS bulk formulae
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsbc_cpl    !   coupled ocean/atmosphere model                       ("key_coupled")
!-----------------------------------------------------------------------
! send
   sn_snd_temp   =   'oce and ice' ,     'no'    ,     ''      ,         ''           ,   ''
   sn_snd_alb    =   'ice'           ,   'no'    ,     ''      ,         ''           ,   ''
   sn_snd_thick  =   'none'         ,    'no'    ,     ''      ,         ''           ,   ''
   sn_snd_crt    =   'none'         ,    'no'    , 'spherical' , 'eastward-northward' ,  'T'
   sn_snd_co2    =   'none'         ,    'no'    ,     ''      ,         ''           ,   ''
! receive
   sn_rcv_w10m   =   'none'         ,    'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_taumod =   'none'         ,    'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_tau    =   'oce and ice'  ,    'no'    , 'spherical' , 'local grid' ,   'U,V'
   sn_rcv_dqnsdt =   'coupled'      ,    'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_qsr    =   'oce and ice'  ,     'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_qns    =   'oce and ice'  ,     'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_emp    =   'conservative' ,    'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_rnf    =   'none'      ,       'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_cal    =   'none'      ,       'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_iceflx =   'none'      ,       'no'    ,     ''      ,         ''           ,   ''
   sn_rcv_co2    =   'none'         ,    'no'    ,     ''      ,         ''           ,   ''
!
   nn_cplmodel   =     1.     !  Maximum number of models to/from which NEMO is potentialy sending/receiving data
   ln_usecplmask = .true.   !  use a coupling mask file to merge data received from several models
/
!-----------------------------------------------------------------------
&namsbc_sas    !   analytical surface boundary condition
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namtra_qsr    !   penetrative solar radiation
!-----------------------------------------------------------------------
   nn_chldta   =      0    !  RGB : Chl data (=1) or cst value (=0)
/
!-----------------------------------------------------------------------
&namsbc_rnf    !   runoffs namelist surface boundary condition
!-----------------------------------------------------------------------
!              !  file name                                              ! frequency (hours) ! variable   ! time interp. !  clim  ! 'yearly'/ ! weights  ! rotation ! land/sea mask !
!              !                                                         !  (if <0  months)  !   name     !   (logical)  !  (T/F) ! 'monthly' ! filename ! pairing  ! filename      !
   sn_rnf      = 'eORCA025-fETISh_rnf_merino_icb_nocoastbuff_norm',        -1         , 'sornficb' , .true.       , .false. , 'yearly'  , ''       , ''       , ''
   sn_cnf      = 'eORCA025-fETISh_rnf_merino_icb_nocoastbuff_norm',         0         , 'socoefr'  , .false.      , .false. , 'yearly'  , ''       , ''       , ''
   ln_rnf_depth=.false.
   ln_rnf_mouth=.true.
   ln_rnf_sal=.false.
   ln_rnf_tem=.false.
   rn_avt_rnf=2.0e-3
   rn_hrnf=10.0
   rn_rfact=1.0
/
!-----------------------------------------------------------------------
&namsbc_isf    !  Top boundary layer (ISF)
!-----------------------------------------------------------------------
!                ! file name                                                ! frequency (hours) ! variable     ! time interpol. !  clim   ! 'yearly'/ ! weights  ! rotation !
!                !                                                          !  (if <0  months)  !   name       !    (logical)   !  (T/F)  ! 'monthly' ! filename ! pairing  !
!   sn_fwfisf     = 'isfmelt.nc' ,       -1         , 'sofwfisf'    ,    .false.     , .true.  , 'yearly'  ,  ''      ,   ''
!   sn_rnfisf     = 'runoff.nc' ,       -12         ,'sofwfisf'    ,    .false.     , .true.  , 'yearly'  ,  ''      ,   ''
   sn_depmax_isf = 'runoff.nc' ,       -12         ,'sozisfmax',    .false.     , .true.  , 'yearly'  ,  ''      ,   ''
   sn_depmin_isf = 'runoff.nc' ,       -12         ,'sozisfmin',    .false.     , .true.  , 'yearly'  ,  ''      ,   ''
   ln_divisf   = .true.  ! apply isf melting as a mass flux or in the salinity trend. (maybe I should remove this option as for runoff?)
! only for nn_isf = 1 or 2
   rn_gammat0  = 2.21e-2   ! gammat coefficient used in blk formula
   rn_gammas0  = 6.4e-4   ! gammas coefficient used in blk formula
! only for nn_isf = 1
   nn_isfblk   =  2       ! 1 ISOMIP ; 2 conservative (3 equation formulation, Jenkins et al. 1991 ??)
   rn_hisf_tbl =  0.      ! thickness of the top boundary layer           (Losh et al. 2008)
                          ! 0 => thickness of the tbl = thickness of the first wet cell
   ln_conserve = .true.   ! conservative case (take into account meltwater advection)
   nn_gammablk = 1        ! 0 = cst Gammat (= gammat/s)
                          ! 1 = velocity dependend Gamma (u* * gammat/s)  (Jenkins et al. 2010)
                          !     if you want to keep the cd as in global config, adjust rn_gammat0 to compensate
                          ! 2 = velocity and stability dependent Gamma    Holland et al. 1999
/
!-----------------------------------------------------------------------
&namsbc_iscpl  !   land ice / ocean coupling option
!-----------------------------------------------------------------------
   nn_drown  = 100       ! number of iteration of the extrapolation loop (fill the new wet cells)
   ln_hsb    = .false.  ! activate conservation module (conservation exact after a time of rn_fiscpl)
   nn_fiscpl = 43800    ! (number of time step) conservation period (maybe should be fix to the coupling frequencey of restart frequency)
/
!
!-----------------------------------------------------------------------
&namsbc_apr    !   Atmospheric pressure used as ocean forcing or in bulk
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsbc_ssr    !   surface boundary condition : sea surface restoring
!-----------------------------------------------------------------------
!              !  file name                                          ! frequency (hours) ! variable  ! time interp. !  clim  ! 'yearly'/ ! weights                                         ! rotation ! land/sea mask !
!              !                                                     !  (if <0  months)  !   name    !   (logical)  !  (T/F) ! 'monthly' ! filename                                        ! pairing  ! filename      !
   sn_sss      = 'so025_noiscpl-40y_Mm_misstonn_smooth9_sss_1990-2009.nc' ,        -1         , 'so',    .true.    , .true. , 'yearly'  ,'', ''       , ''
   cn_dir      = './'      !  root directory for the location of the runoff files
   nn_sstr     =     0     !  add a retroaction term in the surface heat       flux (=1) or not (=0)
   nn_sssr     =     2     !  add a damping     term in the surface freshwater flux (=2)
                           !  or to SSS only (=1) or no damping term (=0)
   rn_dqdt     =   -40.    !  magnitude of the retroaction on temperature   [W/m2/K]
   rn_deds     =  -166.67  !  magnitude of the damping on salinity   [mm/day]
   ln_sssr_bnd =   .true.  !  flag to bound erp term (associated with nn_sssr=2)
   rn_sssr_bnd =   4.e0    !  ABS(Max/Min) value of the damping erp term [mm/day]

/
!-----------------------------------------------------------------------
&namsbc_alb    !   albedo parameters
!-----------------------------------------------------------------------

nn_ice_alb = 1
!rn_alb_oce = 0.066 ! Default
rn_alb_oce = 0.088 ! GMD tuning

! rn_albice_incr = 0.05

 rn_alb_sdry  =  0.87  !  dry snow albedo         : 0.80 (nn_ice_alb = 0); 0.85 (nn_ice_alb = 1/2); obs 0.85-0.87 (cloud-sky)
 rn_alb_smlt  =  0.82  !  melting snow albedo     : 0.65 ( '' )          ; 0.75 ( '' )            ; obs 0.72-0.82 ( '' )
 rn_alb_idry  =  0.65  !  dry ice albedo          : 0.72 ( '' )          ; 0.60 ( '' )            ; obs 0.54-0.65 ( '' )
 rn_alb_imlt  =  0.58  !  bare puddled ice albedo : 0.53 ( '' )          ; 0.50 ( '' )            ; obs 0.49-0.58 ( '' ) -> used only if no ponds
 rn_alb_dpnd  =  0.30  !  bare puddled ice albedo : 0.25 ( '' )          ; 0.27 ( '' )            ; obs 0.10-0.30 ( '' )

/
!-----------------------------------------------------------------------
&namberg       !   iceberg parameters
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namlbc        !   lateral momentum boundary condition
!-----------------------------------------------------------------------
   rn_shlat    =    0.     !  shlat = 0  !  0 < shlat < 2  !  shlat = 2  !  2 < shlat
                           !  free slip  !   partial slip  !   no slip   ! strong slip
/
!-----------------------------------------------------------------------
&namcla        !   cross land advection
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namagrif      !  AGRIF zoom                                            ("key_agrif")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&nam_tide      !   tide parameters (#ifdef key_tide)
!-----------------------------------------------------------------------
/
&nambdy        !  unstructured open boundaries                          ("key_bdy")
!-----------------------------------------------------------------------
    nb_bdy         = 1                    !  number of open boundary sets
    ln_coords_file = .true.               !  =T : read bdy coordinates from file
    cn_coords_file = 'eORCA025-SO_bdy-coords.nc'           !  bdy coordinates files
    ln_mask_file   = .false.
    cn_mask_file   = ''
    cn_dyn2d       = 'flather'               !
    nn_dyn2d_dta   =  1                   !  = 0, bdy data are equal to the initial state
                                          !  = 1, bdy data are read in 'bdydata   .nc' files
                                          !  = 2, use tidal harmonic forcing data from files
                                          !  = 3, use external data AND tidal harmonic forcing
    cn_dyn3d      =  'frs'               !
    nn_dyn3d_dta  =  1                    !  = 0, bdy data are equal to the initial state
                                          !  = 1, bdy data are read in 'bdydata   .nc' files
    cn_tra        =  'frs'               !
    nn_tra_dta    =  1                    !  = 0, bdy data are equal to the initial state
                                          !  = 1, bdy data are read in 'bdydata   .nc' files

    ! If you want to include sea ice in BDY: change nn_ice_lim_dta to 1.
    cn_ice_lim      =  'frs'             ! 'frs' or 'flather'
    nn_ice_lim_dta  =  0                  !  = 0, bdy data are equal to the initial state
                                          !  = 1, bdy data are read in 'bdydata   .nc' files
    rn_ice_tem      = 270.                !  lim3 only: arbitrary temperature of incoming sea ice
    rn_ice_sal      = 10.                 !  lim3 only:      --   salinity           --
    rn_ice_age      = 30.                 !  lim3 only:      --   age                --

    ln_tra_dmp    = .false.                !  open boudaries conditions for tracers
    ln_dyn3d_dmp  = .false.                !  open boundary condition for baroclinic velocities
    rn_time_dmp   =  1.                  ! Damping time scale in days
    rn_time_dmp_out =  1.                 ! Outflow damping time scale
    nn_rimwidth   = 1                    !  width of the relaxation zone
    ln_vol        = .false.               !  total volume correction (see nn_volctl parameter)
    nn_volctl     = 0                     !  = 0, the total water flux across open boundaries is zero
/
!-----------------------------------------------------------------------
&nambdy_dta      !  open boundaries - external data           ("key_bdy")
!-----------------------------------------------------------------------
!              !  file name     	 ! frequency (hours) ! variable   ! time interp.   !  clim   ! 'yearly'/ ! weights  ! rotation ! land/sea mask !
!              !              				         !  (if <0  months)  !   name     !   (logical)    !  (T/F ) ! 'monthly' ! filename ! pairing  ! filename      !
   bn_ssh =     'eORCA025-SO_BDYDTA_grid_T_1m' ,         -1        , 'sossheig' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   bn_u2d =     'eORCA025-SO_BDYDTA_grid_U_1m' ,         -1        , 'uobarotropic' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   bn_v2d =     'eORCA025-SO_BDYDTA_grid_V_1m' ,         -1        , 'vobarotropic' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   bn_u3d =     'eORCA025-SO_BDYDTA_grid_U_1m' ,         -1        , 'uobaroclinic' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   bn_v3d =     'eORCA025-SO_BDYDTA_grid_V_1m' ,         -1        , 'vobaroclinic' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   bn_tem =     'eORCA025-SO_BDYDTA_grid_T_1m' ,         -1        , 'conservative_temperature' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   bn_sal =     'eORCA025-SO_BDYDTA_grid_T_1m' ,         -1        , 'absolute_salinity' ,     .true.     , .false. ,  'yearly'  ,    ''    ,   ''     , ''
   
   cn_dir = './'
   ln_full_vel = .false.
/
!-----------------------------------------------------------------------
&nambdy_tide     ! tidal forcing at open boundaries
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&nambfr        !   bottom friction
!-----------------------------------------------------------------------
nn_bfr      =    2      !  type of bottom friction :   = 0 : free slip,  = 1 : linear friction
                            !                              = 2 : nonlinear friction
rn_bfri1    =    4.e-4  !  bottom drag coefficient (linear case)
rn_bfri2    =    1.e-3  !  bottom drag coefficient (non linear case). Minimum coeft if ln_loglayer    =T
rn_bfri2_max =   1.e-1  !  max. bottom drag coefficient (non linear case and ln_loglayer=T)
rn_bfeb2    =    2.5e-3 !  bottom turbulent kinetic energy background  (m2/s2)
rn_bfrz0    =    3.e-3  !  bottom roughness [m] if ln_loglayer=T 
ln_bfr2d    = .false.   !  horizontal variation of the bottom friction coef (read a 2D mask file )
rn_bfrien   =    50.    !  local multiplying factor of bfr (ln_bfr2d=T)
rn_tfri1    =    4.e-4  !  top drag coefficient (linear case)
!   rn_tfri2    =    2.5e-3 !  top drag coefficient (non linear case). Minimum coeft if ln_loglayer=T
rn_tfri2 = 1e-3
rn_tfri2_max =   1.e-1  !  max. top drag coefficient (non linear case and ln_loglayer=T)
rn_tfeb2    =    0.0    !  top turbulent kinetic energy background  (m2/s2)
rn_tfrz0    =    3.e-3  !  top roughness [m] if ln_loglayer=T
ln_tfr2d    = .false.   !  horizontal variation of the top friction coef (read a 2D mask file )
rn_tfrien   =    50.    !  local multiplying factor of tfr (ln_tfr2d=T)
 
ln_bfrimp   = .true.    !  implicit bottom friction (requires ln_zdfexp = .false. if true)
ln_loglayer = .false.   !  logarithmic formulation (non linear case)
/
!-----------------------------------------------------------------------
&nambbc        !   bottom temperature boundary condition
!-----------------------------------------------------------------------
  ln_trabbc = .true.
  nn_geoflx   =    1      !  geothermal heat flux: = 0 no flux
                           !     = 1 constant flux
  rn_geoflx_cst = 86.4e-3
/
!-----------------------------------------------------------------------
&nambbl        !   bottom boundary layer scheme
!-----------------------------------------------------------------------
nn_bbl_ldf  =  1      !  diffusive bbl (=1)   or not (=0)
nn_bbl_adv  =  0      !  advective bbl (=1/2) or not (=0)
rn_ahtbbl   =  1000.  !  lateral mixing coefficient in the bbl  [m2/s]
rn_gambbl   =  10.    !  advective bbl coefficient                 [s]nn_bbl_adv=0
/
!-----------------------------------------------------------------------
&nameos        !   ocean physical parameters
!-----------------------------------------------------------------------
nn_eos=-1
ln_usect = .true.
/
!-----------------------------------------------------------------------
&namtra_adv    !   advection scheme for tracer
!-----------------------------------------------------------------------
ln_traadv_cen2       =.false.
ln_traadv_msc_ups    =.false.
ln_traadv_muscl      =.false.
ln_traadv_muscl2=.false.
ln_traadv_qck=.false.
ln_traadv_tvd=.true.
ln_traadv_tvd_zts=.false.
ln_traadv_ubs=.false.
/
!-----------------------------------------------------------------------
&namtra_adv_mle !  mixed layer eddy parametrisation (Fox-Kemper param)
!-----------------------------------------------------------------------
ln_mle=.false.
/
!----------------------------------------------------------------------------------
&namtra_ldf    !   lateral diffusion scheme for tracers
!----------------------------------------------------------------------------------
ln_botmix_grif=.false.
ln_traldf_bilap=.false.
ln_traldf_gdia=.false.
ln_traldf_grif=.false.
ln_traldf_hor=.false.
ln_traldf_iso=.true.
ln_traldf_lap=.true.
ln_traldf_level=.false.
ln_triad_iso=.false.
rn_aeiv_0=0.0
rn_aht_0=150.0
rn_ahtb_0=0.0
rn_slpmax=0.01
/
!-----------------------------------------------------------------------
&namtra_dmp    !   tracer: T & S newtonian damping
!-----------------------------------------------------------------------
   ln_tradmp   =  .false.  !  add a damping termn (T) or not (F)
/
!-----------------------------------------------------------------------
&namdyn_adv    !   formulation of the momentum advection
!-----------------------------------------------------------------------
ln_dynadv_cen2=.false.
ln_dynadv_ubs=.false.
ln_dynadv_vec=.true.
nn_dynkeg=1
/
!-----------------------------------------------------------------------
&nam_vvl    !   vertical coordinate options
!-----------------------------------------------------------------------
   ln_vvl_zstar  = .true.           !  zstar vertical coordinate                   

/
!-----------------------------------------------------------------------
&namdyn_vor    !   option of physics/algorithm (not control by CPP keys)
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namdyn_hpg    !   Hydrostatic pressure gradient option
!-----------------------------------------------------------------------
   ln_hpg_zps  = .false.   !  z-coordinate - partial steps (interpolation)
   ln_hpg_sco  = .false.    !  s-coordinate (standard jacobian formulation)
   ln_hpg_isf  = .true.    !  s-coordinate (standard jacobian formulation) for isf

/
!-----------------------------------------------------------------------
&namdyn_ldf    !   lateral diffusion on momentum
!-----------------------------------------------------------------------
ln_dynldf_bilap=.true.
ln_dynldf_hor=.true.
ln_dynldf_iso=.false.
ln_dynldf_lap=.false.
ln_dynldf_level=.false.
rn_ahm_0_blp=-1.5e11
/
!-----------------------------------------------------------------------
&namzdf        !   vertical physics
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namzdf_ric    !   richardson number dependent vertical diffusion       ("key_zdfric" )
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namzdf_tke    !   turbulent eddy kinetic dependent vertical diffusion  ("key_zdftke")
!-----------------------------------------------------------------------
/
!------------------------------------------------------------------------
&namzdf_kpp    !   K-Profile Parameterization dependent vertical mixing  ("key_zdfkpp", and optionally:
!------------------------------------------------------------------------ "key_kppcustom" or "key_kpplktb")
/
!-----------------------------------------------------------------------
&namzdf_gls                !   GLS vertical diffusion                   ("key_zdfgls")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namzdf_ddm    !   double diffusive mixing parameterization             ("key_zdfddm")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namzdf_tmx    !   tidal mixing parameterization                        ("key_zdftmx")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namzdf_tmx_new    !   new tidal mixing parameterization                ("key_zdftmx_new")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsol        !   elliptic solver / island / free surface
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&nammpp        !   Massively Parallel Processing                        ("key_mpp_mpi")
!-----------------------------------------------------------------------
!jpni = 34
!jpnj = 9
!jpnij = 212
/
!-----------------------------------------------------------------------
&namctl        !   Control prints & Benchmark
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namc1d_uvd    !   data: U & V currents                                 ("key_c1d")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namc1d_dyndmp !   U & V newtonian damping                              ("key_c1d")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsto       ! Stochastic parametrization of EOS
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namnc4        !   netcdf4 chunking and compression settings            ("key_netcdf4")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namtrd        !   diagnostics on dynamics and/or tracer trends         ("key_trddyn" and/or "key_trdtra")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namflo       !   float parameters                                      ("key_float")
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namptr       !   Poleward Transport Diagnostic
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namhsb       !  Heat and salt budgets
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&nam_diaharm   !   Harmonic analysis of tidal constituents ('key_diaharm')
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namdct        ! transports through sections
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namobs       !  observation usage switch                               ('key_diaobs')
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&nam_asminc   !   assimilation increments                               ('key_asminc')
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namsbc_wave   ! External fields from wave model
!-----------------------------------------------------------------------
/
!-----------------------------------------------------------------------
&namdyn_nept  !   Neptune effect (simplified: lateral and vertical diffusions removed)
!-----------------------------------------------------------------------
/
EOF
