[[ $@ == *verbose* ]] && set -x

module load ${module_list:?}
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+${LD_LIBRARY_PATH}:}"${extralibs_list}"

mkdir -p ${run_dir}
cd ${run_dir}

run_start_epoch=$(date -u -d"${run_start_date}" +%s)
run_start_date_cos="${run_start_date//-}00"
run_start_date=$(date -uR -d "${run_start_date}")
run_end_date="${run_start_date} + ${run_duration:?}"
run_end_date=$(date -uR -d "${run_end_date}")
run_end_epoch=$(date -u -d"${run_end_date}" +%s)
run_end_date_cos="$(date -u -d"${run_end_date}" +%Y%m%d%H)"

for (( ; run_num_legs>0 ; run_num_legs-- ))
do

    # Check for restart information file and set the current leg start date
    #   Ignore restart information file if force_run_from_scratch is true
    if ! [ -r ${info_file} ]
    then
        leg_is_restart=false
        leg_start_date=${run_start_date}
        leg_start_date_cos=${run_start_date_cos}
        leg_number=1
        echo "leg_is_restart=false"
    else
        leg_is_restart=true
        . ./${info_file}
        leg_start_date=${leg_end_date}
        leg_start_date_cos="$(date -u -d"${leg_start_date}" +%Y%m%d%H)"
        leg_number=$((leg_number+1))
        echo "leg_is_restart=true"
    fi

    # Compute the end date of the current leg
    if [ -n "${rst_freq}" ]
    then
        if (($(date -u -d "${leg_start_date:?} + ${rst_freq}" +%s) > $(date -u -d "${leg_start_date:?} + ${run_duration}" +%s) )); then
           rst_freq=$run_duration
        fi
        leg_end_date=$(date -uR -d "${leg_start_date} + ${rst_freq}")
        leg_end_date_cos="$(date -u -d"${leg_end_date}" +%Y%m%d%H)"
    else
        leg_end_date=${run_end_date}
        leg_end_date_cos=${run_end_date_cos}
    fi

    if [ $(date -u -d "${leg_end_date}" +%s) -gt $(date -u -d "${run_end_date}" +%s) ]
    then
        leg_end_date=${run_end_date}
        leg_end_date_cos=${run_end_date_cos}
    fi

    # Some time variables needed later
    leg_end_epoch=$(date -u -d "${leg_start_date:?} + ${rst_freq:=$run_duration}" +%s)
    leg_length_sec=$(( $(date -u -d "${leg_end_date}" +%s) - $(date -u -d "${leg_start_date}" +%s) ))
    leg_start_sec=$(( $(date -u -d "${leg_start_date}" +%s) - $(date -u -d "${run_start_date}" +%s) ))
    leg_end_sec=$(( $(date -u -d "${leg_end_date}" +%s) - $(date -u -d "${run_start_date}" +%s) ))
    leg_start_date_yyyymmdd=$(date -u -d "${leg_start_date}" +%Y%m%d)
    leg_start_date_yyyy=$(date -u -d "${leg_start_date}" +%Y)
    leg_end_date_yyyy=$(date -u -d "${leg_end_date}" +%Y)

    echo "leg_number: $leg_number"
    echo "leg_start_date: $leg_start_date"
    echo "leg_end_date: $leg_end_date"

    # Check whether there's actually time left to simulate - exit otherwise
    if [ ${leg_length_sec} -le 0 ]
    then
        info "Leg start date equal to or after end of simulation."
        info "Nothing left to do. Exiting."
        exit 0
    fi

    # -------------------------------------------------------------------------
    # *** Prepare the run directory for a run from scratch
    # -------------------------------------------------------------------------
    if ! $leg_is_restart
    then
        # ---------------------------------------------------------------------
        # *** Check if run dir is empty. If not, and if we are allowed to do so
        #     by ${force_run_from_scratch}, remove everything
        # ---------------------------------------------------------------------
        echo "pwd: $PWD"
        if $(ls * >& /dev/null)
        then
            if ${force_run_from_scratch}
            then
                rm -fr ${run_dir}/*
            else
                echo "error: run directory not empty and \$force_run_from_scratch not set."
                exit 0
            fi
        fi

        # ---------------------------------------------------------------------
        # *** Creates some directories
        # ---------------------------------------------------------------------
        mkdir -p ${out_dir}
        mkdir -p ${out_dir}/cclm_out_tmp
        mkdir -p ${out_dir}/cclm_out_tmp/out01
        mkdir -p ${out_dir}/cclm_out_tmp/out02
        mkdir -p ${out_dir}/cclm_out_tmp/out03
        mkdir -p ${out_dir}/cclm_out_tmp/out04
        mkdir -p ${out_dir}/cclm_out_tmp/out05
        mkdir -p ${out_dir}/cclm_out_tmp/out06
        mkdir -p ${out_dir}/cclm_out_tmp/out07
        mkdir -p ${out_dir}/cclm_out_tmp/restarts

	rm -f ${out_dir}/cclm_out_tmp/restarts/*

        # ---------------------------------------------------------------------
        # *** Copy executables and coupling-related files
        # ---------------------------------------------------------------------
        cd ${start_dir}
        cp -u namelists/* "${run_dir}"
        cd ${run_dir}
        cp -u ${start_dir}/namelists/namcouple.base .
        cp -u ${cclm_exe_file} .
        cp -u ${cesm_exe_file} .
        rsync -avz $coupling_files_dir/*.nc ${run_dir}/
        #coupling_files=(`ls -v $coupling_files_dir/*`)
        #for dirfile in "${coupling_files[@]}"; do
        #        file=${dirfile##*/}
        #        ln -sf $dirfile ${run_dir}/$file
        #done

        # ---------------------------------------------------------------------
        # *** Copy potential special restarts 
        # ---------------------------------------------------------------------
        if $special_restart
        then

            ### Getting the right output folders 
            leg_start_date_tmp=$leg_start_date
            leg_end_date_tmp=$leg_end_date
            leg_number_tmp=$leg_number
            rsync -avz  ${run_dir}/../../${special_restart_from}/rundir/${info_file} ${run_dir}
            special_date=$(date -uR -d "${special_restart_date}")
            sed -i "/$special_date/q" ${run_dir}/${info_file}
            . ${run_dir}/${info_file}
            special_restart_leg=$(printf %03d $((leg_number)))

            echo "special_date: $special_date"
            echo "special_restart_leg: $special_restart_leg"
            echo "special_restart_from: $special_restart_from"

            ### CLM
            special_date_cos="$(date -d"${special_date} - 1 hour" +%Y-%m-%d)"
            leg_start_date_cos_forsp="$(date -d"${leg_start_date_tmp} - 1 hour" +%Y-%m-%d)"

            indir="${run_dir}/../../${special_restart_from}/archive/restart/clm/$(printf %03d $((special_restart_leg)))"
            cd ${indir}
            for f in $PROJECT*.*; do
                echo "f: $f"
                nf=`echo "${f/$special_date_cos/$leg_start_date_cos_forsp}"`
                echo "copying $f to ${run_dir}/$nf" 
                cp $f ${run_dir}/$nf
            done

            indir="${run_dir}/../../${special_restart_from}/archive/output/clm/$(printf %03d $((special_restart_leg)))"
            cd ${indir}
            for f in $PROJECT*.h*$special_date_cos*; do
                echo "f: $f"
                nf=`echo "${f/$special_date_cos/$leg_start_date_cos_forsp}"`
                echo "copying $f to ${run_dir}/$nf" 
                cp $f ${run_dir}/$nf
            done

            ### COSMO
            indir="${run_dir}/../../${special_restart_from}/archive/restart/cosmo/$(printf %03d $((special_restart_leg)))"
            mkdir -p ${run_dir}/../cclm_out_tmp/restarts/
            special_date_cos="$(date -d"${special_date} - 1 hour" +%Y%m%d)"
            leg_start_date_cos_forsp="$(date -d"${leg_start_date_tmp} - 1 hour" +%Y%m%d)"
            echo "copying ${indir}/*$special_date_cos* to ${run_dir}/../cclm_out_tmp/restarts/"  
            rsync -avz ${indir}/*$special_date_cos* ${run_dir}/../cclm_out_tmp/restarts/

            ### OASIS
            indir="${run_dir}/../../${special_restart_from}/archive/restart/oasis/$(printf %03d $((special_restart_leg)))"
            rsync -avz ${indir}/$IC_cosmo ${run_dir}
            rsync -avz ${indir}/$IC_clm ${run_dir}

            cd ${run_dir}
            rm -rf ${info_file}
            leg_start_date=$leg_start_date_tmp
            leg_end_date=$leg_end_date_tmp
            leg_number=$leg_number_tmp
        fi # if $special_restart

    else # if ! $leg_is_restart

        # ---------------------------------------------------------------------
        # *** Creates some directories
        # ---------------------------------------------------------------------
        mkdir -p ${out_dir}
        mkdir -p ${out_dir}/cclm_out_tmp
        mkdir -p ${out_dir}/cclm_out_tmp/out01
        mkdir -p ${out_dir}/cclm_out_tmp/out02
        mkdir -p ${out_dir}/cclm_out_tmp/out03
        mkdir -p ${out_dir}/cclm_out_tmp/out04
        mkdir -p ${out_dir}/cclm_out_tmp/out05
        mkdir -p ${out_dir}/cclm_out_tmp/out06
        mkdir -p ${out_dir}/cclm_out_tmp/out07
        mkdir -p ${out_dir}/cclm_out_tmp/restarts

        # ---------------------------------------------------------------------
        # *** Copy restart files in run directory
        # ---------------------------------------------------------------------
        cd ${run_dir}

        ### CLM
        outdir="${archive_dir}/restart/clm/$(printf %03d $((leg_number-1)))"
        RESTART_DATE_TMP=$(($leg_start_date_cos / 100))
        RESTART_DATE=${RESTART_DATE_TMP:0:4}'-'${RESTART_DATE_TMP:4:2}'-'${RESTART_DATE_TMP:6:2}-'00000'
        ln -sf ${outdir}/${PROJECT}*.${RESTART_DATE}* ${run_dir}

        outdir="${archive_dir}/output/clm/$(printf %03d $((leg_number-1)))"
        ln -f ${outdir}/${PROJECT}*.h*.${RESTART_DATE}* ${run_dir}

        ### COSMO
        outdir="${archive_dir}/restart/cosmo/$(printf %03d $((leg_number-1)))"
	rm -f ${run_dir}/../cclm_out_tmp/restarts/*
        ln -sf ${outdir}/lrfd${leg_start_date_cos}o ${run_dir}/../cclm_out_tmp/restarts/

        ### OASIS
        outdir="${archive_dir}/restart/oasis/$(printf %03d $((leg_number-1)))"
        cp -f ${outdir}/$IC_cosmo ${run_dir}
        cp -f ${outdir}/$IC_clm ${run_dir}

    fi # ! $leg_is_restart
