# Function leap days calculates the number of leap days (29th of Februrary) in
# a time intervall between two dates.
#
# Usage leap_days START_DATE END_DATE
function leap_days()
{
    local ld=0
    local frstYYYY=$(date -ud "$1" +%Y)
    local lastYYYY=$(date -ud "$2" +%Y)

    set +e

    # Check first year for leap day between start and end date
    $(date -ud "${frstYYYY}-02-29" > /dev/null 2>&1) \
    && (( $(date -ud "$1" +%s) < $(date -ud "${frstYYYY}-03-01" +%s) )) \
    && (( $(date -ud "$2" +%s) > $(date -ud "${lastYYYY}-02-28" +%s) )) \
    && (( ld++ ))

    # Check intermediate years for leap day
    for (( y=(( ${frstYYYY}+1 )); y<=(( ${lastYYYY}-1 )); y++ ))
    do
        $(date -ud "$y-02-29" > /dev/null 2>&1) && (( ld++ ))
    done

    # Check last year (if different from first year) for leap day between start
    # and end date
    (( $lastYYYY > $frstYYYY )) \
    && $(date -ud "${lastYYYY}-02-29" > /dev/null 2>&1) \
    && (( $(date -ud "$1" +%s) < $(date -ud "${frstYYYY}-03-01" +%s) )) \
    && (( $(date -ud "$2" +%s) > $(date -ud "${lastYYYY}-02-28" +%s) )) \
    && (( ld++ ))

    set -e

    echo "$ld"
}

[[ $@ == *verbose* ]] && set -x

module load ${module_list:?}
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+${LD_LIBRARY_PATH}:}"${extralibs_list}"

mkdir -p ${run_dir}
cd ${run_dir}

run_start_epoch=$(date -u -d"${run_start_date}" +%s)
run_start_date_cos="${run_start_date//-}00"
run_start_date=$(date -uR -d "${run_start_date}")
run_end_date="${run_start_date} + ${run_duration:?}"
run_end_date=$(date -uR -d "${run_end_date}")
run_end_epoch=$(date -u -d"${run_end_date}" +%s)
run_end_date_cos="$(date -u -d"${run_end_date}" +%Y%m%d%H)"


if [[ ${special_rs_nemo} == "true" && ${special_restart} == "true" ]]; then
    echo "ERROR: both special_rs_nemo and special_restart are true. only can be true (at most). exit".
    exit 2
fi


for (( ; run_num_legs>0 ; run_num_legs-- ))
do

    # Check for restart information file and set the current leg start date
    #   Ignore restart information file if force_run_from_scratch is true
    if ! [ -r ${info_file} ]
    then
        leg_is_restart=false
        leg_start_date=${run_start_date}
        leg_start_date_cos=${run_start_date_cos}
        leg_number=1
        echo "leg_is_restart=false"
    else
        leg_is_restart=true
        . ./${info_file}
        leg_start_date=${leg_end_date}
        leg_start_date_cos="$(date -u -d"${leg_start_date}" +%Y%m%d%H)"
        leg_number=$((leg_number+1))
        echo "leg_is_restart=true"
    fi

    # Compute the end date of the current leg
    if [ -n "${rst_freq}" ]
    then
        if (($(date -u -d "${leg_start_date:?} + ${rst_freq}" +%s) > $(date -u -d "${leg_start_date:?} + ${run_duration}" +%s) )); then
           rst_freq=$run_duration
        fi
        leg_end_date=$(date -uR -d "${leg_start_date} + ${rst_freq}")
        leg_end_date_cos="$(date -u -d"${leg_end_date}" +%Y%m%d%H)"
    else
        leg_end_date=${run_end_date}
        leg_end_date_cos=${run_end_date_cos}
    fi

    if [ $(date -u -d "${leg_end_date}" +%s) -gt $(date -u -d "${run_end_date}" +%s) ]
    then
        leg_end_date=${run_end_date}
        leg_end_date_cos=${run_end_date_cos}
    fi

    # Some time variables needed later
    leg_end_epoch=$(date -u -d "${leg_start_date:?} + ${rst_freq:=$run_duration}" +%s)
    leg_length_sec=$(( $(date -u -d "${leg_end_date}" +%s) - $(date -u -d "${leg_start_date}" +%s) ))
    leg_start_sec=$(( $(date -u -d "${leg_start_date}" +%s) - $(date -u -d "${run_start_date}" +%s) ))
    leg_end_sec=$(( $(date -u -d "${leg_end_date}" +%s) - $(date -u -d "${run_start_date}" +%s) ))
    leg_start_date_yyyymmdd=$(date -u -d "${leg_start_date}" +%Y%m%d)
    leg_start_date_yyyy=$(date -u -d "${leg_start_date}" +%Y)
    leg_end_date_yyyy=$(date -u -d "${leg_end_date}" +%Y)

    echo "leg_number: $leg_number"
    echo "leg_start_date: $leg_start_date"
    echo "leg_end_date: $leg_end_date"

    # Check whether there's actually time left to simulate - exit otherwise
    if [ ${leg_length_sec} -le 0 ]
    then
        info "Leg start date equal to or after end of simulation."
        info "Nothing left to do. Exiting."
        exit 0
    fi

    # -------------------------------------------------------------------------
    # *** Prepare the run directory for a run from scratch
    # -------------------------------------------------------------------------
    if ! $leg_is_restart
    then
        # ---------------------------------------------------------------------
        # *** Check if run dir is empty. If not, and if we are allowed to do so
        #     by ${force_run_from_scratch}, remove everything
        # ---------------------------------------------------------------------
        echo "pwd: $PWD"
        if $(ls * >& /dev/null)
        then
            if ${force_run_from_scratch}
            then
                rm -fr ${run_dir}/*
            else
                echo "error: run directory not empty and \$force_run_from_scratch not set."
                exit 0
            fi
        fi

        # ---------------------------------------------------------------------
        # *** Creates some directories
        # ---------------------------------------------------------------------
        mkdir -p ${out_dir}
        mkdir -p ${out_dir}/cclm_out_tmp
        mkdir -p ${out_dir}/cclm_out_tmp/out01
        mkdir -p ${out_dir}/cclm_out_tmp/out02
        mkdir -p ${out_dir}/cclm_out_tmp/out03
        mkdir -p ${out_dir}/cclm_out_tmp/out04
        mkdir -p ${out_dir}/cclm_out_tmp/out05
        mkdir -p ${out_dir}/cclm_out_tmp/out06
        mkdir -p ${out_dir}/cclm_out_tmp/out07
        mkdir -p ${out_dir}/cclm_out_tmp/restarts

	rm -f ${out_dir}/cclm_out_tmp/restarts/*

        # ---------------------------------------------------------------------
        # *** Copy executables and coupling-related files
        # ---------------------------------------------------------------------
        cd ${start_dir}
        cp -u xios_config/*xml "${run_dir}"
        cp -u namelists/* "${run_dir}"
        cd ${run_dir}
        cp -u "${nem_exe_file:?}" .
        cp -u "${xio_exe_file:?}" .
        cp -u ${start_dir}/namelists/namcouple.base .
        cp -u ${cclm_exe_file} .
        cp -u ${cesm_exe_file} .
        rsync -avz $coupling_files_dir/*.nc ${run_dir}/
        #coupling_files=(`ls -v $coupling_files_dir/*`)
        #for dirfile in "${coupling_files[@]}"; do
        #        file=${dirfile##*/}
        #        ln -sf $dirfile ${run_dir}/$file
        #done

        # ---------------------------------------------------------------------
        # *** Link ic, forcing and shared files for NEMO
        # ---------------------------------------------------------------------
        [[ ! -f EMPave_old.dat ]] && echo "                               0  0.0000000000000000E+00  0.0000000000000000E+00" > EMPave_old.dat
        for file in "${ic_files[@]}"; do
            [[ ! -e ${file#*> } ]] && ln -sf $(sed 's/ *=> */ /' <<< "${ini_data_dir}/${ic_subdir}/${nem_grid}/$file")
        done
        for file in "${forcing_files[@]}"; do
            [[ ! -e ${file#*> } ||  "$file" == \** ]] && ln -sf $(sed 's/ *=> */ /' <<< "${ini_data_dir}/${forcing_subdir}/${nem_forcing_set}/$file")
        done
        for file in "${shared_files[@]}"; do
            [[ ! -e ${file#*> } ]] && ln -sf $(sed 's/ *=> */ /' <<< "${shared_dir}/$file")
        done

        # ---------------------------------------------------------------------
        # *** Copy potential special restarts 
        # ---------------------------------------------------------------------
        if $special_restart
        then

            ### Getting the right output folders 
            leg_start_date_tmp=$leg_start_date
            leg_end_date_tmp=$leg_end_date
            leg_number_tmp=$leg_number
            rsync -avz  ${run_dir}/../../${special_restart_from}/rundir/${info_file} ${run_dir}
            special_date=$(date -uR -d "${special_restart_date}")
            sed -i "/$special_date/q" ${run_dir}/${info_file}
            . ${run_dir}/${info_file}
            special_restart_leg=$(printf %03d $((leg_number)))

            echo "special_date: $special_date"
            echo "special_restart_leg: $special_restart_leg"
            echo "special_restart_from: $special_restart_from"

            ### NEMO
            indir="${run_dir}/../../${special_restart_from}/archive/restart/nemo/$(printf %03d $((special_restart_leg)))"
            cd ${indir}
            for f in *.nc; do
                nf=`echo "${f/$special_restart_from/$exp_name}"`
                echo "copy $f to ${run_dir}/$nf" 
               cp $f ${run_dir}/$nf
            done
            cd -
            cd ${run_dir}
            echo "pwd: $PWD"
            for f in ${exp_name}_????????_restart_???_????.nc; do
                nf=`echo ${f: -19}`
                echo "linking $f to ${run_dir}/$nf" 
                ln -s $f $nf
            done

            ### CLM
            special_date_cos="$(date -d"${special_date} - 1 hour" +%Y-%m-%d)"
            leg_start_date_cos_forsp="$(date -d"${leg_start_date_tmp} - 1 hour" +%Y-%m-%d)"

            indir="${run_dir}/../../${special_restart_from}/archive/restart/clm/$(printf %03d $((special_restart_leg)))"
            cd ${indir}
            for f in $PROJECT*.*; do
                echo "f: $f"
                nf=`echo "${f/$special_date_cos/$leg_start_date_cos_forsp}"`
                echo "copying $f to ${run_dir}/$nf" 
                cp $f ${run_dir}/$nf
            done

            indir="${run_dir}/../../${special_restart_from}/archive/output/clm/$(printf %03d $((special_restart_leg)))"
            cd ${indir}
            for f in $PROJECT*.h*$special_date_cos*; do
                echo "f: $f"
                nf=`echo "${f/$special_date_cos/$leg_start_date_cos_forsp}"`
                echo "copying $f to ${run_dir}/$nf" 
                cp $f ${run_dir}/$nf
            done

            ### COSMO
            indir="${run_dir}/../../${special_restart_from}/archive/restart/cosmo/$(printf %03d $((special_restart_leg)))"
            mkdir -p ${run_dir}/../cclm_out_tmp/restarts/
            special_date_cos="$(date -d"${special_date} - 1 hour" +%Y%m%d)"
            leg_start_date_cos_forsp="$(date -d"${leg_start_date_tmp} - 1 hour" +%Y%m%d)"
            echo "copying ${indir}/*$special_date_cos* to ${run_dir}/../cclm_out_tmp/restarts/"  
            rsync -avz ${indir}/*$special_date_cos* ${run_dir}/../cclm_out_tmp/restarts/

            ### OASIS
            indir="${run_dir}/../../${special_restart_from}/archive/restart/oasis/$(printf %03d $((special_restart_leg)))"
            rsync -avz ${indir}/$IC_cosmo ${run_dir}
            rsync -avz ${indir}/$IC_clm ${run_dir}
            rsync -avz ${indir}/$IC_nemo ${run_dir}
            rsync -avz ${indir}/$IC_cclm ${run_dir}

            cd ${run_dir}
            rm -rf ${info_file}
            leg_start_date=$leg_start_date_tmp
            leg_end_date=$leg_end_date_tmp
            leg_number=$leg_number_tmp

	elif ${special_rs_nemo}
	then

	    dir_rs=`dirname ${special_rs_nemo_stencil}`
	    base_rs=`basename ${special_rs_nemo_stencil}`
	    
	    n_oce_rs=`find ${dir_rs} -name "${base_rs}oce_????.nc" | wc -l`
	    if(( n_oce_rs != nem_numproc )); then
		echo "in special_rs_nemo, not the right amount of ocean restart files."
		echo "${n_oce_rs} instead of ${nem_numproc}. exiting."
		exit 2
	    fi

	    n_ice_rs=`find ${dir_rs} -name "${base_rs}ice_????.nc" | wc -l`
	    if(( n_ice_rs != nem_numproc )); then
		echo "in special_rs_nemo, not the right amount of ice restart files."
		echo "${n_ice_rs} instead of ${nem_numproc}. exiting."
		exit 2
	    fi
	    
            for (( n=0 ; n<nem_numproc ; n++ ))
            do
		np=$(printf %04d ${n})
		ln -fs ${special_rs_nemo_stencil}oce_${np}.nc restart_oce_${np}.nc
		ln -fs ${special_rs_nemo_stencil}ice_${np}.nc restart_ice_${np}.nc
            done

        fi # if $special_restart


    else # if ! $leg_is_restart

        # ---------------------------------------------------------------------
        # *** Creates some directories
        # ---------------------------------------------------------------------
        mkdir -p ${out_dir}
        mkdir -p ${out_dir}/cclm_out_tmp
        mkdir -p ${out_dir}/cclm_out_tmp/out01
        mkdir -p ${out_dir}/cclm_out_tmp/out02
        mkdir -p ${out_dir}/cclm_out_tmp/out03
        mkdir -p ${out_dir}/cclm_out_tmp/out04
        mkdir -p ${out_dir}/cclm_out_tmp/out05
        mkdir -p ${out_dir}/cclm_out_tmp/out06
        mkdir -p ${out_dir}/cclm_out_tmp/out07
        mkdir -p ${out_dir}/cclm_out_tmp/restarts

        # ---------------------------------------------------------------------
        # *** Copy restart files in run directory
        # ---------------------------------------------------------------------
        cd ${run_dir}
        ### NEMO
        outdir="${archive_dir}/restart/nemo/$(printf %03d $((leg_number-1)))"

	
        ns=$(printf %08d $(( leg_start_sec / nem_time_step_sec )))
        for f in oce ice
        do
	    echo "${outdir}/${exp_name}_${ns}_restart_${f}_????.nc"
	    ln -sf ${outdir}/${exp_name}_${ns}_restart_${f}_????.nc ${run_dir}
        done

        for (( n=0 ; n<nem_numproc ; n++ ))
        do
	    np=$(printf %04d ${n})
	    ln -fs ${exp_name}_${ns}_restart_oce_${np}.nc restart_oce_${np}.nc
	    ln -fs ${exp_name}_${ns}_restart_ice_${np}.nc restart_ice_${np}.nc
        done


        ### CLM
        outdir="${archive_dir}/restart/clm/$(printf %03d $((leg_number-1)))"
        RESTART_DATE_TMP=${leg_start_date_cos::-2}
        RESTART_DATE=${RESTART_DATE_TMP:0:4}'-'${RESTART_DATE_TMP:4:2}'-'${RESTART_DATE_TMP:6:2}-'00000'
        ln -sf ${outdir}/${PROJECT}*.${RESTART_DATE}* ${run_dir}

        outdir="${archive_dir}/output/clm/$(printf %03d $((leg_number-1)))"
        ln -sf ${outdir}/${PROJECT}*.h*.${RESTART_DATE}* ${run_dir} 

        ### COSMO
        outdir="${archive_dir}/restart/cosmo/$(printf %03d $((leg_number-1)))"
	rm -f ${run_dir}/../cclm_out_tmp/restarts/*
        ln -sf ${outdir}/lrfd${leg_start_date_cos}o ${run_dir}/../cclm_out_tmp/restarts/

        ### OASIS
        outdir="${archive_dir}/restart/oasis/$(printf %03d $((leg_number-1)))"
        cp -f ${outdir}/$IC_cosmo ${run_dir}
        cp -f ${outdir}/$IC_clm ${run_dir}
        cp -f ${outdir}/$IC_nemo ${run_dir}
        cp -f ${outdir}/$IC_cclm ${run_dir}

    fi # ! $leg_is_restart
