    # If YU* files present, remove.
    if ls ${run_dir}/YU* 1> /dev/null 2>&1; then
        echo "YU files do exist, remove."
        rm YU*
    fi

    # -------------------------------------------------------------------------
    # *** Update ocean namelist
    # -------------------------------------------------------------------------
    source build_namelist_cfg.sh > namelist_cfg

    # -------------------------------------------------------------------------
    # *** Build ice sheet launch
    # -------------------------------------------------------------------------
    if (( isfcpl == 1)); then
	source ${icesheet_launch_template} > launch_icesheet.sh
	chmod +x launch_icesheet.sh
    fi
    
    # -------------------------------------------------------------------------
    # *** Input path for cclm
    # -------------------------------------------------------------------------
    namelistdir=$CLMINDIR/data/namelists_cclm-clm2-${PROJECT}
    cp -f $namelistdir/* .
    if [ $YDATE_NOW == $YDATE_INI ]; then
        inputdir=$CLMINDIR/data/input_cclm_clm2-${PROJECT}/final
        cp -fr $inputdir/* .
        echo "Starting a new run" # FK utile ? 
        clm_start_type='startup'
        touch $run_dir/$clm_start_type
    else
        echo "Continue run"
        clm_start_type='continue'
        touch $run_dir/$clm_start_type
    fi

    rm -rf timing
    mkdir -p timing/checkpoints # FK: est-ce utile ? 

    datm_proc=$clm_numproc

    sed -e s/'case_name.*=.*'/"case_name = '"$PROJECT"'"/ \
        -e s/'calendar.*=.*'/"calendar = 'GREGORIAN'"/ \
        -e s/'_cpl_dt.*=.*'/'_cpl_dt   = '$CLM_TSTEP/ \
        -e s/'start_type.*=.*'/"start_type = '"$clm_start_type"'"/ \
        -e s/' start_ymd.*=.*'/" start_ymd = "$STARTYMD/ \
        -e s/'stop_option.*=.*'/"stop_option = 'nseconds'"/ \
        -e s/'stop_n.*=.*'/"stop_n = "$RUNTIME/ \
        -e s/'restart_option.*=.*'/"restart_option = 'nseconds'"/ \
        -e s/'restart_n.*=.*'/"restart_n = "$RESTART_CLM/ \
        -e s/'_ntasks.*=.*'/'_ntasks   = '$clm_numproc/ \
        -e s/'atm_ntasks.*=.*'/'atm_ntasks   = '$datm_proc/ drv_in > tmp.stdin
    mv tmp.stdin drv_in

    sed -e s/'dtime.*=.*'/'dtime   = '$CLM_TSTEP/ lnd_in > tmp.stdin
    mv tmp.stdin lnd_in

    for i in atm cpl glc ice lnd ocn rof wav
    do
            sed -e s@'diri.*\=.*'@'diri   = "'$run_dir'"'@ \
                -e s@'diro.*\=.*'@'diro   = "'$run_dir'"'@ ${i}_modelio.nml > tmp.stdin
    mv tmp.stdin ${i}_modelio.nml
    done

    sed \
          -e s/_run_time_/${RUNTIME}/ \
          -e s/_cpl_freq_cosmo_clm_/${CPL_FREQ_cosmo_clm}/ \
          -e s/_cpl_freq_nemo_cclm_/${CPL_FREQ_nemo_cclm}/ \
	  -e s/_cpl_freq_cclm_nemo_/${CPL_FREQ_cclm_nemo}/ \
          -e s/_lag_cos_to_clm_/${LAG_cosmo_to_clm}/ \
          -e s/_lag_clm_to_cosmo_/${LAG_clm_to_cosmo}/ \
          -e s/_lag_nemo_to_cclm_/${LAG_nemo_to_cclm}/ \
          -e s/_lag_cclm_to_nemo_/${LAG_cclm_to_nemo}/ \
          -e s/_EX_type_clm_to_cosmo_/${EX_type_clm_to_cosmo}/ \
          -e s/_EX_type_cosmo_to_clm_/${EX_type_cosmo_to_clm}/ \
          -e s/_EX_type_nemo_to_cclm_/${EX_type_nemo_to_cclm}/ \
          -e s/_EX_type_cclm_to_nemo_/${EX_type_cclm_to_nemo}/ \
          -e s/_IC_cosmo_/${IC_cosmo}/ \
          -e s/_IC_clm_/${IC_clm}/ \
          -e s/_IC_nemo_/${IC_nemo}/ \
          -e s/_IC_cclm_/${IC_cclm}/ namcouple.base > namcouple

    # -------------------------------------------------------------------------
    # *** Launch experiment
    # -------------------------------------------------------------------------
    echo "START of: mpirun -np "${clm_numproc}" "${cesm_exe_file}" : -np "${cos_numproc}" "${cclm_exe_file}" : -np "${nem_numproc:?}" "${nem_exe_file:?}" : -np "${xio_numproc:?}" "${xio_exe_file:?}""
    time_begin=$(date +%s)
    mpirun -np "${clm_numproc}" "${cesm_exe_file}" : -np "${cos_numproc}" "${cclm_exe_file}" : -np "${nem_numproc:?}" "${nem_exe_file:?}" : -np "${xio_numproc:?}" "${xio_exe_file:?}"
    time_end=$(date +%s)
    echo "END of: mpirun -np "${clm_numproc}" "${cesm_exe_file}" : -np "${cos_numproc}" "${cclm_exe_file}" : -np "${nem_numproc:?}" "${nem_exe_file:?}" : -np "${xio_numproc:?}" "${xio_exe_file:?}""

    # -------------------------------------------------------------------------
    # *** Ice sheet coupling
    # -------------------------------------------------------------------------

    if (( isfcpl == 1 )); then

        cnt_res_isfcpl=$(( $(( ${leg_number} - 1 )) % ${n_res_perisfcpl} ))
        fmt_res_isfcpl=$(printf "%02d" ${cnt_res_isfcpl})

	module purge
	load_modules_isfcpl
	
	out_smb_stencil="${exp_name}_smb_res"
        if (( ice_atm_cpl == 1 )); then
            in_smb_stencil="../cclm_out_tmp/${where_is_smb}/lffd??????????.nc"
            out_file="${out_smb_stencil}${fmt_res_isfcpl}.nc"

            cdo -O copy ${in_smb_stencil} tmp_smb.nc
            cdo selname,${smb_org_var_name} tmp_smb.nc ${out_file}

	    mv -f ${out_file} ${out_dir}/isfcpl_tmp
	    
        fi

        in_fwfisf_stencil="${exp_name}_${frequency_fwfisf}_${leg_start_date_yyyymmdd}_${leg_almostend_date_yyyymmdd}_${suffix_fwfisf}"
        out_fwfisf_stencil="${exp_name}_fwfisf_res"

	out_fwfisf_curr="${out_fwfisf_stencil}${fmt_res_isfcpl}"
	
        if (( is_xios2 == 0 )); then

            module purge
            module load ${module_list:?}
            export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+${LD_LIBRARY_PATH}:}"${extralibs_list}"

            /bin/bash ${rebuild_nemo_exe} ${in_fwfisf_stencil} ${xio_numproc}

	    module purge
	    load_modules_isfcpl

	    
	    cdo selname,fwfisf ${in_fwfisf_stencil}.nc tmp_fwfisf.nc
	    cdo monmean tmp_fwfisf.nc ${out_fwfisf_curr}.nc
	    
	    rm -f ${in_fwfisf_stencil}.nc tmp_fwfisf.nc
	
        else

	    cdo selname,fwfisf ${in_fwfisf_stencil}.nc tmp_fwfisf.nc
	    cdo monmean tmp_fwfisf.nc ${out_fwfisf_curr}.nc

	    rm -f tmp_fwfisf.nc

        fi
	
	mv -f ${out_fwfisf_curr}.nc ${out_dir}/isfcpl_tmp/
	
        if (( ${leg_number} % ${n_res_perisfcpl} == 0 )); then
            date_end_isfcpl=${leg_almostend_date_yyyymmdd}
            sed -i "s/.*date_end_isfcpl=.*/date_end_isfcpl=${date_end_isfcpl}/" ${start_dir}/isfcpl.track

            module purge
            module load ${module_list:?}
            export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+${LD_LIBRARY_PATH}:}"${extralibs_list}"

            /bin/bash ${rebuild_nemo_exe} mesh_mask ${nem_numproc}

            if (( ice_atm_cpl == 1 )); then

		outfile_raw_atm2ice="${exp_name}_atmosphere2ice_raw_${date_beg_isfcpl}-${date_end_isfcpl}.nc"
                outfile_atm2ice="${exp_name}_atmosphere2ice_${date_beg_isfcpl}-${date_end_isfcpl}.nc"

                module purge
                module load ${smb_modules}

                # Concatenate all smb files from previous coupling windows

		smb_records=""
		for((rs2=0; rs2<n_res_perisfcpl; rs2++)); do
                    fmt_rs2=$(printf "%02d" ${rs2})
		    curr_file="${out_dir}/isfcpl_tmp/${out_smb_stencil}${fmt_rs2}.nc"
		    if [ -f ${curr_file} ] ; then
			smb_records="${smb_records} ${curr_file}"
		    else
			echo "${curr_file} is not in ${out_dir}/isfcpl_tmp. It is needed for CCLM - ice sheet coupling. Exiting"
			exit 2
		    fi
		done
		
                cdo -O copy ${smb_records} ${outfile_raw_atm2ice}
		rm -f ${smb_records}

                # Monthly mean
                cdo monmean ${outfile_raw_atm2ice} tmp2_smb.nc
                # Mask SMB over ocean nodes
                ncks -C -v mask ${mask_file_smb} -A tmp2_smb.nc
                ncatted -a _FillValue,${smb_org_var_name},o,f,-1.e20 tmp2_smb.nc
                ncap2 -O -s "where(mask==0) ${smb_org_var_name}=${smb_org_var_name}@_FillValue;" tmp2_smb.nc tmp_smb.nc
                ncks -x -v mask tmp_smb.nc -O tmp2_smb.nc

                # NOT NEEDED ANYMORE WITH FETISH-V171, SO COMMENTED OUT
                # Extrapolate SMB to nodes with 0% < land_pct < 50% (masked by COSMO, but has some land as per ice sheet model)
                # cdo setmisstonn -setgrid,${gridfile_cosmo_unmasked} tmp2_smb.nc tmp_smb.nc

                # cdo remap,${gridfile_cosmo_nonzeroland},${weights_interp_nn_smb} -setgrid,${gridfile_cosmo_unmasked} tmp_smb.nc tmp2_smb.nc

                # Convert from [kg m-2 s-1] to [kg m-2 yr-1]
                cdo mulc,31536000. tmp2_smb.nc ${outfile_atm2ice}
                ncatted -a units,${smb_org_var_name},o,c,"kg m-2 yr-1" ${outfile_atm2ice}
                ncrename -v ${smb_org_var_name},${smb_dest_var_name} ${outfile_atm2ice}

		rm -f tmp*_smb.nc
		mv -f ${outfile_raw_atm2ice} ${archive_dir}/isfcpl/
		
	    fi

            module purge
            load_modules_isfcpl

            infile_ocean2ice="${exp_name}_ocean2ice_raw_${date_beg_isfcpl}-${date_end_isfcpl}.nc"
            outfile_ocean2ice="${exp_name}_ocean2ice_${date_beg_isfcpl}-${date_end_isfcpl}.nc"

	    fwfisf_records=""
	    for((rs2=0; rs2<n_res_perisfcpl; rs2++)); do
                fmt_rs2=$(printf "%02d" ${rs2})
		curr_file="${out_dir}/isfcpl_tmp/${out_fwfisf_stencil}${fmt_rs2}.nc"
		if [ -f ${curr_file} ] ; then
		    fwfisf_records="${fwfisf_records} ${curr_file}"
		else
		    echo "${curr_file} is not in ${out_dir}/isfcpl_tmp. It is needed for NEMO - ice sheet coupling. Exiting"
		    exit 2
		fi
	    done
	    
            cdo -O copy ${fwfisf_records} ${infile_ocean2ice}
	    rm -f ${fwfisf_records}
	    
	    source build_pp_nemo_to_icesheet.sh > ./pp_nemo_to_${name_icesheet_model}.py
	    python pp_nemo_to_${name_icesheet_model}.py

	    mv -f ${infile_ocean2ice} ${archive_dir}/isfcpl/
            mv -f mesh_mask.nc ${archive_dir}/isfcpl/${exp_name}_mesh_mask_${date_beg_isfcpl}-${date_end_isfcpl}.nc
	    
            module purge
            module load ${icesheet_modules}
            if [ "${icesheet_library_path}" != "" ]; then
                export LD_LIBRARY_PATH="${icesheet_library_path}"
            fi
	    /bin/bash ./launch_icesheet.sh

	    mv -f ${outfile_ocean2ice} ${archive_dir}/isfcpl
	    if (( ice_atm_cpl == 1 )); then
		mv -f ${outfile_atm2ice} ${archive_dir}/isfcpl
	    fi

	    if [[ "${name_icesheet_model}" == "fETISh" ]]; then
		find . -type l -name "${exp_name}_${name_icesheet_model}Run_??????_toto.mat" -exec rm -f {} \;
		mv -f ${exp_name}_${name_icesheet_model}Run_??????_toto.mat ${archive_dir}/${name_icesheet_model}
		ln -sf ${archive_dir}/${name_icesheet_model}/${exp_name}_${name_icesheet_model}Run_${date_end_isfcpl::-2}_toto.mat
	    else
		echo "Ice-sheet models other than fETISh not implemented yet."
		exit 2
	    fi
	    
            curr_geom_file="${exp_name}_ice2ocean_${date_end_isfcpl}.nc"
	    new_bathy_file="${exp_name}_bathymetry_isfdraft_${date_end_isfcpl}.nc"
	    source build_pp_icesheet_to_nemo.sh > pp_${name_icesheet_model}_to_nemo.py

	    if [ "${init_bathy}" == 0 ]; then
		mv -f ${curr_bathy_file} ${archive_dir}/isfcpl
	    else
		init_bathy=0
		cp -f ${curr_bathy_file} ${archive_dir}/isfcpl
	    fi
	    	    
            module purge
            load_modules_isfcpl
	    python pp_${name_icesheet_model}_to_nemo.py

	    mv -f ${curr_geom_file} ${archive_dir}/isfcpl

	    curr_bathy_file=${new_bathy_file}
            ln -sf ${curr_bathy_file} ${run_dir}/bathy_meter.nc
            ln -sf ${curr_bathy_file} ${run_dir}/isf_draft_meter.nc

            printf "# Information for the ice sheet model\n" > ${start_dir}/isfcpl.track
            printf "run_name=${exp_name}\n" >> ${start_dir}/isfcpl.track
            printf "initial_date=${run_start_date_yyyymmdd}\n" >> ${start_dir}/isfcpl.track
            printf "n_month_isfcpl=${n_month_isfcpl}\n" >> ${start_dir}/isfcpl.track
            printf "n_isfcpl_window=${n_isfcpl_window}\n" >> ${start_dir}/isfcpl.track
            printf "date_beg_isfcpl=${nextleg_start_date_yyyymmdd}\n" >> ${start_dir}/isfcpl.track
            printf "date_end_isfcpl=YYYYMMDD\n" >> ${start_dir}/isfcpl.track
            printf "ice_atm_cpl=${ice_atm_cpl}\n\n" >> ${start_dir}/isfcpl.track

            printf "# Information for NEMO to keep up\n" >> ${start_dir}/isfcpl.track
            printf "init_bathy=${init_bathy}\n" >> ${start_dir}/isfcpl.track
            printf "curr_bathy_file=${curr_bathy_file}\n" >> ${start_dir}/isfcpl.track

        fi

        module purge
        module load ${module_list:?}
        export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+${LD_LIBRARY_PATH}:}"${extralibs_list}"

    fi

    # -------------------------------------------------------------------------
    # *** Move output files to archive directory
    # -------------------------------------------------------------------------
    ### NEMO
    outdir="${archive_dir}/output/nemo/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}

    for v in grid_U grid_V grid_W grid_T icemod SBC scalar SBC_scalar diad_T \
             grid_T_2D grid_U_2D grid_V_2D grid_W_2D grid_T_3D grid_U_3D grid_V_3D grid_W_3D \
             grid_1point grid_T_3D_ncatice vert_sum \
             grid_ptr_W_3basin_3D grid_ptr_T_3basin_2D grid_ptr_T_2D \
             zoom_700_sum zoom_300_sum zoom_2000_sum
    do
        for f in ${exp_name}_*_????????_????????_*${v}*.nc
        do
            test -f $f && mv $f $outdir/
        done
    done

    ### CLM
    cat YU* > OUTPUT_${YDATE_NOW}
    outdir="${archive_dir}/output/clm/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}

    mv $PROJECT.*.h*nc $outdir/

    ### COSMO
    outdir="${archive_dir}/output/cosmo/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}

    mv -f ../cclm_out_tmp/out0* $outdir/        

    # -------------------------------------------------------------------------
    # *** Move restart files to archive directory
    # -------------------------------------------------------------------------
    ### NEMO
    outdir="${archive_dir}/restart/nemo/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}

    ns=$(printf %08d $(( leg_end_sec / nem_time_step_sec )))
    for f in oce ice
    do
        mv ${exp_name}_${ns}_restart_${f}_????.nc ${outdir}
    done

    ### CLM
    outdir="${archive_dir}/restart/clm/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}
    RESTART_DATE_TMP=${leg_end_date_cos::-2}
    RESTART_DATE=${RESTART_DATE_TMP:0:4}'-'${RESTART_DATE_TMP:4:2}'-'${RESTART_DATE_TMP:6:2}-'00000'
    mv -f ${PROJECT}*.r*.${RESTART_DATE}* $outdir

    ### COSMO
    outdir="${archive_dir}/restart/cosmo/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}
    mv -f ../cclm_out_tmp/restarts/lrfd${leg_end_date_cos}o $outdir/

    ### OASIS
    outdir="${archive_dir}/restart/oasis/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}
    cp -f $IC_cosmo ${outdir}
    cp -f $IC_clm ${outdir}
    cp -f $IC_nemo ${outdir}
    cp -f $IC_cclm ${outdir}

    # -------------------------------------------------------------------------
    # *** Move log files to archive directory
    # -------------------------------------------------------------------------
    outdir="${archive_dir}/log/$(printf %03d $((leg_number)))"
    mkdir -p ${outdir}

    for f in \
        *.log.* OUTPUT_???????? ocean.output \
        time.step solver.stat \
        nout.000000 debug*.?? A*_??.nc O*_??.nc
    do
        test -f ${f} && mv ${f} ${outdir}
    done

    # ---------------------------------------------------------------------
    # *** Remove all leftover output files from previous legs
    # ---------------------------------------------------------------------
    ### clm files
    rm -f ${PROJECT}*

    ### cosmo files
    rm -f ${out_dir}/cclm_out_tmp/out0?/*
    rm -f ${out_dir}/cclm_out_tmp/restarts/*

    ### NEMO files
    rm -f ${exp_name}_??_????????_????????_{grid_U,grid_V,grid_W,grid_T,icemod,SBC,scalar,SBC_scalar}.nc
    rm -f ${exp_name}_*restart_*nc
    ns=$(printf %08d $(( leg_start_sec / nem_time_step_sec )))
    for (( n=0 ; n<nem_numproc ; n++ ))
    do
        np=$(printf %04d ${n})
        rm -f restart_oce_${np}.nc
        rm -f restart_ice_${np}.nc
    done

    # -------------------------------------------------------------------------
    # *** Write the restart control file
    # -------------------------------------------------------------------------
    shopt -u nullglob

    tr=$(date -d "0 -$time_begin sec + $time_end sec" +%T)
    current_date=$(date +'%F %T')
    {
      echo "#"
      echo "# Finished leg at ${current_date} after ${tr} (hh:mm:ss)" 
      echo "leg_number=${leg_number}"
      echo "leg_start_date=\"${leg_start_date}\""
      echo "leg_end_date=\"${leg_end_date}\""
    } | tee -a "${info_file}"

    # Need to reset force_run_from_scratch in order to avoid destroying the next leg
    force_run_from_scratch=false
    special_restart=false

done # Loop over legs

cd ${start_dir}

[[ $@ == *noresubmit* ]] && exit 0
if (( leg_end_epoch < run_end_epoch )) ; then
    echo "Leg end earlier than end of simulation."
    echo "Submitting another job."
    if [[ "$@" == *"local"* ]] ; then
        exec "$0" "$@"
    elif [ "qsub" == *sbatch* ] ; then
        sbatch $0 $@ | awk '{print $4}' >> ${run_dir}/.coral_jobs
    elif [[ $USER == "vsc"* ]]; then
        if [ -f ${PBS_O_WORKDIR-$PWD}/tmp/credits_file ]; then
            credits=`cat ${PBS_O_WORKDIR-$PWD}/tmp/credits_file`
        else
            credits_tmp=`grep "account string:" ${PBS_O_WORKDIR-$PWD}/*.o* -R | sed 's/^.*: //'`
            credits=`echo $credits_tmp | awk '{ print $1 }'`

            cd ${start_dir}
            mkdir -p tmp
            echo ${credits} > tmp/credits_file
        fi
        if [ -f ${PBS_O_WORKDIR-$PWD}/tmp/script_name ]; then
            script_name=`cat ${PBS_O_WORKDIR-$PWD}/tmp/script_name`
        else
            script_name="run.sh"
        fi
        ssh login2 "cd $start_dir; qsub -A $credits $script_name | tee -a coral_jobs"
        sleep 2
        jobid=`cat coral_jobs`
        rm -f coral_jobs
        jobid=${jobid%%.*}
        echo "${jobid}" >> "${run_dir}"/.coral_jobs ;
    else
        "qsub" -v PBS_OPTIONS="$@" "$0" | tee -a coral_jobs
        sleep 2
        jobid=`cat coral_jobs`
        rm -f coral_jobs
        jobid=${jobid%.*}
        echo "${jobid}" >> "${run_dir}"/.coral_jobs
    fi
else
    if [[ $USER == "vsc"* ]]; then
        rm -rf ${start_dir}/tmp
    fi

    rm -rf ${run_dir}/../cclm_out_tmp
    echo "Nothing left to do. Exiting." 

cat << "EOF" 
   _____ _                 _       _   _                                         _      _           _ 
  / ____(_)               | |     | | (_)                                       | |    | |         | |
 | (___  _ _ __ ___  _   _| | __ _| |_ _  ___  _ __     ___ ___  _ __ ___  _ __ | | ___| |_ ___  __| |
  \___ \| | '_ ` _ \| | | | |/ _` | __| |/ _ \| '_ \   / __/ _ \| '_ ` _ \| '_ \| |/ _ \ __/ _ \/ _` |
  ____) | | | | | | | |_| | | (_| | |_| | (_) | | | | | (_| (_) | | | | | | |_) | |  __/ ||  __/ (_| |
 |_____/|_|_| |_| |_|\__,_|_|\__,_|\__|_|\___/|_| |_|  \___\___/|_| |_| |_| .__/|_|\___|\__\___|\__,_|
                                                                          | |                         
                                                                          |_|                         
EOF

fi

exit 0
