set -u

cat << EOF

import numpy as np
from netCDF4 import Dataset
import sys

mesh_file = "./mesh_mask.nc"
input_file = "./${infile_ocean2ice}"
output_file = "./${outfile_ocean2ice}"

out_nc = Dataset(output_file, mode='w')

tmpdat = Dataset(mesh_file, mode='r')

lat = tmpdat.variables['gphit'][0,:,:]
lon = tmpdat.variables['glamt'][0,:,:]
bathy = tmpdat.variables['bathy'][0,:,:]
draft = tmpdat.variables['isfdraft'][0,:,:]
mask = tmpdat.variables['tmask'][0,:,:,:] == 0
maskutil = tmpdat.variables['tmaskutil'][0,:,:] == 0
depth = tmpdat.variables['gdept_0'][0,:,:,:]

tmpdat.close()

tmpdat = Dataset(input_file, mode='r')


nt = np.size(tmpdat.variables['time_counter'][:])
ny, nx = np.shape(lat)

out_nc.createDimension('y', ny)
out_nc.createDimension('x', nx)
out_nc.createDimension('bnds', 2)
out_nc.createDimension('time_counter', 0)

copy_vars = [ 'time_counter', 'time_counter_bnds', \
              'nav_lat', 'nav_lon']
    
for name, variable in tmpdat.variables.items():
    if name in copy_vars:
        x = out_nc.createVariable(name, variable.datatype, variable.dimensions)
        out_nc[name].setncatts(tmpdat[name].__dict__)
        out_nc[name][:] = tmpdat[name][:]


tmp = out_nc.createVariable('bathymetry', datatype='f4', dimensions=['y', 'x'])
tmp[:] = bathy
tmp.standard_name = "bathymetry"
tmp.positive = "down"
tmp.coordinates = "nav_lat nav_lon"

tmp = out_nc.createVariable('isfdraft', datatype='f4', dimensions=['y', 'x'])
tmp[:] = draft
tmp.standard_name = "ice_shelf_draft"
tmp.positive = "down"
tmp.coordinates = "nav_lat nav_lon"

        

tmp = out_nc.createVariable('fwfisf', datatype='f4', dimensions=['time_counter', 'y', 'x'])
dict_tmp = tmpdat.variables['${name_var_fwfisf}'].__dict__
tmp.setncatts(dict_tmp)
fillv = dict_tmp['_FillValue']

tmp[:] = np.where( maskutil, fillv, tmpdat.variables['${name_var_fwfisf}'][:])

out_nc.close()

EOF
