# NEMO_DEV
NEMO reference version (forced mode) to be used for future developments. Includes melt ponds. Created on 16/04/2018. 

Fork of : [NEMO release 3.6](http://forge.ipsl.jussieu.fr/nemo/svn/NEMO/releases/release-3.6/NEMOGCM/)
+ Branch Jonathan

## Install
### On a ELIC machine 
Clone this repo in `~/modeles/nemo/` 

```bash
mkdir -p ~/modeles/nemo
cd ~/modeles/nemo

git clone ssh://git@www.climate.be:3022/TECLIM/NEMO_DEV.git
```

> ... or git clone ssh://regit/TECLIM/NEMO_DEV.git

Go to the `EXTERNAL` directory, then:

```bash
cd NEMO_DEV/EXTERNAL

git clone ssh://git@www.climate.be:3022/TECLIM/Coral.git coral

git clone ssh://git@www.climate.be:3022/TECLIM/xios-1.0.git
```
### On a remote machine (Zenobe, Genius, ...) 
Add in your `~/.ssh/config` file:
```bash
Host gogs*
User git
Hostname www.climate.be
Port 3022
Proxycommand ssh -xaqW %h:%p gwcism
```

Clone the repo in `~/modeles/nemo/` 

```bash
mkdir -p ~/modeles/nemo
cd ~/modeles/nemo

git clone ssh://gogs/TECLIM/NEMO_DEV.git
```

Go to the `EXTERNAL` directory, then:

```bash
cd NEMO_DEV/EXTERNAL

git clone ssh://gogs/TECLIM/Coral.git coral 

git clone ssh://gogs/TECLIM/xios-1.0.git 
```

## Compile

### Zenobe

First, got to frontal1 and load your configuration:
```
ssh frontal1

module purge
module load compiler/intel/composerxe/2015.5.223 
module load intelmpi/5.0.3.049/64 
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+${LD_LIBRARY_PATH}:}"/projects/acad/limhr/opt/netcdf/4.4.0_intel_2015.5_impi_5.0.3/lib:/projects/acad/limhr/opt/udunits2/lib"
export PATH=${PATH:+${PATH}:}"/projects/acad/limhr/opt/ncview/bin"
source $HOME/modeles/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh
```

Compile XIOS:

```bash
cd ~/modeles/nemo/NEMO_DEV/EXTERNAL/xios-1.0

./make_xios --arch zenobe --netcdf_lib netcdf4_par -j6
```

Check the paths for the arch Nemo file in `~/modeles/nemo/NEMO_DEV/ARCH` and compile it:

```bash
cd ../../ARCH
cat arch-zenobe_prod.fcm

cd ../CONFIG
./makenemo -n ORCA1_LIM3_REF -m zenobe_prod -j6
```

Leave frontal1:
```
exit
```

### Lemaitre3

Compile XIOS:

```bash
cd ~/modeles/nemo/NEMO_DEV/EXTERNAL/xios-1.0

module purge
module load 2016a netCDF-Fortran/4.4.4-intel-2016a

./make_xios --arch intel_lemaitre3 --netcdf_lib netcdf4_par -j6
```

Check the paths for the arch Nemo file in `~/modeles/nemo/NEMO_DEV/ARCH` and compile it:

```bash
cd ../../ARCH
cat arch-lemaitre3.fcm

cd ../CONFIG
./makenemo -n ORCA1_LIM3_REF -m lemaitre3 -j6
```

### Genius

Compile XIOS:

```bash
cd ~/modeles/nemo/NEMO_DEV/EXTERNAL/xios-1.0

module purge
module use /apps/leuven/skylake/2015a/modules/all/
module load netCDF-Fortran/4.4.0-intel-2015a

./make_xios --arch intel_genius --netcdf_lib netcdf4_par -j6
```
Check the paths for the arch Nemo file in `~/modeles/nemo/NEMO_DEV/ARCH` and compile it:

```bash
cd ../../ARCH
cat arch-genius.fcm

cd ../CONFIG
./makenemo -n ORCA1_LIM3_REF -m genius -j6
```
## Run

### Zenobe

```bash
cd ORCA1_LIM3_REF
source $HOME/modeles/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh

mkdir CORAL00 ; cd CORAL00
coral init zenobe_xios-1.0
coral build

coral submit
```

### Lemaitre3

```bash
cd ORCA1_LIM3_REF
source $HOME/modeles/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh

mkdir CORAL00 ; cd CORAL00
coral init lemaitre3
coral build

coral submit
```
### Genius

```bash
cd ORCA1_LIM3_REF
source $HOME/modeles/nemo/NEMO_DEV/EXTERNAL/coral/setup-coral.sh

mkdir CORAL00 ; cd CORAL00
coral init genius
coral build

qsub -A default_project run.sh
```

## Benchmarks

### Zenobe 

* Compiler: intel-15.0.5.223
* Flags: -O2 -fp-model precise -xHost -r8

| XIOS |  NEMO  | WTIME per YEAR | #CORES | MEM/CPU |
| ---- | ------ | -------------- | ------ | ------- |
|   4  |   260  |   **47 min**   |   264  |   3072  |
|   4  |   520  |      |   524  |   3072  |
|   8  |   760  |      |   768  |   3584  |
|  12  |  1040  |      |  1052  |   3072  |


### Lemaitre3 

> Blocking Switch = 2 x 576 cores & 1 x 768 cores

* Compiler: intel-2016a
* Flags: -O3 -i4 -r8 -no-prec-div

| XIOS |  NEMO  | WTIME per YEAR | #CORES | MEM/CPU |
| ---- | ------ | -------------- | ------ | ------- |
|   2  |    22  |  **110 min**   |    24  |   3072  |
|   2  |    46  |   **69 min**   |    48  |   3072  |
|   2  |   140  |   **43 min**   |   142  |   3072  |
|   2  |   260  |   **41 min**   |   262  |   3072  |
|   4  |   260  |   **39 min**   |   264  |   3072  |
|   4  |   520  |   **36 min**   |   524  |   3072  |
|   8  |   520  |   **36 min**   |   528  |   4096  |
|   8  |   520  |   **37 min**   |   528  |   3072  |
|   8  |   760  |  **37min10**   |   768  |   3584  |
|  12  |  1040  |   **35 min**   |  1052  |   3072  |
|  16  |  1040  |   **35 min**   |  1056  |   3072  |
|  16  |  1856  |   **32 min**   |  1872  |   3072  |
