import numpy as np
from netCDF4 import Dataset
import os
import sys

# geom_in_stencil: set of restart files (2*n_proc files) with the subdomain decomposition you want to get in output
# rebuilt_rs_stencil: stencil for two restart files (oce and ice, rebuilt aka full domain), that you want to extract data from
# n_newdisp: number of output proc

geom_in_stencil = "/afast/pelletie/nemo_rs/so025_fET-v171/dest_geom/so025_rs_00000012_restart_"
rebuilt_rs_stencil = "/afast/pelletie/nemo_rs/so025_fET-v171/org_data/so025_fET-v171_00175296_restart_"

output_stencil = "/afast/pelletie/nemo_rs/so025_fET-v171/remapped_data/so025_fET-v171_00175296_remapped-212_restart_"

n_newdisp = 212
nz = 75

media = ['ice', 'oce']

for medium in media:

    big_rs = Dataset(rebuilt_rs_stencil+medium+'.nc', mode='r')
        
    for m in range(0,n_newdisp):

        geom_nc = Dataset(geom_in_stencil+medium+'_%04d'%m+'.nc', mode='r')

        nx_nd, ny_nd = geom_nc.DOMAIN_size_local
        sx_beg, sy_beg = geom_nc.DOMAIN_position_first - 1
        sx_end, sy_end = geom_nc.DOMAIN_position_last
        nx_nd, ny_nd = geom_nc.DOMAIN_size_local

        if( sx_end - sx_beg != nx_nd or sy_end - sy_beg != ny_nd):
            print('error nx, beg end')
            sys.exit()
        
        fmt = "%04d"%m
        outfile = output_stencil+medium+'_'+fmt+'.nc'

        out_nc = Dataset(outfile, mode='w')

        out_nc.createDimension('x', nx_nd)
        out_nc.createDimension('y', ny_nd)
        out_nc.createDimension('z', nz)
        out_nc.createDimension('t', 0)

        print(sx_beg,sx_end,sy_beg,sy_end)
        for name, variable in big_rs.variables.items():
            varout = out_nc.createVariable(name, variable.datatype, variable.dimensions)
            len_tmp = len(variable.dimensions)
            
            if( 'x' in variable.dimensions):
                if(len_tmp == 2):
                    varout[:] = big_rs.variables[name][sy_beg:sy_end,sx_beg:sx_end]
                elif(len_tmp == 3):
                    varout[:] = big_rs.variables[name][:,sy_beg:sy_end,sx_beg:sx_end]
                elif(len_tmp == 4):
                    varout[:] = big_rs.variables[name][:,:,sy_beg:sy_end,sx_beg:sx_end]
                    
            else:
                varin = big_rs.variables[name][:]
                if(len_tmp > 0 ):
                    varout[:] = varin[:]
                else:
                    varout[:] = varin
            # [name][:] = src[name][:]
            #     # copy variable attributes all at once via dictionary
            #     dst[name].setncatts(src[name].__dict__)



        out_nc.setncatts(geom_nc.__dict__)
        # out_nc.DOMAIN_number_total = 
        # out_nc.DOMAIN_number = 
        # out_nc.DOMAIN_dimensions_ids = 
        # out_nc.DOMAIN_size_global = 
        # out_nc.DOMAIN_size_local = 
        # out_nc.DOMAIN_position_first = 
        # out_nc.DOMAIN_position_last = 
        # out_nc.DOMAIN_halo_size_start = 
        # out_nc.DOMAIN_halo_size_end = 
        # out_nc.DOMAIN_type = 
                
        out_nc.close()
        print('generated '+outfile)

        geom_nc.close()
        
    big_rs.close()

